////////////////// GmailAgent class ////////////////////////////////
function GmailAgent( oEmailServiceAgentBase )
{
    this.m_oEmailServiceAgentBase = oEmailServiceAgentBase;
    this.m_sLastModified = null;
    this.m_sReadMailsLink = null;
}

GmailAgent.prototype.connectToServiceHost = function()
{
    this.importUnreadMails();
}

GmailAgent.prototype.updateUnreadMailsCount = function()
{
    this.importUnreadMails(); 
}

GmailAgent.prototype.revokeCredentials = function()
{	
	this.m_oEmailServiceAgentBase.queryProxy("getRevokeOAHeader", this, GmailAgent.prototype.queryTokenRevokedHeaderHandler);
}

GmailAgent.prototype.queryTokenRevokedHeaderHandler = function(oAjax)
{
	try
	{
		if (oAjax.readyState != 4)
			return;

		if (oAjax.status != 200)
			throw new MNException(g_sWebServiceId + '_tokenRevoke', oAjax.status, this.m_oEmailServiceAgentBase.m_oEmailService);

		var oResponse = parseIcqProtocol(oAjax.responseText);
		if (oResponse.statusCode != 200)
			throw new MNException(g_sWebServiceId + '_tokenRevoke', oResponse.statusCode, this.m_oEmailServiceAgentBase.m_oEmailService);

		// the function returns an error if one of passed parameters in invalid
		g_oIcqConnector.Owner.EMailNotifierService.SendRequest(this, 'https://www.google.com/accounts/AuthSubRevokeToken', oResponse.data.authorizationHeader, 'queryTokenRevokedHandler');

		//Performance
		delete oResponse; 
		oResponse = null;
	}
	catch (e) { handleException(e); }
}

GmailAgent.prototype.queryTokenRevokedHandler = function(ecode, x)
{
	// ecode  error code. 0 is OK
	// revoke completed - nothing to do here
	if (ecode != 0)
	{
		var oService = this.m_oEmailServiceAgentBase.m_oEmailService;
		handleException(new MNException(oService.m_sId, ecode, oService));
	}
}

GmailAgent.prototype.importUnreadMails = function() 
{	
	var oCredentials = this.m_oEmailServiceAgentBase.m_oEmailService.m_oCredentials;
	if (!oCredentials)
		return;	
	this.m_oEmailServiceAgentBase.queryProxy("getOAHeader", this, GmailAgent.prototype.queryFeedsHeaderHandler);
}

GmailAgent.prototype.queryFeedsHandler = function(ecode, x)
{
	try
	{
		// ecode  error code. 0 is OK
		// x  string with data. In case of error it will be empty

		if (ecode != 0)
		{
			var oService = this.m_oEmailServiceAgentBase.m_oEmailService;
			throw new MNException(oService.m_sId, ecode, oService);
		}

		this.m_oEmailServiceAgentBase.m_oEmailService.onRequestState(ecode);
		var data = String(x);
		this.parseFeeds(data);
	}
	catch (e) { handleException(e); }

}

GmailAgent.prototype.queryFeedsHeaderHandler = function(oAjax) 
{
	try
	{
		if (oAjax.readyState != 4)
			return;
				
		if (oAjax.status != 200)
			throw new MNException(g_sWebServiceId + '_feedsHeader', oAjax.status, this.m_oEmailServiceAgentBase.m_oEmailService);

		var oResponse = parseIcqProtocol(oAjax.responseText);
		if (oResponse.statusCode != 200)
			throw new MNException(g_sWebServiceId + '_feedsHeader', oResponse.statusCode, this.m_oEmailServiceAgentBase.m_oEmailService);

		// the function returns an error if one of passed parameters in invalid
		g_oIcqConnector.Owner.EMailNotifierService.SendRequest(this, 'https://mail.google.com/mail/feed/atom/', oResponse.data.authorizationHeader, 'queryFeedsHandler');

		//Performance
		delete oResponse;
		oResponse = null;
	}
	catch (e) { handleException(e); }
}

GmailAgent.prototype.parseFeeds = function(sFeedsXml)
{
	try
	{
		var oService = this.m_oEmailServiceAgentBase.m_oEmailService;

		var oGmailFeedsXml = createClientCompatibleXmlDom();
		// MSXML4 and later xpath should add namespaces to query - I prefer to ignore namespaces
		sFeedsXml = sFeedsXml.replace('xmlns="http://purl.org/atom/ns#"', "");
		oGmailFeedsXml.loadXML(sFeedsXml);

		if (oGmailFeedsXml.parseError.errorCode != 0)
			throw new MNException(this.m_oEmailServiceAgentBase.m_oEmailService.m_sId, MEExeptions.kInvalidFeedsFormat, this.m_oEmailServiceAgentBase.m_oEmailService);

		var sModified = getXmlNodeValue(oGmailFeedsXml, "feed/modified", "");
		if (sModified == this.m_sLastModified)
		{
			//Performance
			oGmailFeedsXml = null;
			return; // no changes we done.
		}

		// set last Modified date
		this.m_sLastModified = sModified;

		var aEntries = oGmailFeedsXml.selectNodes("feed/entry");
		var nLength = aEntries.length;
		var aNewEmailsList = new Array(nLength);
		for (i = 0; i < nLength; i++)
		{
			var oNode = aEntries[i];
			var sSubject = getXmlNodeValue(oNode, "title");
			var sPreview = getXmlNodeValue(oNode, "summary");
			var sId = getXmlNodeValue(oNode, "id");
			var sSenderName = getXmlNodeValue(oNode, "author/name");
			var sSenderMailAddress = getXmlNodeValue(oNode, "author/email");
			var bHasAttachment = false;
			var dtReceiveDate = convertDateObjRcfToJs(getXmlNodeValue(oNode, "issued"));

			var oEmail = new Email(sSenderMailAddress, sSubject, dtReceiveDate, sSenderName, bHasAttachment, sId, sPreview);

			var oLinkNode = oNode.selectSingleNode("link");
			oEmail.m_sLink = oLinkNode ? getXmlAttributeValue(oLinkNode.attributes, "href") : "";

			aNewEmailsList[i] = oEmail;
		}

		// set m_sReadMailsLink if needed
		if (!this.m_sReadMailsLink)
		{
			var oLinkNode = oGmailFeedsXml.selectSingleNode("feed/link");
			this.m_sReadMailsLink = getXmlAttributeValue(oLinkNode.attributes, "href", "http://mail.google.com/mail");
		}

		var nNewFullCount = getXmlNodeValue(oGmailFeedsXml, "feed/fullcount", 0);

		//Performance
		oGmailFeedsXml = null;

		// save old state for calculation        
		var aOldEmailList = oService.m_aUnreadEmails;
		var nOldFullCount = oService.m_nUnreadEmailsCount;

		// set new state
		oService.m_aUnreadEmails = aNewEmailsList;
		oService.m_nUnreadEmailsCount = nNewFullCount;

		// calc new state        
		var nNewMailsCount = this.m_oEmailServiceAgentBase.calcDiffState(nOldFullCount, aOldEmailList, nNewFullCount, aNewEmailsList);

		oService.onInboxModified(nNewMailsCount);

		g_oBalloonsManager.resetBallonState(oService, nNewMailsCount, nOldFullCount, nNewFullCount);
	}
	catch (e) { handleException(e); }
}

GmailAgent.prototype.readMail = function( sMailId )
{   
    var oEmail =  this.m_oEmailServiceAgentBase.m_oEmailService.m_aUnreadEmails.find( makeEmailIdMatcher(sMailId) );
    if (oEmail && oEmail.m_sLink) 
        g_oIcqConnector.OpenUrl(oEmail.m_sLink, false);    
}

GmailAgent.prototype.openCalendar = function()
{
    g_oIcqConnector.OpenUrl("https://www.google.com/calendar/render?tab=mc");
}

GmailAgent.prototype.openAddressBook = function()
{
    g_oIcqConnector.OpenUrl("https://mail.google.com/mail/?shva=1#contacts");
}

GmailAgent.prototype.composeMail = function()
{
    g_oIcqConnector.OpenUrl("https://mail.google.com/mail/?shva=1#compose");
}

GmailAgent.prototype.openInbox = function()
{   
    if (this.m_sReadMailsLink) 
        g_oIcqConnector.OpenUrl(this.m_sReadMailsLink, false);
}