// --------------------------- Enums  -----------------------------------

MEServiceState = 
{ 
    kOffline    : 1, 
    kOnline     : 2
}  
                 
MECredentialsState = 
{ 
    kNotSet     : 1,        
    kNotValid   : 2,
    kSet        : 3,
    kExpired    : 4, 
    kValid      : 5
}

MEExeptions =
{
	kServiceGenericError	: 750,
	kInvalidJsonFormat		: 760,
	kFailToLoadConfigFile	: 770,
	kInvalidConfigFile		: 780,
	kLoadUnsupportedAgent	: 790,
	kInvalidFeedsFormat		: 800
}

// --------------------------- constants  -----------------------------------
var STAT_CATEGORY_ID	= "CUsage";
var LINKS_STAT_ID		= "ENU";
var SERVICES_STAT_ID	= "ENE";
var ERROR_STAT_ID		= "ENR";

// --------------------------- globals  -----------------------------------

var g_nDefaultPollingInterval           = 5 * 60; // 5 minutes in seconds
var g_sCredentialsApiUrl				= 'https://japi.icq.com/ICQ/CredentialManager';                    
var g_trueRegExp                        = /^(true|yes|1)$/i;
var g_oLastException					= null; // Xcore js engine not support custom exception :(
var g_sWebServiceId						= "webSrc";


var g_nothing; // use for url sing method

// --------------------------------- string extension  -----------------------------------
String.prototype.trim = function() { return this.replace(/^\s*/, "").replace(/\s*$/, ""); }


// --------------------------- Mail Notifier Exception -----------------------------------
function MNException(sErrorCategory, nErrorCode, oService)
{	
	this.m_nErrorCode		= nErrorCode;
	this.m_sErrorCategory	= sErrorCategory;
	this.m_oService			= oService;

	// THIS IS PATCH
	// Xcore js engine not support custom exception :(
	g_oLastException = this;
}

MNException.prototype.handleError = function()
{
	if (this.m_oService)
		this.m_oService.onRequestState(this.m_nErrorCode);

	updateXtraFaultSrp(this.m_sErrorCategory, this.m_nErrorCode);
}

function handleException( oException )
{	
	if (oException && oException.handleError)
	{
		oException.handleError();
	}
	// THIS IS PATCH
	// Xcore js engine not support custom exception :(
	else if(g_oLastException && g_oLastException.handleError)
		g_oLastException.handleError();

	//Performance
	delete g_oLastException;

	g_oLastException = null;
}

function updateXtraFaultSrp(sErrorCategory, nErrorCode)
{	
	var oCollection = g_oIcqConnector.Owner.OwnerStatistic.Collection;
	if (!oCollection)
		return;

	var sErrorId = sErrorCategory + ":" + nErrorCode;
	oCollection.IncrementMapSecondDItem(STAT_CATEGORY_ID, ERROR_STAT_ID, sErrorId, "");	
}

function updateXtraProductSrp(sStatCategoryID, sStatCodeId, sStatValue)
{	
	var oCollection = g_oIcqConnector.Owner.OwnerStatistic.Collection;
	if (!oCollection)
		return;

	oCollection.IncrementMapSecondDItem(sStatCategoryID, sStatCodeId, sStatValue, "");	
}

// --------------------------- Utils functions  -----------------------------------

function secureJsonParse( sJson )
{
	if (!this.JSON)
		if (!importScript(g_sServiceFolderPath + "json_parser.js"))
		return null;
	
	try { return JSON.secureParse(sJson); } catch (e) { }

	return null;
}

function parseJson( sJson )
{
	if (!this.JSON)
		if (!importScript(g_sServiceFolderPath + "json_parser.js"))
			return null;

	try { return JSON.parse(sJson); } catch (e) { }
	
	return null;
}

function convert2Json( oObject )
{
	if (!this.JSON)
		if (!importScript(g_sServiceFolderPath + "json_parser.js"))
		return "";

	return JSON.stringify(oObject);
}

function parseIcqProtocol( sJson )
{
	var oResponse = parseJson( sJson );
	oResponse = oResponse ? oResponse.response : { statusCode: MEExeptions.kInvalidJsonFormat };

	return oResponse;
}

function makeAjaxCallback(oAjax, thisArg, funcCallback, nRequestId)
{
	
	g_aAjaxsCalls.push(oAjax);

	return function()
	{
		funcCallback.call(thisArg, oAjax, nRequestId);
		if (oAjax.readyState === 4)
		{
			// remove from live list
			g_aAjaxsCalls.find(oAjax, true/**remove*/);

			oAjax.abort();
			oAjax.onreadystatechange = null;
			oAjax = null;
		}
	}
}

function makeEmailIdMatcher( sMailId )
{
    return function( oEmail ){ return oEmail.getId() == sMailId; }
}

function makeServiceIdMatcher( sServiceId )
{
    return function( oService ){ return oService.getId() == sServiceId; }
}

function createToaster(sServiceId, sTitleId)
{
	var sIconUrl	= g_oMainService.m_oUIStub.getService(sServiceId).getSmallIcon();
	var sTitle		= g_oIcqConnector.GetString(sTitleId);  

	var oToaster = g_oMainService.m_oComponent.gadget.scene.createBox("email_not_toaster_content");

	oToaster.init(g_oIcqConnector, sTitle, sIconUrl);

	return oToaster;
}

function convertValueType( sValue, sType )
{
    switch( sType ) 
    {
	case 'boolean':
		sValue = sValue.trim();
		return g_trueRegExp.test(sValue);
    case 'number':
    	sValue = sValue.trim();
    	return Number(sValue);    
    }    
    
    return sValue;
}

function createClientCompatibleXmlDom()
{
	var oXmlDom = null;
	try
	{
		oXmlDom = g_oIcqConnector.GetObject("Msxml.DOMDocument");
		if (!oXmlDom) throw false;	 
	}
	catch (e)
	{
		oXmlDom =  new ActiveXObject("Msxml.DOMDocument");
	}	
	
	return oXmlDom;
}

function getXmlNodeValue( oParentNode, sXPathQuery, defaultValue )
{
    var oNode = oParentNode.selectSingleNode(sXPathQuery);	
    if (!oNode) 
        return defaultValue;
     
	return convertValueType( oNode.text, typeof(defaultValue) ); // default value type is 'String' 
}

function getXmlAttributeValue( oAttributesMap, sAttributeName, defaultValue )
{
    var oAttribute = oAttributesMap.getNamedItem(sAttributeName);	

    if (!oAttribute) 
        return defaultValue;
        
    return convertValueType( oAttribute.text, typeof(defaultValue) ); // default value type is 'String'  
}

function serviceDisplayNameFromId( sServiceId )
{
    return g_oIcqConnector.GetString( sServiceId + ".DisplayName" );
}

function embedServiceDisplayName( sText, sServiceId )
{
    return sText.replace("#serviceId#", serviceDisplayNameFromId( sServiceId ));
}

function embedServiceId( sText, sServiceId )
{    
    return sText.replace("#serviceId#" , sServiceId );
}

// --------------------------------- Date extention  -----------------------------------
function convertDateObjRcfToJs(s_timestamp)
{
	var rgx_atomPattern = /^(\d{4})-(\d{2})-(\d{2})[T ](\d{2}):(\d{2}):(\d{2})((?:\.\d+)?)(Z(([+-])(\d{2}):(\d{2}))?)$/gi;
	var a_timeData = rgx_atomPattern.exec(s_timestamp);
	if(a_timeData == null) //if its not atom timestamp return the string.
		return s_timestamp;
	
	var year 	= new Number(a_timeData[1] ? a_timeData[1] : 0 );
	var month 	= new Number(a_timeData[2] ? a_timeData[2]-1 : 0 );
	var day 	= new Number(a_timeData[3] ? a_timeData[3] : 0 );
	var hours 	= new Number(a_timeData[4] ? a_timeData[4] : 0 );
	var minutes	= new Number(a_timeData[5] ? a_timeData[5] : 0 );
	var seconds	= new Number(a_timeData[6] ? a_timeData[6] : 0 );
	var milsecs	= new Number(a_timeData[7] ? a_timeData[7] : 0 );
	
	var timeZoneHour 	= new Number(a_timeData[11] ? a_timeData[11] : 0 ); 
	var timeZoneMinutes = new Number(a_timeData[12] ? a_timeData[12] : 0 ); 
	var timeZone 		= (a_timeData[10] ? a_timeData[10] : '');
	
	switch(timeZone)
	{
		case '+':
			minutes += ((timeZoneHour * 60) + timeZoneMinutes)
			break;
		case '-':
			minutes -= ((timeZoneHour * 60) + timeZoneMinutes)
			break;
		case '':
			var date 	= new Date();
			var timeZoneOffset = date.getTimezoneOffset()/60;
			minutes -= (timeZoneOffset  * 60);
			break;
	}
	
	return new Date(year,month,day,hours,minutes,seconds,milsecs);
}

// --------------------------------- array extention  -----------------------------------

Array.prototype.find = function(match, bRemove)
{
	var bIsFunction = typeof (match) === "function";
	var nLength = this.length;

	var ret = null;

	for (var i = 0; i < nLength; i++)
	{
		var item = this[i];

		if (bIsFunction)
		{
			if (match(item))
			{
				ret = item;
				break;
			}
		}
		else if (match === item)
		{			
			ret = item;
			break;	
		}
	}

	if ( bRemove && (i < nLength) )
		this.splice(i,1); // remove found item

	return ret;
}



// --------------------------- ToasterAction -----------------------------------
function ToasterAction( sEventId , sServiceId )
{	
	this.m_sEventId		= sEventId;
	this.m_sServiceId	= sServiceId;
}

ToasterAction.prototype.run = function()
{
	try
	{
		switch (this.m_sEventId)
		{
		    case "onNewServiceClicked":			
				g_oMainService.m_oUIStub.registerToService(this.m_sServiceId);
				break;
			case "onNewMailClicked":
				g_oMainService.m_oUIStub.getService(this.m_sServiceId).readMail(this.m_sMailId);
				break;
			case "onNewMailsClicked":				
				g_oIcqConnector.App.MainDlgController.SelectContentPanel("MeTab", true);
				(new Event("onTabSelection", this.m_sServiceId)).fire();
				break;
		}
	}
	catch (e) {}
}

// --------------------------- Event -----------------------------------
function Event( sEventTarget, sServiceId ) 
{
    this.m_sEventTarget = sEventTarget;
    
    if ( sServiceId ) 
        this.m_sServiceId = sServiceId;
}

Event.prototype.fire = function()
{
	var callback = g_oMainService.m_oUIStub.m_uiEventsCallback;

	if (!callback || typeof (callback) != 'function')
		return; // UI not ready for event - so we ignore event

	try { callback(this); } catch (e) { }	
}

// incoming events
function MainService.prototype.onPluginNotification(sNotification)
{   
    try
    {   		
        var oEvent = parseJson(sNotification);        
        switch(oEvent.sEventTarget)
        {
        case "onServiceCredentialsAvailable":       
            this.onServiceCredentialsAvailable(oEvent.sServiceId);
            break;
        }
    }
    catch(e){}   
}

function onCredentialsQueryResponse(oAjax)
{
    try
    {   
        if(oAjax.readyState != 4) 
             return;
            
        if (oAjax.status != 200)
        	throw new MNException("credApi_get", oAjax.status);

        var oResponse = parseIcqProtocol(oAjax.responseText);
        if (oResponse.statusCode != 200)
        	throw new MNException("credApi_get", oResponse.statusCode);
        	            
        var aServicesCredentials  = oResponse.data.credentials;
        if (aServicesCredentials) 
        {                
            var aServices   = g_oMainService.m_aEmailServices;      
            var nLength     = aServicesCredentials.length;
            for(var i = 0; i < nLength; i++)
            {
                var oServiceCredentialsRecord = aServicesCredentials[i]
                var oService = aServices.find( makeServiceIdMatcher( oServiceCredentialsRecord.serviceId ) );
                if ( !oService ) 
                    continue; // service not available on this configuration LSP/GEO
               
                oService._getImpl().reinitCredentials(oServiceCredentialsRecord);     
            }

            g_oMainService.initCoregService();	
         }
    }
    catch (e) { handleException(e); }

    g_oMainService.raiseNewServicesPromoToaster(); // now we know if user already registered to new service so we will not show it. 
    
    // Fire services ready event to UI
    if (g_oMainService.m_bReady == false)
    {
        g_oMainService.m_bReady = true;
        var oEvent = new Event("onServicesReady", "");
        oEvent.fire();
    }
 }

 MainService.prototype.cleanup = function()
 {
 	// clean UI proxy
 	this.m_oUIStub.cleanup();
 	delete this.m_oUIStub; 	
 	
 	// close all timers
 	nLength = this.m_aEmailServices.length;
 	for (var i = 0; i < nLength; i++)
 	{
 		var oService = this.m_aEmailServices[i]._getImpl();
 		if (oService.m_updateTimerId)
 			oService.m_updateTimerId.cancel(); 			
 	}

 	delete this.m_aEmailServices; 	
 }

MainService.prototype.InitServices = function()
{
	try
	{		
		g_nClientLspId = Number(g_oIcqConnector.GetIMClientData("LSP_ID"));
		
		// get all services info from client config file        
		var sConfigXmlPath = g_oIcqConnector.GetConfigurationFile("emailServicesInfo.xml");
		if (!sConfigXmlPath)
			throw new MNException("mainSrc", MEExeptions.kFailToLoadConfigFile);
		
		var oConfigXml = createClientCompatibleXmlDom();
		oConfigXml.load(sConfigXmlPath);
		if (oConfigXml.parseError.errorCode != 0)
			throw new MNException("mainSrc", MEExeptions.kInvalidConfigFile);

		// init default polling interval from config file   
		var oServicesRootNode = oConfigXml.selectSingleNode("services");
		var oServicesRootNodeAttributes = oServicesRootNode.attributes;
		g_nDefaultPollingInterval = getXmlAttributeValue(oServicesRootNodeAttributes, "defaultPollingInterval", g_nDefaultPollingInterval);

		var aServicesInfo = oConfigXml.selectNodes("services/service");
		var nLength = aServicesInfo.length;
		var aEmailServices = new Array(nLength);
		for (var i = 0; i < nLength; i++)
		{
			aEmailServices[i] = new EmailServiceStub(new EmailService(aServicesInfo[i].attributes));
		}

		this.m_aEmailServices = aEmailServices;

		//Performance
		oConfigXml = null;
	}
	catch (e) { handleException(e); }
}

MainService.prototype.initCoregService = function()
{
}

MainService.prototype.onServiceCredentialsAvailable = function (serviceId)
{
    // ignore service id - will be used for optimization on future 
    this.queryCredentials();
}

MainService.prototype.queryCredentials = function()
{
	var sCredentialsApiUrl = g_oIcqConnector.Owner.Url.SignUrl(g_sCredentialsApiUrl + '/get?f=json', g_nothing);
		
	var oAjax = new XMLHttpRequest();
	oAjax.open("GET", sCredentialsApiUrl, true);
	oAjax.send('');
	oAjax.onreadystatechange = makeAjaxCallback(oAjax, null, onCredentialsQueryResponse);
}

MainService.prototype.raiseServicePromoToaster = function(sServiceId)
{	
	var oToaster = createToaster(sServiceId, "promoToaster.NewServiceTitle");

	var sMsgBoddy = embedServiceDisplayName(g_oIcqConnector.GetString("promoToaster.MsgBoddy"), sServiceId);
	var oAction = new ToasterAction("onNewServiceClicked", sServiceId);
	var sLinkText = g_oIcqConnector.GetString("promoToaster.ConnectLink");

	oToaster.popNewEmailServiceToaster(sMsgBoddy, sLinkText, oAction);
}

MainService.prototype.raiseNewServicesPromoToaster = function()
{	
	var bSuppressedToasterPromoListChanged = false;
	var aEmailServices = this.m_aEmailServices;
	var aSuppressedToasterPromoList = g_oPersistPrefences.m_aSuppressedToasterPromoList;
	var nLength = aEmailServices.length;
	for (var i = 0; (i < nLength); i++)
	{
		var oService = aEmailServices[i];
		if (	oService.isNeedPromo() &&
				!oService.isRegisterd() &&
                !aSuppressedToasterPromoList.find(oService.getId()))
		{
			var sServiceId = oService.getId();
			this.raiseServicePromoToaster(sServiceId);

			aSuppressedToasterPromoList.push(sServiceId);
			bSuppressedToasterPromoListChanged = true;
		}
	}

	if (bSuppressedToasterPromoListChanged)
		g_oPersistPrefences.save();
}

////////////////// PersistPrefences class ////////////////////////////////
function PersistPrefences()
{
    this.m_aSuppressedBallobPromoList = [];
    this.m_aSuppressedToasterPromoList = [];
    this.m_sLastTabId = "";  
    
    this.load();
}

PersistPrefences.prototype.save = function()
{	
	g_oIcqConnector.SetPrefsValues(false, convert2Json(this));
}

PersistPrefences.prototype.load = function()
{
    var sObjectDescription = g_oIcqConnector.GetPrefsValues(false);
    
    if ( sObjectDescription == "" ) 
        return; // no prefs
    
    var oPerfs = parseJson( sObjectDescription );

    if (oPerfs) 
	{
		this.m_aSuppressedBallobPromoList	= oPerfs.m_aSuppressedBallobPromoList;
		this.m_aSuppressedToasterPromoList	= oPerfs.m_aSuppressedToasterPromoList;
		this.m_sLastTabId					= oPerfs.m_sLastTabId;      
	}    
}

////////////////// UIStub class ////////////////////////////////

UIStub.prototype.setEventsCallback =  function( callback )
{
    this.m_uiEventsCallback = callback;
}

UIStub.prototype.cleanup = function()
{
	g_oIcqConnector.RemoveSharedObject("emailNotifierService");
	this.m_uiEventsCallback = null;
}

UIStub.prototype.setLastTab = function(sServiceId)
{
	// need to reset service balloon
	g_oBalloonsManager.onSelectedTabChanged(sServiceId);

	if (g_oPersistPrefences.m_sLastTabId != sServiceId)
	{
		// need to reset selected service balloon value too
		g_oPersistPrefences.m_sLastTabId = sServiceId;
		g_oPersistPrefences.save();
	}
}

UIStub.prototype.getLastTab = function()
{
    return g_oPersistPrefences.m_sLastTabId;
}

UIStub.prototype.suppressNewServiceBallon =  function( )
{    
    var aEmailServices = g_oMainService.m_aEmailServices;
    var aSuppressedBallobPromoList = g_oPersistPrefences.m_aSuppressedBallobPromoList;
    var nLength = aEmailServices.length;  
    var bSuppressedBallobPromoListChanged = false;    
    for (var i = 0; (i < nLength) ; i++)
    {
        var oService = aEmailServices[i];        
        if (    oService.isNeedPromo() &&     
                !aSuppressedBallobPromoList.find(oService.getId()) ) 
                {
                    aSuppressedBallobPromoList.push(oService.getId());
                    bSuppressedBallobPromoListChanged = true;
                }                
    }

    if (bSuppressedBallobPromoListChanged) 
        g_oPersistPrefences.save();
}

UIStub.prototype.isNeedShowNewServiceBallon =  function( )
{   
    var aEmailServices = g_oMainService.m_aEmailServices;
    var aSuppressedBallobPromoList = g_oPersistPrefences.m_aSuppressedBallobPromoList;
    var nLength = aEmailServices.length;  
    for (var i = 0; (i < nLength) ; i++)
    {
        var oService = aEmailServices[i];        
        if (    oService.isNeedPromo() &&     
                !aSuppressedBallobPromoList.find(oService.getId()) ) 
                return true;
    }   
       
    return false;    
}

UIStub.prototype.removeService = function( sServiceId )
{   
    var oService = this.getService(sServiceId);
    if ( !oService )
        return; // abort - error 
    
    oService._getImpl().revokeCredentials();
}

UIStub.prototype.registerToService = function(sServiceId)
{
	// will back to UI xtra with ServiceUpdated notification

	var oInitData = 
	{
		emailServiceId		: sServiceId,
		serviceDisplayName	: serviceDisplayNameFromId(sServiceId)
	}

	g_oIcqConnector.OpenPlugin("", "email_notifier_login", convert2Json(oInitData));
}

UIStub.prototype.getService =  function( sServiceId )
{   
    return g_oMainService.m_aEmailServices.find( makeServiceIdMatcher( sServiceId ) );
}

UIStub.prototype.getServices =  function()
{
    return g_oMainService.m_aEmailServices;    
}

UIStub.prototype.registerdServicesCount =  function(  )
{
    var nCount = 0;
    
    // loop to count active services 
    var aEmailServices = g_oMainService.m_aEmailServices;
    var nLength = aEmailServices.length;  
    for (var i = 0; (i < nLength) ; i++)
        if ( aEmailServices[i].isRegisterd() ) 
            nCount++;
    
    return nCount;
}


////////////////// EmailServiceStub class ////////////////////////////////

function EmailServiceStub( oMailService )
{
    // private - only EmailServiceStub.prototype methods should use it 
    this._getImpl = function(){return oMailService;}
}

EmailServiceStub.prototype.getId = function()
{
    return this._getImpl().m_sId;
}

EmailServiceStub.prototype.getDisplayName = function()
{
	return serviceDisplayNameFromId( this.getId() );
}

EmailServiceStub.prototype.getSmallIcon = function()
{
    var ret = "#img." + this.getId() + ".small";
    return ret;
}

EmailServiceStub.prototype.getBigIcon = function()
{
    var ret = "#img." + this.getId() + ".big";
    return ret;
}

EmailServiceStub.prototype.isCapable = function( sCapability )
{
    return this._getImpl().m_aCapabilities.find(sCapability) != null;
}

EmailServiceStub.prototype.isNeedPromo = function()
{
    // if not set on init state we return false
    // if we got date from config we return true when m_dtPromoExpiration > current date
    return (this._getImpl().m_dtPromoExpiration instanceof Date) ? (this._getImpl().m_dtPromoExpiration > new Date()) : false;
}

EmailServiceStub.prototype.retryToConnect = function()
{
	this._getImpl().retryToConnect();   
}

EmailServiceStub.prototype.isRegisterd = function()
{
    return this.getCredentialsState() != MECredentialsState.kNotSet;
}

EmailServiceStub.prototype.unreadMailesCount = function()
{
    return this._getImpl().m_nUnreadEmailsCount;
}
  
EmailServiceStub.prototype.importUnreadMails = function()
{
    return this._getImpl().m_oAgent.importUnreadMails();
}

EmailServiceStub.prototype.getUnreadMails = function()
{  
    return this._getImpl().m_aUnreadEmails;
}

EmailServiceStub.prototype.readMail = function( sMailId )
{
    return this._getImpl().m_oAgent.readMail( sMailId );
}

EmailServiceStub.prototype.deleteMail = function( sMailId )
{
    return this._getImpl().m_oAgent.deleteMail( sMailId );
}

EmailServiceStub.prototype.markAsRead = function( sMailId )
{
    return this._getImpl().m_oAgent.markAsRead( sMailId );
}

EmailServiceStub.prototype.composeMail = function()
{
    return this._getImpl().m_oAgent.composeMail();
}

EmailServiceStub.prototype.openCalendar = function()
{
    return this._getImpl().m_oAgent.openCalendar();
}

EmailServiceStub.prototype.openAddressBook = function()
{
    return this._getImpl().m_oAgent.openAddressBook();
}

EmailServiceStub.prototype.openInbox = function()
{
    return this._getImpl().m_oAgent.openInbox();
}

EmailServiceStub.prototype.getStatus = function()
{
    return this._getImpl().m_eStatus;
}

EmailServiceStub.prototype.getCredentialsState = function()
{
    return this._getImpl().m_eCredentialsState;
   }

EmailServiceStub.prototype.needLspTab = function()
{
	return this._getImpl().m_bNeedLspTab;
}

EmailServiceStub.prototype.getBalloonCount = function()
{
	return this._getImpl().m_nUIBalloonCount;
}


////////////////// EmailService class ////////////////////////////////
// EmailService class holdes:
// its metadata - id, urls for activations read/compose etc mails 
// variant data - emails count, emails content, status(notRegisterd, online, offline)
// service protocol object - use to conmunicat with service host 

function EmailService(oServiceInfo)
{   
    // metadata as read only
    this.m_sId                  = getXmlAttributeValue(oServiceInfo, "id", '');
    this.m_aCapabilities        = this.initCapabilities(oServiceInfo); 
    this.m_nPollingInterval     = this.initPollingInterval(oServiceInfo);
    this.m_dtPromoExpiration	= this.initPromoExpiration(oServiceInfo);
    this.m_bNeedLspTab			= this.initNeedLspTab(oServiceInfo);

    // variant data
    this.m_nUIBalloonCount		= 0;  // show only new (+ sync)	    
    this.m_nUnreadEmailsCount   = 0;    
    this.m_aUnreadEmails        = []; // array of unread miles as Email objects    
    this.m_eStatus              = MEServiceState.kOffline; // one of MEServiceState {"online", "offline"}
    this.m_eCredentialsState	= MECredentialsState.kNotSet; // one of MECredentialsState {"notSet", "notValid" "valid", "expired"}
    this.m_oCredentials			= null;    

    // privet members	    
    this.m_oAgent = new EmailServiceAgentBase(this);
}

function EmailService.prototype.compareCredentials(oServiceCredentials)
{
    for( i in oServiceCredentials )
    {
        if ( !this.m_oCredentials.hasOwnProperty(i) )
            return false;
            
        if (this.m_oCredentials[i] != oServiceCredentials[i])
            return false;
    }

    return true;
}


EmailService.prototype.initCapabilities = function(oServiceInfo)
{	
	var aCapabilities = getXmlAttributeValue(oServiceInfo, "capabilities", '').split(',');
	var nLength = aCapabilities.length;
	for (var i = 0; i < nLength; i++)
		aCapabilities[i] = aCapabilities[i].trim();

	return aCapabilities;
}

EmailService.prototype.initPollingInterval = function( oServiceInfo ) 
{
    return getXmlAttributeValue(oServiceInfo, "pollingInterval", g_nDefaultPollingInterval);
}

EmailService.prototype.initNeedLspTab = function(oServiceInfo)
{	
	var nLspId = getXmlAttributeValue(oServiceInfo, "lspId", 0);
	return nLspId && (nLspId == g_nClientLspId) ? true : false;
}

EmailService.prototype.initPromoExpiration = function( oServiceInfo )    
{
    var sDate =  getXmlAttributeValue(oServiceInfo, "promoExpiration");
    return sDate ? new Date(sDate) : null; 
}

EmailService.prototype.onRemoveServiceCredentialsResponse = function(oAjax)
{
	try
	{
		if (oAjax.readyState != 4)
			return;
		
		if (oAjax.status != 200)
			throw new MNException("credApi_remove", oAjax.status);
		
		var oResponse = parseIcqProtocol(oAjax.responseText);
		if (oResponse.statusCode != 200)
			throw new MNException("credApi_remove", oResponse.statusCode);

		this.reinitCredentials(null);

		//Performance
		delete oResponse;
		oResponse = null;
	}
	catch (e) { handleException(e); }
}

EmailService.prototype.reinitCredentials = function(oServiceCredentialsRecord)
{
    if ( oServiceCredentialsRecord ) // Credentials available for service
    {
        if( this.m_eCredentialsState == MECredentialsState.kNotSet ) // No previous credentials - New service added
        {  
            this.m_oCredentials = oServiceCredentialsRecord;
            this.m_eCredentialsState = MECredentialsState.kSet;
            var oEvent = new Event("onServiceAdded", this.m_sId);
            oEvent.fire();

            // when new credentials set - we need to reconnect using them
            this.m_oAgent.connectToServiceHost();
    		
            //statistics
            updateXtraProductSrp(STAT_CATEGORY_ID, SERVICES_STAT_ID, this.m_sId);      
        }
        else // Previous credentials available - Edit credentials if needed. Revoke previous token if needed
        {
            if(false == this.compareCredentials( oServiceCredentialsRecord ) )   
            {
                // First let agent revoke the previous credentials
                this.m_oAgent.revokeCredentials();

                this.m_oCredentials = oServiceCredentialsRecord;
                this.m_oAgent.connectToServiceHost(); // Re-read user mails
            }
        }
    }
	else if (this.m_eCredentialsState > MECredentialsState.kNotSet) // No credentials available - Remove current credentials if needed
	{
		this.m_oCredentials = null; 
		this.m_eCredentialsState = MECredentialsState.kNotSet;
		var oEvent = new Event("onServiceRemoved", this.m_sId);
		oEvent.fire();
	}
}

EmailService.prototype.revokeCredentials = function()
{    
    this.m_oAgent.revokeCredentials(); // give agent chance to do its specification here
        
    var sArgs = '&serviceId=' + this.m_sId;
    var sUrl = g_sCredentialsApiUrl + '/set?f=json&&credToken=&credSecret=' + sArgs;
    
    sUrl = g_oIcqConnector.Owner.Url.SignUrl(sUrl, g_nothing);

	var oAjax = new XMLHttpRequest();	    
    oAjax.open("GET", sUrl, true);	    
	oAjax.send('');
	oAjax.onreadystatechange = makeAjaxCallback( oAjax, this, EmailService.prototype.onRemoveServiceCredentialsResponse );
}

EmailService.prototype.onInboxModified = function(nNewMailsCount)
{
	var oEvent = new Event("onServiceMailsModified", this.m_sId);
	oEvent.fire();

	if (nNewMailsCount == 1)
	{
		this.raiseNewMailToaster(this.m_aUnreadEmails[0]);
	}
	else if (nNewMailsCount > 1)
	{

		this.raiseNewMailsToaster(nNewMailsCount);
	}
}

EmailService.prototype.raiseNewMailToaster = function(oEmail)
{
	var sServiceId = this.m_sId;
	var oToaster = createToaster(sServiceId, "newMailsToaster.SingleTitle");

	var sLinkText = g_oIcqConnector.GetString("newMailsToaster.OpenLink");
	var oAction = new ToasterAction("onNewMailClicked", sServiceId);
	oAction.m_sMailId = oEmail.getId();

	oToaster.popNewMailToaster(oEmail.getDisplayName(), oEmail.getTitle(), oEmail.getFormatedDate(), sLinkText, oAction);
}

EmailService.prototype.raiseNewMailsToaster = function(nNewMailsCount)
{
	var sServiceId = this.m_sId;
	var oToaster = createToaster(sServiceId, "newMailsToaster.MultiTitle");

	var oAction = new ToasterAction("onNewMailsClicked", sServiceId);	
	var sLinkText = g_oIcqConnector.GetString("newMailsToaster.OpenLink");
	var sMsgBoddy = g_oIcqConnector.GetString("newMailsToaster.MsgBoddy");

	sMsgBoddy = sMsgBoddy.replace("#NUMBER#", nNewMailsCount);

	oToaster.popNewMailsToaster(sMsgBoddy, sLinkText, oAction);
}

EmailService.prototype.reset = function()
{
	this.m_aUnreadEmails = [];
	this.m_nUnreadEmailsCount = 0;
	this.m_nUIBalloonCount = 0;
	this.m_oAgent.reset();
}

EmailService.prototype.onRequestState = function(requestStatus)
{
	// first trigger next try timer
	if (this.m_updateTimerId)
		this.m_updateTimerId.cancel();

	this.m_updateTimerId = setTimeoutObject(this.m_oAgent, "updateUnreadMailsCount", this.m_nPollingInterval * 1000);

	var eNewStatus;
	var eNewCredentialsState;

	switch (requestStatus)
	{
		case 0:
		case 200:
			eNewStatus = MEServiceState.kOnline;
			eNewCredentialsState = MECredentialsState.kValid;
			break;

		case 401:
			eNewStatus = MEServiceState.kOffline;
			eNewCredentialsState = MECredentialsState.kExpired;
			break;

		default:
			eNewStatus = MEServiceState.kOffline;
			eNewCredentialsState = MECredentialsState.kNotValid;
			break;
	}

	if (eNewStatus === MEServiceState.kOffline)
		this.reset();

	if ((eNewStatus != this.m_eStatus) || (eNewCredentialsState != this.m_eCredentialsState))
	{
		this.m_eStatus = eNewStatus;
		this.m_eCredentialsState = eNewCredentialsState;

		(new Event("onServiceStateChanged", this.m_sId)).fire();
	}
}

EmailService.prototype.retryToConnect = function()
{
	if ( !this.m_oCredentials ) 
		return false;

	// reset UI to connecting mode
	this.m_eCredentialsState = MECredentialsState.kSet;
	(new Event("onServiceStateChanged", this.m_sId)).fire();

	this.m_oAgent.connectToServiceHost();
}

////////////////// Email class ////////////////////////////////
function Email(sSenderMailAddress, sSubject, dtReceiveDate, sSenderName, bHasAttachment, sId, sPreview)
{
    this.getId = function(){ return (sId != undefined) ? sId : "";}     
    this.getSubject = function(){ return (sSubject != undefined) ? sSubject : "";}
    this.getPreview = function(){ return (sPreview != undefined) ? sPreview : "";} 
    this.getSenderName = function(){ return (sSenderName != undefined) ? sSenderName : "";} 
    this.getHasAttachment = function(){ return bHasAttachment || 0;} 
    this.getSenderMailAddress = function(){ return (sSenderMailAddress != undefined) ? sSenderMailAddress : "";} 
    this.getReceiveDate = function()
    { 
        return (dtReceiveDate != undefined) && (dtReceiveDate instanceof Date) ? dtReceiveDate : null;
    }
}

Email.prototype.getDisplayName = function()
{
	var sSenderName = this.getSenderName();
	
	if ( sSenderName == "" )
		sSenderName = this.getSenderMailAddress();

	return sSenderName;
}

Email.prototype.getFormatedDate = function()
{
	var dt = this.getReceiveDate();
	if (!dt)
	    return "";

    var today = new Date();
    var isCurrYear = today.getFullYear() == dt.getFullYear();
	var isToday = today.getDate() == dt.getDate() &&
	              today.getMonth() == dt.getMonth() &&
	              isCurrYear;

	if (isToday)
	{
	    var minutes = dt.getMinutes();
	    if (minutes < 10) // Add trailing 0 if needed
	        minutes = "0" + minutes;
	        
		return ( String(dt.getHours()) + ':' + minutes );
	}
	else
	{
	    return  ( String(dt.getDate()) + '/' +          // Day
	            String(dt.getMonth() + 1) + '/' +     // Month
	            String(dt.getYear()).substr(2, 2) );	// Two last digits of year
	}
}

Email.prototype.getTitle = function()
{
	var sMsg = this.getSubject();
	if (sMsg == "")
		sMsg = g_oIcqConnector.GetString("newMailsToaster.NoSubject");
	return sMsg;
}


////////////////// EmailServiceAgentBase class ////////////////////////////////
// this class is common bass class for all agents implementors objects as GmailAgent

function EmailServiceAgentBase( oEmailService )
{    
    this.m_oEmailService	= oEmailService;    
}

EmailServiceAgentBase.prototype.getImpl = function()
{
	if (!this.m_oAgentImpl)
		this.m_oAgentImpl = this.initImplementationObject();	
	return this.m_oAgentImpl;
}

EmailServiceAgentBase.prototype.initImplementationObject = function()
{
	switch (this.m_oEmailService.m_sId)
	{
		case "gmail":
			if (!importScript(g_sServiceFolderPath + "gmailagent.js"))
				return null;
			return new GmailAgent(this);
		case "yahoo":			
			if (!importScript(g_sServiceFolderPath + "yahooagent.js"))
				return null;
			return new YahooAgent(this);
	}

	//error handling
	handleException(new MNException("mainSrc", MEExeptions.kLoadUnsupportedAgent));
	return null; 
}


EmailServiceAgentBase.prototype.connectToServiceHost = function()
{
    this.getImpl().connectToServiceHost();
}

EmailServiceAgentBase.prototype.importUnreadMails = function()
{
    this.getImpl().importUnreadMails();
}

EmailServiceAgentBase.prototype.updateUnreadMailsCount = function()
{
	this.getImpl().updateUnreadMailsCount();
}   

EmailServiceAgentBase.prototype.reset = function()
{
	if (this.m_oAgentImpl)
		delete this.m_oAgentImpl; // will create new one on getImpl()
}

EmailServiceAgentBase.prototype.revokeCredentials = function()
{
    if ( this.getImpl().revokeCredentials )
        return this.getImpl().revokeCredentials();
    
    return true; // service should continue...
}

EmailServiceAgentBase.prototype.readMail = function( sMailId )
{   
    if ( this.getImpl().readMail )
    {
        this.getImpl().readMail( sMailId );
        updateXtraProductSrp(STAT_CATEGORY_ID, LINKS_STAT_ID, this.m_oEmailService.m_sId + ":read");
    }
}

EmailServiceAgentBase.prototype.deleteMail = function( sMailId )
{
    if ( this.getImpl().deleteMail )
    {
        this.getImpl().deleteMail( sMailId );
        updateXtraProductSrp(STAT_CATEGORY_ID, LINKS_STAT_ID, this.m_oEmailService.m_sId + ":delete");
    }

}

EmailServiceAgentBase.prototype.markAsRead = function( sMailId )
{
    if ( this.getImpl().markAsRead )
    {
        this.getImpl().markAsRead( sMailId );
        updateXtraProductSrp(STAT_CATEGORY_ID, LINKS_STAT_ID, this.m_oEmailService.m_sId + ":mark");
    }
}

EmailServiceAgentBase.prototype.composeMail = function()
{
     if ( this.getImpl().composeMail )
     {
        this.getImpl().composeMail();
        updateXtraProductSrp(STAT_CATEGORY_ID, LINKS_STAT_ID, this.m_oEmailService.m_sId + ":compose");
     }
}

EmailServiceAgentBase.prototype.openCalendar = function()
{
    if ( this.getImpl().openCalendar )
    {
        this.getImpl().openCalendar();
        updateXtraProductSrp(STAT_CATEGORY_ID, LINKS_STAT_ID, this.m_oEmailService.m_sId + ":calendar");
    }
}

EmailServiceAgentBase.prototype.openAddressBook = function()
{
    if ( this.getImpl().openAddressBook )
    {
        this.getImpl().openAddressBook();
        updateXtraProductSrp(STAT_CATEGORY_ID, LINKS_STAT_ID, this.m_oEmailService.m_sId + ":addressBook");
    }
}

EmailServiceAgentBase.prototype.openInbox = function()
{
    if ( this.getImpl().openInbox )
    {
        this.getImpl().openInbox();
        updateXtraProductSrp(STAT_CATEGORY_ID, LINKS_STAT_ID, this.m_oEmailService.m_sId + ":inbox");
    }
}

EmailServiceAgentBase.prototype.queryProxy = function(sAction, oCallbackOwner, callback, nRequestId)
{
	try
	{
		var oCredentials = this.m_oEmailService.m_oCredentials;

		var sUrl = 'https://urlp.icq.com/api/email_notifier/' + sAction + '?';
		var oParams =
  			{
  				sn: g_oIcqConnector.Owner.ScreenName,
  				service: this.m_oEmailService.m_sId,
  				oauth_token: oCredentials.token,
  				oauth_token_secret: oCredentials.secretKey,
  				nonce: String(Math.random()).substr(2),
  				time: Math.floor((new Date()).getTime() / 1000)
  			};

		if (nRequestId)
			oParams.r = nRequestId;

		if (oCredentials.credSessionHandle)
			oParams.oauth_session_handle = oCredentials.credSessionHandle;

		var sParams = "";
		for (i in oParams)
			sParams += '&' + i + '=' + encodeURIComponent(oParams[i]);
				
		sUrl += sParams.substring(1);
		var oAjax = new XMLHttpRequest();
		oAjax.open("GET", sUrl, true);
		oAjax.send('');

		oAjax.onreadystatechange = makeAjaxCallback(oAjax, oCallbackOwner, callback, nRequestId);
	} catch (e) { }
}

EmailServiceAgentBase.prototype.calcDiffState = function(nOldFullCount, aOldEmailList, nNewFullCount, aNewEmailsList)
{
	var nNewMailsCount = 0;
	if (nNewFullCount == 0)
	{
		nNewMailsCount = 0; // will clean ballon
	}
	else
	{
		if (nOldFullCount == 0)
			nNewMailsCount = nNewFullCount;
		else
		{
			var nResult = 0;
			var nNewFeedsCount = aNewEmailsList.length;
			var oLatestOldMail = aOldEmailList[0];
			var dtLatestOldMailDate = oLatestOldMail.getReceiveDate();
			var nNewMailPosition = 0;

			if (dtLatestOldMailDate >= aNewEmailsList[0].getReceiveDate())
			{
				nResult = 1; //  later then all unread mails - so no new mails                 
			}
			else if (dtLatestOldMailDate < aNewEmailsList[nNewFeedsCount - 1].getReceiveDate())
			{
				nResult = -1; //  earlier then all new mails                 
			}
			else
			{
				var i = nNewFeedsCount - 1;
				for (; i >= 0; i--)
				{
					var dtEmail = aNewEmailsList[i].getReceiveDate();

					if (dtLatestOldMailDate == dtEmail)
					{
						nNewMailPosition = i - 1;
						break;
					}
					else if (dtLatestOldMailDate < dtEmail)
					{
						nNewMailPosition = i;
						break;
					}
				}
			}
			//6, 5, 4, 3, 2, 1
			//0, 1, 2, 3, 4, 5
			nNewMailsCount = nResult == 1 ? 0 :
                                 nResult == -1 ? Math.max(nNewFullCount - nOldFullCount, nNewFeedsCount) :
			/*nResult == 0*/nNewMailPosition + 1;
		}
	}
	
	return nNewMailsCount;
}

////////////////////////	BalloonsManager	///////////////////////////
function BalloonsManager()
{
	this.m_sClientSelectedTab	= "";	
	g_aClientEventshandlers.push(g_oIcqConnector.App.AttachEvent("OnUIContentPanelSelected", makeCallback(this, BalloonsManager.prototype.onUIContentPanelSelected)));	
}

BalloonsManager.prototype.isServiceSelected = function(sServiceId)
{
	return g_oPersistPrefences.m_sLastTabId === sServiceId;
}

BalloonsManager.prototype.isServiceVisible = function( sServiceId )
{
	return	this.isMeTabVisible() && this.isServiceSelected(sServiceId);
}

BalloonsManager.prototype.isMeTabVisible = function()
{
	return this.m_sClientSelectedTab === "MeTab";
}

BalloonsManager.prototype.resetServiceBalloonCounter = function( sServiceId )
{
	var oService = g_oMainService.m_oUIStub.getService(sServiceId);
	if (oService)
	{
		oService._getImpl().m_nUIBalloonCount = 0;
	}
}

BalloonsManager.prototype.onSelectedTabChanged = function(sServiceId)
{	
	this.resetServiceBalloonCounter(sServiceId);
}

BalloonsManager.prototype.onUIContentPanelSelected = function( oEvent )
{	
	this.m_sClientSelectedTab = oEvent.Params.szPanelId;
	if (this.isMeTabVisible())
	{
		// reset selected service counter
		this.resetServiceBalloonCounter(g_oPersistPrefences.m_sLastTabId);
	}
}

BalloonsManager.prototype.resetBallonState = function(oService, nNewMailsCount, nOldFullCount, nNewFullCount)
{	
	if ( this.isServiceVisible( oService.m_sId ) )  
		return; // do nothing

	// if user was read at least one mail in its Inbox we reset this service balloon counter
	var nUIBalloonCount = (nNewFullCount - nOldFullCount < nNewMailsCount) ? nNewMailsCount : oService.m_nUIBalloonCount + nNewMailsCount;
	nUIBalloonCount = Math.min(nUIBalloonCount, oService.m_nUnreadEmailsCount);

	if (nUIBalloonCount == oService.m_nUIBalloonCount)
		return; //not changed

	oService.m_nUIBalloonCount = nUIBalloonCount;

	this.updateMeTabBalloon();

	// if service tab selected - no need UI balloon
	if ( !this.isServiceSelected(oService.m_sId) )
	{
		(new Event("onServiceBallonCountChanged", oService.m_sId)).fire();
	}
}

BalloonsManager.prototype.updateMeTabBalloon = function()
{	
	if (this.m_sClientSelectedTab == "MeTab")
		return; // do nothing

	var aEmailServices = g_oMainService.m_aEmailServices;
	var nLength = aEmailServices.length;
	var nBalloonCounter = 0;
	for (var i = 0; (i < nLength); i++)
		nBalloonCounter += aEmailServices[i]._getImpl().m_nUIBalloonCount;

	var command = g_oIcqConnector.App.CommandManager.CreateCommand("UpdateUnreadMailsCount");

	command.Owner = g_oIcqConnector.Owner;
	command.UnreadMailsCount = nBalloonCounter;
	command.Execute();
}