// --------------------------- globals  -----------------------------------

var g_oIcqConnector     	= null; // icq connector should be public for all code
var g_oMainService			= null;
var g_nClientLspId			= 0;
var g_oPersistPrefences 	= null;
var g_oBalloonsManager		= null;
var g_aAjaxsCalls			= [];
var g_aClientEventshandlers	= [];
var g_sServiceFolderPath	= "box://xtraz/icq/content/email_notifier_service/";

// --------------------------- Utils functions  -----------------------------------

function importScript( sPath )
{
	return g_oMainService.m_oComponent.gadget.importScript(sPath, "javascript");
}

function makeCallback(thisArg, funcCallback)
{
	return function() { funcCallback.apply(thisArg, arguments); }
}

// --------------------------- component -----------------------------------
function component(_gadget)
{
	this.gadget = _gadget;
}

component.prototype.OnInitXtra = function(conector)
{
	g_oIcqConnector = conector;
	g_oMainService = new MainService(this);
}

////////////////// MainService class ////////////////////////////////
// MainService is a root class that implement all the client/server logic 
// it will serve the UI needs using some stub object
// all the meta data as mail services info and variant data as emails / service state etc will holded here
// this class is singleton so we do not need to use prototype
function MainService(oComponent)
{		
    this.m_bStarted = false;
    this.m_bReady = false;
	this.m_oComponent = oComponent;
	
	// init services for client config file  
	// sorted and restricted arry of EmailService objects (metadata + variant data)    
	this.m_aEmailServices = [];

	this.m_oUIStub = new UIStub();   // UI xtra using this stub for all operations

	g_oIcqConnector.AttachEvent("ShutDown", onXtraShutdown);
		
	g_aClientEventshandlers.push(g_oIcqConnector.App.AttachEvent("OnUIContactListReady", makeCallback(this, MainService.prototype.onUIContactListReady)));

	// if CL is read we need to startUp      
	var oContactList = g_oIcqConnector.Owner.ContactList;
	if (oContactList && oContactList.IsReady)
    {
        this.startUp();
        this.queryCredentials();
	}
}

function MainService.prototype.onUIContactListReady(Event)
{
    this.startUp();
    this.m_bReady = false;
    this.queryCredentials();
}

MainService.prototype.isReady = function()
{
    return this.m_bReady;
}

MainService.prototype.startUp = function()
{
	if (!this.m_bStarted)
	{
		if (!importScript(g_sServiceFolderPath + "infrastructure.js"))
			return;

		g_oPersistPrefences = new PersistPrefences();
		g_oBalloonsManager = new BalloonsManager();
				
		g_oIcqConnector.AttachEvent("OnPluginNotification", makeCallback(this, MainService.prototype.onPluginNotification));

		this.InitServices();		
		this.m_bStarted = true;
    }   
}

////////////////// UIStub class ////////////////////////////////
function UIStub()
{
	this.m_uiEventsCallback = null; // when UI read should set callback for events

	// expose this object as UI API 
	g_oIcqConnector.ShareObject("emailNotifierService", this); // share for UI Xtra Usage
}

UIStub.prototype.startUp = function()
{
    if (!g_oMainService.m_bStarted)
        g_oMainService.startUp();
}

UIStub.prototype.isReady = function()
{
    return g_oMainService.isReady();
   }

function onXtraShutdown()
{
   	// clean open ajax calls
   	var nLength = g_aAjaxsCalls.length;
   	for (var i = nLength - 1; i >= 0; i--)
   	{
   		var oAjax = g_aAjaxsCalls[i];
   		if (oAjax)
   		{
   			oAjax.abort();
   			oAjax = null;
   		}
   	}
   	g_aAjaxsCalls = [];

   	if (g_oMainService.cleanup)
   		g_oMainService.cleanup();

   	nLength = g_aClientEventshandlers.length;
   	for (var i = nLength - 1; i >= 0; i--)
   	{
   		g_aClientEventshandlers[i].DetachEvent();
   		g_aClientEventshandlers[i] = null;
   	}

   	g_aClientEventshandlers = null;
   	g_aClientEventshandlers = [];

   	g_oPersistPrefences = null;
   	g_oBalloonsManager = null;
   	g_oMainService = null;
   	g_oIcqConnector = null;
}