////////////////// YahooAgent class ////////////////////////////////
function YahooAgent( oEmailServiceAgentBase )
{
	this.m_oEmailServiceAgentBase = oEmailServiceAgentBase;
	this.m_nQueriesCounter = 1;
	this.m_aQueries = [];	
}

YahooAgent.prototype.getNexyQueryId = function()
{
	return this.m_nQueriesCounter++;
}

YahooAgent.prototype.connectToServiceHost = function()
{
    this.importUnreadMails();
}

YahooAgent.prototype.updateUnreadMailsCount = function()
{
    this.importUnreadMails(); 
}

YahooAgent.prototype.importUnreadMails = function()
{	
	var oCredentials = this.m_oEmailServiceAgentBase.m_oEmailService.m_oCredentials;
	if (!oCredentials)
		return;

	var oParams = [{ fid: "Inbox", startInfo: 0, numInfo: 20, sortKey: "date", sortOrder: "down", filterBy: { isRead: 0}}];
	var oRequest = { method: 'ListMessages', params: oParams };

	this.queryYahooService(oRequest, YahooAgent.prototype.queryFeedsResponseHandler)
}

YahooAgent.prototype.queryYahooService = function(oRequest, funcHandler)
{	
	oRequest.id = this.getNexyQueryId();
	oRequest.handler = funcHandler;

	this.m_aQueries.push(oRequest);

	this.m_oEmailServiceAgentBase.queryProxy("getOAHeader", this, YahooAgent.prototype.queryHeaderHandler, oRequest.id);
}

YahooAgent.prototype.queryHeaderHandler = function(oAjax, nRequestId)
{
	try
	{
		if (oAjax.readyState != 4)
			return;
		
		if (oAjax.status != 200)
			throw new MNException(g_sWebServiceId + '_feedsHeader', oAjax.status, this.m_oEmailServiceAgentBase.m_oEmailService);

		var oResponse = parseIcqProtocol(oAjax.responseText);
		if (oResponse.statusCode != 200)
			throw new MNException(g_sWebServiceId + '_feedsHeader', oResponse.statusCode, this.m_oEmailServiceAgentBase.m_oEmailService);

		this.sendQueryToYahoo(oResponse.data.authorizationHeader, nRequestId);

		//Performance
		delete oResponse;
		oResponse = null;
	}
	catch (e) { handleException(e); }
}

function makeRequestMatcher(nRequestId)
{
	return function(oRequest) { return oRequest.id == nRequestId; }
}

YahooAgent.prototype.sendQueryToYahoo = function(sHeader, nRequestId)
{	
	var oRequest = this.m_aQueries.find(makeRequestMatcher(nRequestId));

	if (!oRequest)
		return;

	var sRequest = convert2Json(oRequest);

	var oAjax = new XMLHttpRequest();
	oAjax.open("POST", 'http://mail.yahooapis.com/ws/mail/v1.1/jsonrpc', true);
	oAjax.setRequestHeader("Content-Type", "application/json");
	oAjax.setRequestHeader("Accept", "application/json");
	oAjax.setRequestHeader("Content-length", sRequest.length);
	oAjax.setRequestHeader("Authorization", sHeader);
	oAjax.setRequestHeader("Connection", "close");
	oAjax.send(sRequest);
	oAjax.onreadystatechange = makeAjaxCallback(oAjax, this, YahooAgent.prototype.yahooServiceResponseHandler, nRequestId);
}

YahooAgent.prototype.yahooServiceResponseHandler = function(oAjax, nRequestId)
{
	try
	{
		if (oAjax.readyState != 4)
			return;
		
		var oService = this.m_oEmailServiceAgentBase.m_oEmailService;
		var nRequestStatus = oAjax.status;
		var sResponse = oAjax.responseText;
		
		if ((nRequestStatus == 401) && (/token_expired/ig.test(sResponse)))
		{ // we need to refresh token			
			this.getRefreshTokenOAHeader(nRequestId);
			return;
		}

		if (nRequestStatus != 200)
			throw new MNException(this.m_oEmailServiceAgentBase.m_oEmailService.m_sId, nRequestStatus, this.m_oEmailServiceAgentBase.m_oEmailService);

		oService.onRequestState(nRequestStatus);

		var oYahooResponse = secureJsonParse(sResponse);
		if (!oYahooResponse)
			throw new MNException(this.m_oEmailServiceAgentBase.m_oEmailService.m_sId, MEExeptions.kInvalidJsonFormat, this.m_oEmailServiceAgentBase.m_oEmailService);

		var oRequest = this.m_aQueries.find(makeRequestMatcher(nRequestId), true/*remove*/);

		if (!oRequest)
			return;

		if (oRequest.handler)
			oRequest.handler.call(this, oYahooResponse);

		//Performance
		delete oYahooResponse;
		oYahooResponse = null;
		delete oRequest;
		oRequest = null;
	}
	catch (e) { handleException(e); }
}

YahooAgent.prototype.getRefreshTokenOAHeader = function(nRequestId)
{
	this.m_oEmailServiceAgentBase.queryProxy("refreshTokenOAHeader", this, YahooAgent.prototype.onRefreshTokenOAHeaderArrive, nRequestId);
}

YahooAgent.prototype.onRefreshTokenOAHeaderArrive = function(oAjax, nRequestId)
{
	try
	{
		if (oAjax.readyState != 4)
			return;
		
		if (oAjax.status != 200)
			throw new MNException(g_sWebServiceId + '_sessionRenew', oAjax.status, this.m_oEmailServiceAgentBase.m_oEmailService);

		var oResponse = parseIcqProtocol(oAjax.responseText);
		if (oResponse.statusCode != 200)
			throw new MNException(g_sWebServiceId + '_sessionRenew', oResponse.statusCode, this.m_oEmailServiceAgentBase.m_oEmailService);

		var oCredentials		= this.m_oEmailServiceAgentBase.m_oEmailService.m_oCredentials;
		var oData				= oResponse.data;

		oCredentials.token				= oData.token;
		oCredentials.secretKey			= oData.secret;
		oCredentials.credSessionHandle	= oData.session;
		this.sendQueryToYahoo(oData.authorizationHeader, nRequestId);

		//Performance
		delete oResponse;
		oResponse = null;		
    }
    catch (e) { handleException(e); }
}

YahooAgent.prototype.queryFeedsResponseHandler = function(oYahooResponse)
{	
	if (oYahooResponse.error)
		return this.handleError(oYahooResponse.error);
		
	this.parseFeedsResponse(oYahooResponse.result);
}

YahooAgent.prototype.parseFeedsResponse = function(oFeeds)
{
	var nNewFullCount = oFeeds.folder.unread;
	var sFolderId = oFeeds.folder.folderInfo.fid;

	var aEntries = oFeeds.messageInfo;
	var nLength = aEntries.length;

	var oService = this.m_oEmailServiceAgentBase.m_oEmailService;

	// save old state for calculation        
	var aOldEmailList = oService.m_aUnreadEmails;
	var nOldFullCount = oService.m_nUnreadEmailsCount;

	var bIsModified = (nOldFullCount != nNewFullCount) ||
						(
							(nNewFullCount > 0) && // if so them both greater then 0
							(
								(aOldEmailList[0].getId() != aEntries[0].mid) ||
								(aOldEmailList[aOldEmailList.length - 1].getId() != aEntries[aEntries.length - 1].mid)
							)
						);

	if (!bIsModified)
		return; // nothing changed	


	var aNewEmailsList = new Array(nLength);
	for (i = 0; i < nLength; i++)
	{
		var oEntry = aEntries[i];
		var sSubject = oEntry.subject;
		var sPreview = "";
		var sId = oEntry.mid;
		var sSenderName = oEntry.from.name;
		var sSenderMailAddress = oEntry.from.email;
		var bHasAttachment = oEntry.flags.hasAttachment;
		var dtReceiveDate = new Date(oEntry.receivedDate * 1000);

		aNewEmailsList[i] = new Email(sSenderMailAddress, sSubject, dtReceiveDate, sSenderName, bHasAttachment, sId, sPreview);
	}

	// set new state
	oService.m_aUnreadEmails = aNewEmailsList;
	oService.m_nUnreadEmailsCount = nNewFullCount;

	// calc new state        
	var nNewMailsCount = this.m_oEmailServiceAgentBase.calcDiffState(nOldFullCount, aOldEmailList, nNewFullCount, aNewEmailsList);

	oService.onInboxModified(nNewMailsCount);

	g_oBalloonsManager.resetBallonState(oService, nNewMailsCount, nOldFullCount, nNewFullCount);
}

YahooAgent.prototype.handleError = function(oError)
{
	var nErrorCode = oError.code ? oError.code : MEExeptions.kServiceGenericError;
	var oService = this.m_oEmailServiceAgentBase.m_oEmailService;
	handleException(new MNException(oService.m_sId, nErrorCode, oService));
}

YahooAgent.prototype.readMail = function( sMailId )
{
	var oEmail = this.m_oEmailServiceAgentBase.m_oEmailService.m_aUnreadEmails.find(makeEmailIdMatcher(sMailId));
	if (!oEmail)
		return false; // try to operate something that not exist

	var sUrl = 'http://mrd.mail.yahoo.com/msg?';
	var oParams =
	{
		mid : sMailId,
		fid : "Inbox"
	};

	var sParams = "";
	for (i in oParams)
		sParams += '&' + i + '=' + encodeURIComponent(oParams[i]);

	sParams = sParams.substring(1);
	sUrl += sParams;
	g_oIcqConnector.OpenUrl(sUrl, false);         
}

YahooAgent.prototype.deleteMail = function(sMailId)
{	
	var oEmail = this.m_oEmailServiceAgentBase.m_oEmailService.m_aUnreadEmails.find(makeEmailIdMatcher(sMailId));
	if (!oEmail)
		return false; // try to delete something that not exist

	var oListMessagesParams = { fid: "Inbox", startInfo: 0, numInfo: 20, sortKey: "date", sortOrder: "down", filterBy: { isRead: 0} };
	var oListMessagesCall = { ListMessages: oListMessagesParams };

	var oMoveMessagesParams = { sourceFid: "Inbox", destinationFid: "Trash", mid: sMailId };
	var oMoveMessagesCall = { MoveMessages: oMoveMessagesParams };

	var oParams = [{ call: [oMoveMessagesCall, oListMessagesCall]}];
	var oRequest = { method: 'BatchExecute', params: oParams };

	this.queryYahooService(oRequest, YahooAgent.prototype.batchOperationResponseHandler)
}

YahooAgent.prototype.batchOperationResponseHandler = function(oYahooResponse)
{
	try
	{
		if (oYahooResponse.error)
			return; //TODO error handling

		var aResponse = oYahooResponse.result.response;
		var oListMessagesResponse = aResponse[aResponse.length - 1];

		if (oListMessagesResponse &&
			oListMessagesResponse.ListMessagesResponse &&
			oListMessagesResponse.success)
			this.parseFeedsResponse(oListMessagesResponse.ListMessagesResponse);
	}
	catch (e) { }
}

YahooAgent.prototype.markAsRead = function(sMailId)
{	
	var oEmail = this.m_oEmailServiceAgentBase.m_oEmailService.m_aUnreadEmails.find(makeEmailIdMatcher(sMailId));
	if (!oEmail)
		return false; // try to operate something that not exist
	
	var oListMessagesParams = { fid: "Inbox", startInfo: 0, numInfo: 20, sortKey: "date", sortOrder: "down", filterBy: { isRead: 0} };
	var oListMessagesCall = { ListMessages : oListMessagesParams };

	var oFlagMessagesParams = { fid: "Inbox" , mid: sMailId, setFlags: { read: 1} };
	var oFlagMessagesCall = { FlagMessages : oFlagMessagesParams };

	var oParams = [{ call: [oFlagMessagesCall, oListMessagesCall]}];
	var oRequest = { method: 'BatchExecute', params: oParams };

	this.queryYahooService(oRequest, YahooAgent.prototype.batchOperationResponseHandler)
}

YahooAgent.prototype.openAddressBook = function()
{
    g_oIcqConnector.OpenUrl("http://mrd.mail.yahoo.com/contacts");
}

YahooAgent.prototype.composeMail = function()
{
    g_oIcqConnector.OpenUrl("http://mrd.mail.yahoo.com/compose");
}

YahooAgent.prototype.openInbox = function()
{
    g_oIcqConnector.OpenUrl("http://mrd.mail.yahoo.com/inbox");
}

