// EmailNotSettings.js
// Settings dialog for email notification implementation file

var kPromptTypeYesNoQuestion = 0x101;
var kPromptRetYes = 6;

// Called to initialize the dialog. Shared object should be passed via shell.prefsArr object
function OnInitDialog()
{

    if (shell.emailNotSettingsPrefs.length != 2)
        return false;

    this.m_interfaceObj = shell.emailNotSettingsPrefs[0];
    this.m_icqConnector = shell.emailNotSettingsPrefs[1];
    
    InitServices();

    // Remove given attributes
    shell.emailNotSettingsPrefs = null;

    return true;
}

function InitServices()
{
    // Loop over the registered services and add them to the list control
    var services = this.m_interfaceObj.getServices();

    var nNumServices = services.length;
    var rows = scene.rootBox.getPartById("idListBox").getBoxByTag("rows");

    for (index = 0; index < nNumServices; index++) {
    
        // Show only registered services
         if (false == services[index].isRegisterd())
            continue;
            
        var svcLine = scene.createBox("ENSServiceLine");
        var svcId = services[index].getId();
        
        // TODO: Change the way we build the icon path
        var sIconPath = services[index].getSmallIcon();
        
        svcLine.setAttribute("svcId", svcId);
        svcLine.setAttribute("svcIcon", sIconPath);
        svcLine.setAttribute("svcName", services[index].getDisplayName());
        
        // Do not draw a line above the first service
        if (index == 0)
            svcLine.setAttribute("drawline", "false");

        rows.addChild(svcLine);
    }

    scene.rootBox.setAttribute("servicesAvail", rows.childCount > 0 ? "true" : "false");
	    
	
}

function OnButtonPressed()
{
    var targetBox = shell.currentEventTargetBox;
    if (!targetBox)
        return;

    var svcLine = targetBox.gadgetParent;
    if (svcLine && svcLine.tag == "ENSServiceLine") {
        var svcId = svcLine.getAttribute("svcId");
        if (targetBox.id == "idRemoveButton") {
        
            var service = this.m_interfaceObj.getService(svcId);

            // Get body message format
            var sFormat = this.m_icqConnector.GetString("email_not_settings.prompt");

            var res = this.m_icqConnector.ShowPrompt(
                sFormat.replace("#SERVICE_NAME#", service.getDisplayName()),
                kPromptTypeYesNoQuestion, kPromptRetYes);
                
            if (res != kPromptRetYes) // If the user didn't click yes, do nothing
                return;

            this.m_interfaceObj.removeService(svcId);
            var prevBox = svcLine.previousSibling;
            var nextBox = svcLine.nextSibling;
            
            var rows = scene.rootBox.getPartById("idListBox").getBoxByTag("rows");
            rows.deleteChild(svcLine);

            if (!prevBox && nextBox)
                nextBox.setAttribute("drawline", "false");
            
            if (rows.childCount == 0)
                scene.rootBox.setAttribute("servicesAvail", "false");
        }
        else if (targetBox.id == "idEditButton") {
            this.m_interfaceObj.registerToService(svcId);
        }    
    }

}

function OnClose()
{
    scene.close();
}

