/***************************************************************************

 File Name:   TabControl.js

***************************************************************************/

var selectedIndex; // Current selected index

function component (_gadget)
{
    this.gadget = _gadget;
    this.scene = _gadget.scene;
    this.headers = null;
    this.panels = null;
    selectedIndex = -1;
}

component.prototype.OnPresented = function() {
    this.init();

    //Parent gadget may implement these event handlers
    this.gadget.addEventHandler("needTabPanel", "gadget:needTabPanel();");
    this.gadget.addEventHandler("selTabChanged", "gadget:selTabChanged();");
}


// fired whenever an attribute is changed on headers
component.prototype.OnHeaderAttrSet = function() {
    var event = shell.currentEvent;
    if (event.attributeName == "cursorIndex") {
        /*
        if (removingTab)
            return;
        */
        var newSelectedIndex = this.headers.selectedIndex;

        //Do something only if the selection has changed
        if (newSelectedIndex >= 0 && newSelectedIndex != selectedIndex) {

            var oldSelectedIndex = selectedIndex;
            selectedIndex = newSelectedIndex;

            // If we still don't have a panel for the given index
            // fire an event to gadget's parent. The application layer is in charge of creating the actual panel
            if (this.isBogusPanel(selectedIndex)) {

                var event = shell.factory.createEvent(this.gadget);
                event.type = "needTabPanel";
                this.gadget.dispatchEvent(event);
            }

            // Hide the previously selected panel
            if (oldSelectedIndex >= 0) {
                var prevSelectedPanel = this.panels.getChildAt(oldSelectedIndex);
                prevSelectedPanel.collapsed = true;
            }

            // show the new panel
            var selectedPanel = this.panels.getChildAt(newSelectedIndex);
            selectedPanel.collapsed = false;

            // fire an event to the gadget's parent, indicating the selection had changed
            var event2 = shell.factory.createEvent(this.gadget);
            event2.type = "selTabChanged";
            this.gadget.dispatchEvent(event2);
        }
    }
}

//****************************************************************************
// Adds the given header to the tab headers. Creates a bogus panel in order to keep the panels array at sync
component.prototype.AddTab = function(box) {
    this.init();

    if (this.headers == null || this.panels == null)
        return null;

    this.headers.addChild(box);
    this.panels.addChild(this.createBogusPanel(this.headers.childCount));
}

// Inserts a header in the given index. Creates a bogus panel in order to keep the panels array at sync
component.prototype.InsertTabAt = function(box, index) {
    this.init();

    if (this.headers == null || this.panels == null)
        return null;

    this.headers.insertChildAt(box, index);
    this.panels.insertChildAt(this.createBogusPanel(index), index);
}

// Removes the header and panel at the given index. Resets the selection if needed
component.prototype.RemoveTab = function(index) {
    this.init();

    if (this.headers == null || this.panels == null)
        return false;

    // Check validity of index
    if (index >= this.panels.childCount)
        return false;

    // Remove the header and panel    
    this.panels.deleteChild(this.panels.getChildAt(index));
    this.headers.deleteChild(this.headers.getChildAt(index));

    // If the removed tab was the selected tab, reset the current selection
    if (index == selectedIndex) {
        selectedIndex = -1;
        this.headers.selectedIndex = -1;
    }
    // Update the selection after the remove ends
    else if (selectedIndex > index) {
        selectedIndex = selectedIndex - 1;
        this.headers.selectedIndex = selectedIndex;
    }

    return true;
}

// Changes the current active tab
component.prototype.SelectTab = function(index) {
    this.init();

    if (this.headers == null || this.panels == null)
        return null;

    this.headers.selectedIndex = index;
    selectedIndex = index;
}

//Replace the panel at a given index
component.prototype.InsertItemPanel = function(box, index) {
    if (index < this.panels.childCount)
        this.panels.deleteChild(this.panels.getChildAt(index));
        
    this.panels.insertChildAt(box, index);

    box.setStyle("flex", "1");
    box.setStyle("width", "auto");
    box.setStyle("maxwidth", "auto");
    box.setStyle("height", "auto");
    box.setStyle("maxheight", "auto");
    
    if(this.GetSelectedIndex() != index)
        box.collapsed = true;
}

// Returns the current selected tab
component.prototype.GetSelectedIndex = function() {
    return selectedIndex;
}

// Returns the number of tabs
component.prototype.GetCount = function() {
    this.init();
    return this.headers.childCount;
}

//return the header of the tab
component.prototype.GetHeader = function(headerIndex) {
    selectedHeader = this.headers.getChildAt(headerIndex);
    return selectedHeader;
}

//return the header of the tab
component.prototype.GetPanel = function(panelIndex) {
    selectedPanel = this.panels.getChildAt(panelIndex);
    return selectedPanel;
}

//return header item (box) by key and value
component.prototype.GetHeaderByKey = function(sKey, sValue) {
    
    var nNumServices = this.GetCount()-1;
    var index = 0;
    
    while(index < nNumServices)
    {
        oCurrentHeader = this.headers.getChildAt(index);
        if(oCurrentHeader.getAttribute(sKey) == sValue)
        {
            return oCurrentHeader;
        }
        index++;
    }
    
    return null;
}

//return header value in the controller by key and value
component.prototype.GetHeaderIndexByKey = function(sKey, sValue) {
   
    var nNumServices = this.GetCount();
    var index = 0;
    
    while(index < nNumServices)
    {
        oCurrentHeader = this.headers.getChildAt(index);
        if(oCurrentHeader.getAttribute(sKey) == sValue)
        {
            return index;
        }
        index++;
    }
    
    return -1;
}

//return the panel of the service using the service's Id
component.prototype.GetPanel = function(serviceId) {
    selectedPanel = this.panels.getChildAt(serviceId);
    return selectedPanel;
}

//return the service ID of a service tab
component.prototype.GetTabAttibuteByIndex = function(headerIndex, sKey) {
    var tabIndex = (headerIndex != undefined)?headerIndex:this.GetSelectedIndex();
    selectedPanel = this.headers.getChildAt(tabIndex);
    return selectedPanel.getAttribute(sKey);
}

//updates the controller on tabs change(add/remove) 
component.prototype.updateFirstTab = function(sKey)
{
	var oFirstTab = null;
	var nNumTabs = this.GetCount();
	
	if(nNumTabs == 1)       //case only settings tab exist
	{
	    oFirstTab = this.GetHeader(0);
	    oFirstTab.setAttribute(sKey, "true");
	}
	else
	{
	    var oSettingsTab = this.GetHeader(nNumTabs-1);
	    (oSettingsTab.getAttribute("isFirstTab") == "true")?oSettingsTab.setAttribute("isFirstTab", "false"):'';
	    
	    oFirstTab = this.GetHeader(1);
	    oFirstTab.setAttribute(sKey, "false");
	    
	    oSecondTab = this.GetHeader(0);
	    oSecondTab.setAttribute(sKey, "true");
	}
}

/////////////////////////////////////////////////////
// Helper functions
function component.prototype.init() {
    // Initialize only once
    if (this.headers != null)
        return;

    this.headers = this.gadget.getPartById("headers");
    this.panels = this.gadget.getPartById("panels");
}

// Creates a temporary panel
function component.prototype.createBogusPanel(index) {
    var bogusPanel = this.scene.createBox("bogusPanel");
    if (bogusPanel == null)
        this.scene.alert("null", "null");

    var text = "bogus panel" + index;
    bogusPanel.setAttribute("text", text);
    bogusPanel.collapsed = true;
    bogusPanel.setStyle("flex", "1");

    return bogusPanel;
}

// Returns true if the panel in the given index is a temporary panel
function component.prototype.isBogusPanel(index) {
    if (this.panels == null)
        return false;

    var panel = this.panels.getChildAt(index);
    if (panel != null && panel.tag == "bogusPanel")
        return true;

    return false;
}
