//Constants vars
var sXTRA_THEME_PATH = "box://xtraz/icq/theme/icq_email_notifier_ex/";
var sXTRA_CONTENT_PATH = "box://xtraz/icq/content/icq_email_notifier_ex/";
var sXTRA_SETTINGS_WND_NAME = "emailNotSettings";
var sXTRA_ERROR_BALLOON_SYMBOL = "!";
var sXTRA_MAX_REGISTERED_SERVICES = 3;
var STAT_CATEGORY_ID = "CUsage";
var USAGE_STAT_ID = "ENP";
var SERVICES_STAT_ID = "ENE";

//Enums
MEServiceState = 
{ 
    kOffline    : 1, 
    kOnline     : 2
};  
                 
MECredentialsState = 
{ 
    kNotSet     : 1,        
    kNotValid   : 2,
    kSet        : 3,
    kExpaierd   : 4, 
    kValid      : 5
};  

//Global vars
var g_oUiApi = null;    //global: reference to stub

function component(_gadget)
{
	this.gadget = _gadget;	
	this.m_gTabControl = this.gadget.getPartById("idMyTab");    //the tab controller
	this.m_aServices = new Object;	                            //will hold the list of services
	this.nSettingsTabIndex = 0;
}


component.prototype.OnInitXtra = function(connector)
{
	this.icqConnector = connector;
	g_oUiApi = this.icqConnector.GetSharedObject("emailNotifierService");   //get the shared object from stub
	g_oUiApi.startUp();
	g_oUiApi.setEventsCallback(serviceCallBack);                            //register with callaback function at the service xtra
	serviceCallBack.owner = this;                                           //prevent multiply intances of code

	this.m_aServices = g_oUiApi.getServices();                              //get list of services from stub
	
	this.icqConnector.AttachEvent("BeforeShutDown",OnXtraClosed);           //regsiter on "BeforeShutDown" event, to release globals,close windows etc.
	OnXtraClosed.owner = this;                                              //prevent multiply intances of code
}

component.prototype.onPresented = function()
{   
    this.createServicesTab();   //create services tab
    this.createSettingsTab();   //create settings tab

	this.m_gTabControl.updateFirstTab("isFirstTab");

	if (g_oUiApi.isReady())
    {
	    var nLastSelectedTab = this.m_gTabControl.GetHeaderIndexByKey("serviceId", g_oUiApi.getLastTab());
	    this.m_gTabControl.SelectTab(nLastSelectedTab >= 0 ? nLastSelectedTab : 0);
	}
	else // Select settings tab
	{
	    this.m_gTabControl.SelectTab(0);
	}
}

component.prototype.onSettings = function()
{
    // if the scene already exists, just bring it to front
    var settingsScene = shell.getSceneByName(sXTRA_SETTINGS_WND_NAME);
    if (settingsScene != null)
    {
        settingsScene.rootBox.bringToFront();
        return;
    }
    
    shell.emailNotSettingsPrefs = new Array();
    shell.emailNotSettingsPrefs[0] = g_oUiApi;
    shell.emailNotSettingsPrefs[1] = this.icqConnector;

    var boxFile = sXTRA_CONTENT_PATH + "EmailNotSettings.box";
    settingsScene = shell.openScene(boxFile, null, sXTRA_SETTINGS_WND_NAME, null, null, false);
    settingsScene = null;
}

//destructor function
component.prototype.OnXtraClosed = function()
{
    g_oUiApi.setEventsCallback(null);
    g_oUiApi = null;
    
    //statistics
    this.sendStatistics();

    var scn = shell.getSceneByName(sXTRA_SETTINGS_WND_NAME);        //check if settings window is opened
	if (scn)
    {
        shell.closeScene(scn);
    }
}

component.prototype.sendStatistics = function()
{
    for(index=0; index < this.m_aServices.length; index++)
    {
        var bIsRegistered = this.m_aServices[index].isRegisterd();

        if(bIsRegistered)
        {
            var sServiceId = this.m_aServices[index].getId();
            this.icqConnector.Owner.OwnerStatistic.Collection.IncrementMapSecondDItem(STAT_CATEGORY_ID, SERVICES_STAT_ID, sServiceId, sServiceId); 
        }
    }
}

//global event hanlder function to prevent multi instances - call the actual function
function serviceCallBack(oEvent)
{
    arguments.callee.owner.serviceCallback(oEvent);
}

//global event hanlder function to prevent multi instances - call the actual function
function OnXtraClosed(reason)
{
	arguments.callee.owner.OnXtraClosed(reason);
}

//the actual event hanlder
component.prototype.serviceCallback = function(oEvent)
{
    switch( oEvent.m_sEventTarget )
    {
        case "onServicesReady":
                                        this.createServicesTab();
                                        var nLastSelectedTab = this.m_gTabControl.GetHeaderIndexByKey("serviceId", g_oUiApi.getLastTab());
                                        this.m_gTabControl.SelectTab(nLastSelectedTab >= 0 ? nLastSelectedTab : 0);
                                        this.m_gTabControl.updateFirstTab("isFirstTab");
                                        break;
        case "onServiceAdded":
                                        if(!g_oUiApi.isReady())
                                            return;
                                            
                                        this.m_aServices = g_oUiApi.getServices();  //first get the new services array
                                        this.addServiceTab(oEvent.m_sServiceId);
                                        this.m_gTabControl.SelectTab(0);
                                        this.m_gTabControl.updateFirstTab("isFirstTab");
                                        break;
                                           
        case "onServiceRemoved":
                                        this.removeService(oEvent.m_sServiceId);
                                        this.m_aServices = g_oUiApi.getServices();  //update the new services array
                                        this.m_gTabControl.updateFirstTab("isFirstTab");
                                        break; 
          
        case "onServiceStateChanged":
                                        var nIndexInServicesArr = this.getServiceIndexById(oEvent.m_sServiceId);
                                        var sServiceStatus = this.m_aServices[nIndexInServicesArr].getStatus();               //get currrent service status
                                        var sCredentialsState = this.m_aServices[nIndexInServicesArr].getCredentialsState();  //get currrent service Credentials State
                                        
                                        if(sServiceStatus != MEServiceState.kOnline)
                                        {
                                            switch(sCredentialsState)
                                            {
                                                
                                                case MECredentialsState.kNotValid:
                                                case MECredentialsState.kExpaierd:
                                                                                    this.updateTabBubble(oEvent.m_sServiceId, sXTRA_ERROR_BALLOON_SYMBOL);                                                         
                                                                                    break;
                                                default:
                                                                                    break;
                                            }
                                        } 
                                        
                                        var nPanelIndex = this.m_gTabControl.GetHeaderIndexByKey("serviceId", oEvent.m_sServiceId);
                                        if (false == this.m_gTabControl.isBogusPanel(nPanelIndex))
                                            this.needTabPanel(nPanelIndex);
                                            
                                        break;                              
        case "onServiceMailsModified":
                                        var nPanelIndex = this.m_gTabControl.GetHeaderIndexByKey("serviceId", oEvent.m_sServiceId);
                                        if (false == this.m_gTabControl.isBogusPanel(nPanelIndex))
                                            this.needTabPanel(nPanelIndex);
                                        break;
                                        
        case "onServiceBallonCountChanged":  
                                        var nIndexInServicesArr = this.getServiceIndexById(oEvent.m_sServiceId);
                                        var nNumUnreadEmails = this.m_aServices[nIndexInServicesArr].getBalloonCount();
                                        this.updateTabBubble(oEvent.m_sServiceId, nNumUnreadEmails);
                                        break; 
                                         
        case "onTabSelection":
                                        var nSelectedTab = this.m_gTabControl.GetHeaderIndexByKey("serviceId", oEvent.m_sServiceId);
	                                    this.m_gTabControl.SelectTab(nSelectedTab);
                                        break;      
    }
}

//loops through services member and create tab for each
component.prototype.createServicesTab = function()
{  
    var nNumServices = this.m_aServices.length;
    var bRaiseBubble = false;
    
	for(i=0; i < nNumServices; i++)
	{
	    var sServiceId = this.m_aServices[i].getId();
	    var bIsRegistered = this.m_aServices[i].isRegisterd();  //use stub to check if user is registered to service
	    
	    (bIsRegistered)?this.addServiceTab(sServiceId):'';
	}
}

//creates settings tab for each
component.prototype.createSettingsTab = function()
{ 
    var settingBox = this.gadget.scene.createBox("MailHeader"); 
    settingBox.setAttribute("isSettingsTab", "true");
    settingBox.setAttribute("serviceIcon", "#settingsTabIcon");
    settingBox.setAttribute("serviceId", "settings");
    settingBox.setAttribute("tooltip", this.icqConnector.GetString("email_notifier.addService"));
    
    this.m_gTabControl.InsertTabAt(settingBox, this.nSettingsTabIndex);
    
    var bNewService = g_oUiApi.isNeedShowNewServiceBallon();

    //poll (shared obj api) for promoted services, if at least 1 service is promoted, raise bubble
    if(bNewService)
    {      
        settingBox.setAttribute("newServiceAvailable", "true");	
        settingBox.getPartById("idMailBalloon").setAttribute("collapsed", "false");	 
    }
}

//fired from lower layer, creates a panel to current selected tab
component.prototype.needTabPanel = function(nIndex)
{   
    var nSelectedIndex = this.m_gTabControl.GetSelectedIndex(); //get current selected tab
    var panel = this.gadget.scene.createBox("MailPanel");       //create empty panel box

    if (panel == null)
    {
        this.gadget.scene.alert("null", "null");
    }

    var headerTextBox = null;
    var sText = "";

    //create settings panel if selected tab is a settings tab, otherwise create service panel
    if(nIndex == undefined && nSelectedIndex == this.nSettingsTabIndex)
    {
        panel.setAttribute("isSettingTab","true");
        panel.setAttribute("showNoEmails", "false");
        panel.setAttribute("showEmails", "false");
        
        var settingsImagesBox = panel.getPartById("idSettingsImages");     
        var nNumServices = this.m_aServices.length;
        
        for(index=0; index < nNumServices; index++) //loop through all suggested services for user
        {
            var sServiceId = this.m_aServices[index].getId();
            var serviceButton = this.gadget.scene.createBox("ServiceButton");
            serviceButton.setAttribute("id", "ServiceButton");
            serviceButton.setAttribute("serviceId", sServiceId);
            serviceButton.setAttribute("tooltip", this.icqConnector.GetString("email_notifier.add") + " " + this.m_aServices[index].getDisplayName());
            
            
            var serviceImage = serviceButton.getPartById("idServiceImage"); //set image to service icon
            serviceImage.setAttribute("src", this.m_aServices[index].getBigIcon());
          
            
            var bIsNewService = this.m_aServices[index].isNeedPromo();  //check stub to see if service is promoted
            if(bIsNewService)
            {
                serviceButton.setAttribute("isPromoted", "true");
            }
            
            settingsImagesBox.addChild(serviceButton);
        }
        
        //set up services quota error text 
        var oErrorBox = panel.getPartById("idErrorQuotaText1");
        var sErrorText = this.icqConnector.GetString("email_notifier.quotaText").replace("#NUM_SERVICES_QUOTA#", sXTRA_MAX_REGISTERED_SERVICES);
        oErrorBox.setAttribute("value", sErrorText);
         
        this.m_gTabControl.InsertItemPanel(panel, this.nSettingsTabIndex);
    }
    else
    {       
        var sServiceId = this.m_gTabControl.GetTabAttibuteByIndex(nIndex, "serviceId");
        var nServiceIndex = this.getServiceIndexById(sServiceId);
        
        var sServiceStatus = this.m_aServices[nServiceIndex].getStatus();               //get currrent service status
        var sCredentialsState = this.m_aServices[nServiceIndex].getCredentialsState();  //get currrent service Credentials State

        panel.setAttribute("serviceId", sServiceId);
        
        if(sServiceStatus == MEServiceState.kOnline)                        //use service state to display the correct screen (enums)
        {
            panel.setAttribute("isError", "false"); 
            
            var nUnreadMailsCount = this.m_aServices[nServiceIndex].unreadMailesCount();   //get number of current unread emails
            this.m_aServices[nServiceIndex]['unread'] = nUnreadMailsCount;
            
            sText = this.icqConnector.GetString("email_notifier.inbox") + " (" + nUnreadMailsCount + ")";
            sToolTipText = this.icqConnector.GetString("email_notifier.inboxTooltip").replace("#NUM_UNREAD#", nUnreadMailsCount);                     
            headerTextBox = panel.getPartById("idContentServiceText");
            headerTextBox.setAttribute("label", sText);
            headerTextBox.setAttribute("tooltip", sToolTipText);
            
            var oServiceLinksBox = panel.getPartById("serviceLinks");   //query "service xtra" on service's capabillities
            oServiceLinksBox.setAttribute("serviceId", sServiceId);
            
            if(this.m_aServices[nServiceIndex].isCapable('composeNewMail'))
            {
                var oComposeBox = this.scene.createBox("icqImageButton");
                oComposeBox.setAttribute("id", "composeNewMail");
                oComposeBox.setAttribute("icon", "#composeNewMail");
                oComposeBox.setAttribute("tooltip", this.icqConnector.GetString("email_notifier.compose"));             
                oComposeBox.setAttribute("style", "sServiceLinks");
                oServiceLinksBox.addChild(oComposeBox);
            }
            if(this.m_aServices[nServiceIndex].isCapable('addressBook'))
            {
                var oAddressBox = this.scene.createBox("icqImageButton");
                oAddressBox.setAttribute("id", "addressBook");
                oAddressBox.setAttribute("icon", "#addressBook");
                oAddressBox.setAttribute("tooltip", this.icqConnector.GetString("email_notifier.addressBook"));
                oAddressBox.setAttribute("style", "sServiceLinks");
                oServiceLinksBox.addChild(oAddressBox);
            }
            if(this.m_aServices[nServiceIndex].isCapable('calendar'))
            {
                var oCalendarBox = this.scene.createBox("icqImageButton");
                oCalendarBox.setAttribute("id", "calendar");
                oCalendarBox.setAttribute("icon", "#calendar");
                oCalendarBox.setAttribute("tooltip", this.icqConnector.GetString("email_notifier.calendar"));
                oCalendarBox.setAttribute("style", "sServiceLinks");
                oCalendarBox.setStyle("marginBottom", "1");
                oServiceLinksBox.addChild(oCalendarBox);
            }

            //get emails from service
            var oEmailContainerBox = panel.getPartById("idRows");
            
            if(nUnreadMailsCount > 0) 
            {
                var oUnreadEmails = this.m_aServices[nServiceIndex].getUnreadMails();
                var nShowUnreadEmails = oUnreadEmails.length;
                for (index = 0; index < nShowUnreadEmails; index++)
                {
                    var oEmailBox = this.gadget.scene.createBox("EmailMessage");
                    
                    oEmailBox.setAttribute("serviceId", this.m_aServices[nServiceIndex].getId());
                    oEmailBox.setAttribute("emailId", oUnreadEmails[index].getId());
                    oEmailBox.setAttribute("emailSender", oUnreadEmails[index].getSenderName());
                    oEmailBox.setAttribute("emailSubject", oUnreadEmails[index].getTitle());
                    oEmailBox.setAttribute("emailDate", oUnreadEmails[index].getFormatedDate());
                    
                    var bIsAttachment = (oUnreadEmails[index].getHasAttachment())?"true":"false";
                    var bIsMark = (this.m_aServices[nServiceIndex].isCapable('markAsRead'))?"true":"false";
                    var bIsDelete = (this.m_aServices[nServiceIndex].isCapable('deleteMail'))?"true":"false";
                    
                    if(bIsMark == "true" || bIsDelete == "true" )
                    {
                        oEmailBox.setAttribute("isAttachment", bIsAttachment);
                        oEmailBox.setAttribute("isCapableMark", bIsMark);
                        oEmailBox.setAttribute("isCapableDelete", bIsDelete);
                        oEmailBox.setAttribute("isCapability", "false");
                    }
                    else
                    {
                        oEmailBox.setAttribute("isCapability", "true");
                    } 
                    
                
                    oEmailContainerBox.addChild(oEmailBox);
                }
                
                panel.setAttribute("showNoEmails", "false");
                panel.setAttribute("showEmails", "true");
            }
            else        //show empty emails screen
            {
                panel.setAttribute("showNoEmails", "true");
                panel.setAttribute("showEmails", "false");
            }
        }
        else  //if error in connection
        {
            panel.setAttribute("isError", "true");                     
            panel.setAttribute("showNoEmails", "false");
            panel.setAttribute("showEmails", "false");
             
            switch(sCredentialsState)
            {
                case MECredentialsState.kNotValid:   
                                                    //show connection error panel                     
                                                    var oConnectErrorBox = panel.getPartById("idErrorConnect");
                                                    var oConnectErrorText = panel.getPartById("idErrorConnectText");
                                                    var sErrorText = this.icqConnector.GetString("email_notifier.unableConnect") + " " + sServiceId;
                                                    oConnectErrorText.setAttribute("value", sErrorText);
                                                    oConnectErrorBox.setAttribute("collapsed", "false");     
                                                    break;
                                                    
                case MECredentialsState.kExpaierd:
                                                    //show expiration error panel
                                                    var oExpiredErrorBox = panel.getPartById("idErrorExpired");
                                                    var oExpiredErrorText = panel.getPartById("idErrorExpiredTextPost");
                                                    var oExpiredErrorLink = panel.getPartById("idErrorContentLink");
                                                    oExpiredErrorLink.setAttribute("serviceId", sServiceId);
                                                    
                                                    var sErrorText = oExpiredErrorText.getAttribute("value").replace("#SERVICE_NAME#", "" + sServiceId);     
                                                    oExpiredErrorText.setAttribute("value", sErrorText);
                                                     
                                                    oExpiredErrorBox.setAttribute("collapsed", "false");                
                                                    break;
                          
               case MECredentialsState.kNotSet:     
                                                    var nServiceIndex = this.getServiceIndexById(sServiceId);
                                                    
                                                    if(this.m_aServices[nServiceIndex].needLspTab())
                                                    {
                                                        var oLspBox = panel.getPartById("idLspConnectCont");
                                                        var oHeaderBox = panel.getPartById("idContentHeader");
                                                        var oContentBox = panel.getPartById("idMainContent");
                                                        
                                                        panel.setAttribute("serviceIcon", this.m_aServices[nServiceIndex].getSmallIcon());
                                                        
                                                        var sMsg = this.icqConnector.GetString("email_notifier.lspText").replace("#STRING#", sServiceId);
                                                        panel.setAttribute("lspText", sMsg);

                                                        oLspBox.setAttribute("collapsed", "false");
                                                        oHeaderBox.setAttribute("collapsed", "true");
                                                        oContentBox.setAttribute("collapsed", "true");
                                                    }
                                                    break;
                                                    
               default:                             //show preloader panel
                                                    var oPrelaoderBox = panel.getPartById("idPreloader");
                                                    var oPrelaoderText = panel.getPartById("idPreloaderText");
                                                    var sMsg = this.icqConnector.GetString("email_notifier.connecting") + " " + sServiceId;
                                                    
                                                    oPrelaoderText.setAttribute("value", sMsg);  
                                                    oPrelaoderBox.setAttribute("collapsed", "false");                                      
                                                    break;                                                                    
            }
        }
        
        var nInsertIndex;
        
        if(nIndex != undefined)
        {
            nInsertIndex = nIndex;
        }
        else
        {
            nInsertIndex = nSelectedIndex;
        }
        
        this.m_gTabControl.InsertItemPanel(panel, nInsertIndex);
    } 
    
    
}

component.prototype.updateTabBubble = function(sServiceId, nNumUnread)
{ 
    var nSelectedIndex = this.m_gTabControl.GetSelectedIndex();
    var nServiceIndexInTabs = this.m_gTabControl.GetHeaderIndexByKey("serviceId", sServiceId); 
     
    if(nSelectedIndex != nServiceIndexInTabs)
    {
        var oBox = this.m_gTabControl.GetHeaderByKey("serviceId", sServiceId);
        if (nNumUnread === 0)
        {
            oBox.getPartById("idMailBalloon").setAttribute("collapsed", "true");        
        }
        else
        {
            oBox.setAttribute("msgCount", nNumUnread);
            oBox.getPartById("idMailBalloon").setAttribute("collapsed", "false");        
        }
    }
}

//event fired from controller in case of a tab selection
component.prototype.selTabChanged = function()
{   
    var nSelectedTabIndex = this.m_gTabControl.GetSelectedIndex();
    var oSelectedTab = this.m_gTabControl.GetHeader(nSelectedTabIndex);
    var bIsSettingTab = oSelectedTab.getAttribute("isSettingsTab");
    var bIsBalloonUp;
    
    //suppress balloon: differ between settings tab and service tab
    if(bIsSettingTab == "true")
    {
        bIsBalloonUp = oSelectedTab.getAttribute("newServiceAvailable") == "true";
    }
    else
    {
        bIsBalloonUp = oSelectedTab.getAttribute("msgCount") != "";
        this.icqConnector.Owner.OwnerStatistic.Collection.IncrementMapSecondDItem(STAT_CATEGORY_ID, USAGE_STAT_ID, oSelectedTab.getAttribute("serviceId"), oSelectedTab.getAttribute("serviceId")); 
    }
    
    //TODO - check support
    //suppress tab balloon if necessary
    if(bIsBalloonUp)
    {
        oSelectedTab.getPartById("idMailBalloon").setAttribute("collapsed", "true");
        //(nSelectedTabIndex == this.nSettingsTabIndex)?.suppressNewServiceBallon():'';
    }

    // Update DB with last selected tab
    if (g_oUiApi.isReady())
        g_oUiApi.setLastTab(oSelectedTab.getAttribute("serviceId"));
}

//currently catches any "command" event from lower layer
//uses the target box to determine the origin of the command and acts accordingly
component.prototype.onTabCommand = function()
{
    var currentBox = shell.currentEvent.targetBox;
    var boxID = currentBox.getAttribute("id");
    
    switch(boxID)
    {
        case "composeNewMail":            
        case "addressBook":                                
        case "calendar":
                                        var sServiceId = currentBox.parent.getAttribute("serviceId"); 
                                        var nServiceIndex = this.getServiceIndexById(sServiceId);
                                        
                                        if(boxID == "composeNewMail")
                                        {
                                            this.m_aServices[nServiceIndex].composeMail();
                                        }
                                        else if(boxID == "addressBook")
                                        {
                                            this.m_aServices[nServiceIndex].openAddressBook();
                                        }
                                        else if(boxID == "calendar")
                                        {
                                            this.m_aServices[nServiceIndex].openCalendar();
                                        }
                                        break;   
        case "idReadLinkText":
        case "idMarkLinkText":
        case "idDeleteLinkText":
                                        var sServiceId = currentBox.getAttribute("serviceId");
                                        var nEmailId = currentBox.getAttribute("emailId");
                                        var nServiceIndex = this.getServiceIndexById(sServiceId);
                                        
                                        if(boxID == "idReadLinkText")
                                        {
                                            var bIsMarked = currentBox.getAttribute("isMarked");
                                            this.m_aServices[nServiceIndex].readMail(nEmailId);
                                            (bIsMarked != "true")?this.markMail(sServiceId, nServiceIndex, nEmailId):'';
                                        }
                                        else if(boxID == "idMarkLinkText")
                                        {   
                                            var bIsMarked = currentBox.getAttribute("isMarked");
                                            this.m_aServices[nServiceIndex].markAsRead(nEmailId);
                                            (bIsMarked != "true")?this.markMail(sServiceId, nServiceIndex, nEmailId):'';
                                        }
                                        else if(boxID == "idDeleteLinkText")
                                        {
                                            this.m_aServices[nServiceIndex].deleteMail(nEmailId);
                                            this.deleteMail(sServiceId, nServiceIndex, nEmailId);
                                        }
                                        break;
        case "ServiceButton":
                                        var nRegisteredServices = g_oUiApi.registerdServicesCount();
                                        if(nRegisteredServices == sXTRA_MAX_REGISTERED_SERVICES)
                                        {
                                            this.showServicesQuotaError();
                                            break;
                                        } 
        case "idLspButton":
        case "idErrorContentLink":                                                             
        case "idErrorContentButton":
                                        var sServiceId = currentBox.getAttribute("serviceId");
                                        g_oUiApi.registerToService(sServiceId);
                                        break;
        case "idContentServiceText":
                                        var sServiceId = currentBox.getAttribute("serviceId");
                                        var nServiceIndex = this.getServiceIndexById(sServiceId);
                                        this.m_aServices[nServiceIndex].openInbox();
                                        break;
        case "idErrorConnectButton":
                                        var sServiceId = currentBox.getAttribute("serviceId");
                                        var nServiceIndex = this.getServiceIndexById(sServiceId);
                                        this.m_aServices[nServiceIndex].retryToConnect();
                                        break;
        case "idErrorQuotaImage":
                                        this.collapseError();
                                        break;
    }
}

component.prototype.markMail = function(sServiceId, nServiceIndex, nEmailId)
{
    var nPanelIndex = this.m_gTabControl.GetHeaderIndexByKey("serviceId", sServiceId);
    var oServicePanelBox = this.m_gTabControl.GetPanel(nPanelIndex);
    var oServiceEmailsBox = oServicePanelBox.getPartById("idRows");
    var bFound = false;
    
    for(index=0; !bFound && index < oServiceEmailsBox.childCount ;index++)
    {
        var oEmail = oServiceEmailsBox.getChildAt(index);
        
        if(oEmail.getAttribute("emailId") == nEmailId)
        {  
            var oInboxBox = oServicePanelBox.getPartById("idContentServiceText");   //update current unread emails number
            this.updateInbox(oInboxBox, nServiceIndex);
            oEmail.setAttribute("isMarked", "true");
            bFound = true;
        }
    }
}

component.prototype.deleteMail = function(sServiceId, nServiceIndex, nEmailId)
{
    var nPanelIndex = this.m_gTabControl.GetHeaderIndexByKey("serviceId", sServiceId);
    var oServicePanelBox = this.m_gTabControl.GetPanel(nPanelIndex);
    var oServiceEmailsBox = oServicePanelBox.getPartById("idRows");
    var bFound = false;
    
    for(index=0; !bFound && index < oServiceEmailsBox.childCount ;index++)
    {
        var oEmail = oServiceEmailsBox.getChildAt(index);
        
        if(oEmail.getAttribute("emailId") == nEmailId)
        {
            var bIsMarked = oEmail.getAttribute("isMarked");
            oServiceEmailsBox.deleteChild(oEmail);  //remove email from the list
            
            var oInboxBox = oServicePanelBox.getPartById("idContentServiceText");   //update current unread emails number
            (bIsMarked == "false")?this.updateInbox(oInboxBox, nServiceIndex):'';
            
            if(oServiceEmailsBox.childCount == 0) //if no emails left switch screens
            {
                oServicePanelBox.setAttribute("showNoEmails", "true");
                oServicePanelBox.setAttribute("showEmails", "false");
            } 
            bFound = true;
        }
    }
}

component.prototype.updateInbox = function(oBox, nServiceIndex)
{
    this.m_aServices[nServiceIndex]['unread']--;
    sText = this.icqConnector.GetString("email_notifier.inbox") + " (" + this.m_aServices[nServiceIndex]['unread'] + ")";
    sToolTipText = this.icqConnector.GetString("email_notifier.inboxTooltip").replace("#NUM_UNREAD#", this.m_aServices[nServiceIndex]['unread']);                     
    oBox.setAttribute("label", sText);
    oBox.setAttribute("tooltip", sToolTipText);
}

component.prototype.openEmail = function()
{
    var currentBox = shell.currentEvent.targetBox;
    var boxID = currentBox.getAttribute("id");
    
    switch(boxID)
    {
        case "idEmailSender":
        case "idEmailSubject":
        case "idEmailDate":
                                var sServiceId = currentBox.getAttribute("serviceId");
                                var nEmailId = currentBox.getAttribute("emailId");
                                var nServiceIndex = this.getServiceIndexById(sServiceId);
                                var bIsMarked = currentBox.getAttribute("isMarked");
                                 
                                this.m_aServices[nServiceIndex].readMail(nEmailId); 
                                (bIsMarked != "true")?this.markMail(sServiceId, nServiceIndex, nEmailId):'';
                                break;
        default:
                                break;    
    }
}

//get the index value of a service in the class member ARRAY "m_aServices" using service id as a key
component.prototype.getServiceIndexById = function(serviceId)
{
    var nNumServices = this.m_aServices.length;
    var index = 0;
    var bIsFound = false;
    var sCurrentService;
    
    while(index < nNumServices)     
    {
        sCurrentService = this.m_aServices[index].getId();
        if(sCurrentService == serviceId){return index;}
        index++;
    }
   
    return -1;
}

//removes a service from the UI, use controller API's, first get the index in tabs than remove
component.prototype.removeService = function(sServiceId)
{
    var nServiceIndex = this.m_gTabControl.GetHeaderIndexByKey("serviceId", sServiceId);
    
    //if balloon is up, collapsed it first
    var oServiceTab = this.m_gTabControl.GetHeader(nServiceIndex);
    (oServiceTab.getAttribute("msgCount")!="")?oServiceTab.setAttribute("msgCount", ""):'';
    
    this.nSettingsTabIndex--;
    this.m_gTabControl.RemoveTab(nServiceIndex);
}

//adds a service tab to the UI
component.prototype.addServiceTab = function(sServiceId)
{
    var nServiceIndex = this.getServiceIndexById(sServiceId);
	var box = this.gadget.scene.createBox("MailHeader");
	this.nSettingsTabIndex++;

	box.setAttribute("serviceId", sServiceId);
	box.setAttribute("tooltip", this.m_aServices[nServiceIndex].getDisplayName());
	box.setAttribute("serviceIcon", this.m_aServices[nServiceIndex].getSmallIcon()); //get service tab icon
	
	var sServiceStatus = this.m_aServices[nServiceIndex].getStatus();               //get currrent service status
	var sCredentialsState = this.m_aServices[nServiceIndex].getCredentialsState();  //get currrent service Credentials State

    if(  sServiceStatus == MEServiceState.kOnline && 
       ( sCredentialsState == MECredentialsState.kSet || sCredentialsState == MECredentialsState.kValid ) )
    {
        //if unread emails exist, raise bubble for tab    
        var nNumUnreadEmails = this.m_aServices[nServiceIndex].getBalloonCount(); 

        if(nNumUnreadEmails > 0) 
        {
            box.setAttribute("msgCount", nNumUnreadEmails);
            box.getPartById("idMailBalloon").setAttribute("collapsed", "false");        
        }
    }
	else
	{
	    switch(sCredentialsState)
	    {
	        case MECredentialsState.kNotValid:
	        case MECredentialsState.kExpaierd:
	                                            this.updateTabBubble(sServiceId, sXTRA_ERROR_BALLOON_SYMBOL);	                                                              
                                                break;
            default:
                                                break;
	    }		   
	}
	 
    this.m_gTabControl.InsertTabAt(box, 0);
}
               
//displays an error message in case a 4th service registration
component.prototype.showServicesQuotaError = function()
{
    var oErrorBox = this.gadget.getPartById("idServicesQuotaError");
    oErrorBox.setAttribute("collapsed", "false");
}

//removes an error message in case a 4th service registration
component.prototype.collapseError = function()
{
    var oErrorBox = this.gadget.getPartById("idServicesQuotaError");
    oErrorBox.setAttribute("collapsed", "true");
}
