/* ---------------------------------------------------------
PUMK XTRA JAVASCRIPT IMPLEMINTATION
------------------------------------------------------------
This file is responsible for the behavior of the PUMK xtra.
--------------------------------------------------------- */

/* Constants */
var C_DISPLAY_NONE = "none";
var C_DISPLAY_BLOCK = "block";

var C_IMAGE_PREFIX = "../../theme/pumk/";
var C_PUMK_XTRA_HOST_NAME = "xtra_pumk";

var C_GET_DEVILS_URL = "http://api.icq.net/expressions/get2?f=json&type=buddyIcon";

var C_REQUEST_TIMEOUT = 30000;

var icqEx = {};             // Extended ICQ API object
var buttonHoverColor = {};  // Default client hover color for buttons

MEPumkApi =
{
    kJapi: "ja",
    kMdir: "md",
    kDevils: "ex"
};

MEPumkProtocol =
{
    kJson: "1",
    kHttp: "2"
};

MEPumkState =
{
    kStart: 1,
    kWaitForFriender: 2,
    kWaitForMemberDir: 3,
    kWaitForDevils: 4,
    kFinalResults: 5,
    kError: 6,
    kClosing: 7
};

MEPumkError =
{
    kInvalidJsonFormat: '760'
};

var g_aAjaxCalls = []; // Container for pending ajax calls.find Should be released during shutdown
var kErrorTimeout = '408';

function suggestedUser(jsonObjUser) {
    /* ---------------------------------------------------------
    Creates a single suggested user
    Input:  none.
    Output: void();
    --------------------------------------------------------- */
    this.sn = jsonObjUser.sn;
    this.mutualFriends = jsonObjUser.mutualFriends;
    this.displayName = {};
    this.firstLine = {};
    this.secondLine = {};
    this.gender = '';
    this.onPage = 0;
    this.isAdded = true; // We will set this flag to false only if we get minInfo for this user
}

function pumk(maxRequestedResults, maxPages, maxUsersPerPage, xtraHostName, clickCallbackObj) {
    // Init variables
    this._arrSuggestedUsers = [];       // Screenname indexed suggested users. Used to process mdir response
    this._arrSortedSuggestedUsers = []; // Sorted array, as returned from server API
    this._nSuggestions = 0;             // Number of total suggestions
    this._nAddedUsers = 0;              // Number of users which were already added
    this._nTotalPages = 0;              // N
    this._nCurrentPage = 0;
    this._timeoutId = null;
    this._errorMode = false;
    this._state = MEPumkState.kStart;
    this._closing = false;

    // Needed objects
    this._icqConnector = XtrazConnector.icqConnector;
    this._urlService = this._icqConnector.Owner.Url;
    this._statCollection = this._icqConnector.Owner.OwnerStatistic.Collection;

    // Report clicks to container if available
    this._clickCallbackObj = null;
    if (clickCallbackObj)
        this._clickCallbackObj = clickCallbackObj;

    // Initialize the extended icq API object, and get the current client theme
    icqEx = new icqConnectorEx(this._icqConnector);
    buttonHoverColor = icqEx.getClientColor(MEClientColor.kColor_ButtonHover, '#75DE72');

    // Save the containing Xtra name for statistics reporting. If we are contained within another Xtra, do not attach to shut down event
    this._xtraHostName = xtraHostName;
    if (xtraHostName == C_PUMK_XTRA_HOST_NAME) {
        this._icqConnector.AttachEvent("ShutDown", OnXtraClosed);
        OnXtraClosed.owner = this;
    }

    this._skinChangedEvent = this._icqConnector.App.AttachEvent("OnSkinChanged", OnSkinChanged);
    this._userAddedEvent = this._icqConnector.Owner.AttachEvent("OnUserAdded", OnUserAdded);
    OnUserAdded.owner = this;

    // init max values
    this._maxRequestedResults = maxRequestedResults;
    this._maxPages = maxPages;
    this._maxUsersPerPage = maxUsersPerPage;

    /* Collect data of the client */
    this._currentLanguage = this._icqConnector.GetIMClientData("LANG_ID");
    this._dir = ((this._currentLanguage == 'he-il') || (this._currentLanguage == 'ar-sa')) ? 'rtl' : 'ltr';

    getSuggestions.owner = this;
    var sUrl = "http://japi.icq.com/friender/getSuggestedFriends?f=json&nToGet=" + this._maxRequestedResults;
    this.runUrl(sUrl, getSuggestions, true);
    this._state = MEPumkState.kWaitForFriender;
}

pumk.prototype.getHTML = function() {
    var sHtml =
    '<div id="d-pumk-container" class="d-pumk-container">'
        + '<div id="d-pumk-preloader" class="pumk-preload">'
	        + '<img' + getImageSrc("connecting-anim.gif") + '/>'
		    + '<div id="pumk-preload_text" class="pumk-preloader-text">' + this.getString("pumk.preloaderText") + '</div>'
	    + '</div>'
	    + '<div id="d-pumk-main" class="d-pumk-main" style="display:none">'
	    + '</div>'
        + '<div id="d-pumk-pages" class="d-pumk-pc">'
            + '<img' + getImageSrc("arrow_left_gray.gif") + 'id="id-left-arrow" class="d-pumk-arrow" onclick="javascript:objPumk.onGotoPrevPage();"></img>'
            + '<span id="s-pumk-pages" class="s-pumk-pages"></span>'
            + '<img' + getImageSrc("arrow_right_gray.gif") + 'id="id-right-arrow" class="d-pumk-arrow" onclick="javascript:objPumk.onGotoNextPage();"></img>'
        + '</div>'
    + '</div>';

    return sHtml;
}

function getSuggestions(oAjax) {
    var objThis = arguments.callee.owner;
    objThis.getSuggestions(oAjax);
}

pumk.prototype.getSuggestions = function(oAjax) {
    this.cancelRequestTimer();
    if (this._errorMode === true || this._closing == true)
        return;

    response = this.parseAjax(oAjax, MEPumkApi.kJapi);
    if (response == null)
        return;

    if (response.statusCode == 200) {
        var suggestions = response.data.suggestions;
        this._arrSuggestedUsers = new Array();
        this._sortedSuggestedUsers = new Array();
        if (typeof (suggestions) == 'undefined' || suggestions.length == 0) {
            this.showNoResults();
        }
        else {
            var nSuggestions = suggestions.length;
            this._nSuggestions = 0;
            for (var i = 0; i < nSuggestions; ++i) {
                // If the user already exists in the CL, do not show it
                if (this._icqConnector.Owner.GetUser(suggestions[i].sn, false) == null) {
                    var sUser = new suggestedUser(suggestions[i]);
                    this._arrSortedSuggestedUsers[this._nSuggestions] = sUser; // Some i's maybe skipped. Add user at the end of the array
                    this._arrSuggestedUsers[suggestions[i].sn] = sUser;
                    this._nSuggestions++;
                    this._nAddedUsers++;
                }
            }
            this.getUsersMinInfo();
        }
    }
    else {
        this.onError(MEPumkApi.kJapi, MEPumkProtocol.kJson, response.statusCode);
    }
}

pumk.prototype.getUsersMinInfo = function() {
    minInfoResponse.owner = this;

    var sUrl = "http://api.icq.net/memberDir/get?f=json&r=" +
                Math.random() + "&locale=" + this._currentLanguage + "&infoLevel=min";

    for (var i in this._arrSuggestedUsers)
        sUrl += "&t=" + this._arrSuggestedUsers[i].sn;

    this.runUrl(sUrl, minInfoResponse, true);
    this._state = MEPumkState.kWaitForMemberDir;
}

pumk.prototype.requestDevils = function(sDevilsUrl) {
    devilsResponse.owner = this;

    sDevilsUrl += "&r=" + Math.random();
    this.runUrl(sDevilsUrl, devilsResponse, false);

    this._state = MEPumkState.kWaitForDevils;
}

function minInfoResponse(oAjax) {
    var objThis = arguments.callee.owner;
    objThis.minInfoResponse(oAjax);
}

pumk.prototype.minInfoResponse = function(oAjax) {
    this.cancelRequestTimer();
    if (this._errorMode === true || this._closing == true)
        return;

    response = this.parseAjax(oAjax, MEPumkApi.kMdir);
    if (response == null)
        return;
        
    if (response.statusCode == 200) {
        var infoArray = response.data.infoArray;

        var sn, firstName, lastName, displayName;
        var nInfos = infoArray.length;
        for (var i = 0; i < nInfos; ++i) {

            var firstLine = '';
            var secondLine = ' ';
            sn = infoArray[i]['profile']['aimId'];

            gender = isSet(infoArray[i]['profile']['gender']) ? infoArray[i]['profile']['gender'] : '';
            firstName = isSet(infoArray[i]['profile']['firstName']) ? infoArray[i]['profile']['firstName'] : '';
            lastName = isSet(infoArray[i]['profile']['lastName']) ? infoArray[i]['profile']['lastName'] : '';
            friendlyName = isSet(infoArray[i]['profile']['friendlyName']) ? infoArray[i]['profile']['friendlyName'] : '';

            firstName = trim(firstName).htmlEntities();
            lastName = trim(lastName).htmlEntities();
            friendlyName = trim(friendlyName).htmlEntities();

            // Show first name + last name on the first line. Show the friendly name on the second line.
            // If there is no first name or last name, show the friendly name on the first line
            if (firstName == '' && lastName == '') {
                firstLine = friendlyName;
            }
            else {
                if (firstName != '')
                    firstLine = firstName + ' ';
                firstLine += lastName;

                secondLine = friendlyName;
            }

            if (firstLine != '') {
                // If we have something to display, mark the user as not-added. This will make sure it can be shown in the UI
                // If there is nothing to display, do not show the user. If there is no minInfo response for this user, do not show this user.
                this._arrSuggestedUsers[sn].isAdded = false;
                this._nAddedUsers--;

                if (secondLine == '') // Don't allow a "hole" in the UI
                    secondLine = '&nbsp;';

                if (gender != 'female' && gender != 'male')
                    gender = 'nogender';

                this._arrSuggestedUsers[sn].firstLine = firstLine;
                this._arrSuggestedUsers[sn].secondLine = secondLine != firstLine ? secondLine : '&nbsp;';

                this._arrSuggestedUsers[sn].displayName = friendlyName;
                this._arrSuggestedUsers[sn].gender = gender;
            }
        }

        this.calcPages();
        this.showSuggestedFriends();
    }
    else {
        this.onError(MEPumkApi.kMdir, MEPumkProtocol.kJson, response.statusCode);
    }

}

function devilsResponse(oAjax) {
    var objThis = arguments.callee.owner;
    objThis.devilsResponse(oAjax);
}

pumk.prototype.devilsResponse = function(oAjax) {
    this.cancelRequestTimer();
    if (this._errorMode === true || this._closing == true)
        return;

    response = this.parseAjax(oAjax, MEPumkApi.kDevils);
    if (response == null)
        return;

    if (response.statusCode == 200) {
        var targets = response.data.targets;

        var nTargets = targets.length;
        for (var i = 0; i < nTargets; ++i) {
            if (targets[i].expressions && targets[i].expressions.length) {
                var img = document.getElementById("img_" + targets[i].name);
                if (img) img.src = targets[i].expressions[0].url;
            }
        }
    }
    else {
        // Unable to get devils is not defined as a critical error.kInvalidJsonFormat Do not call onError. do not move to error screen
        this.reportError(MEPumkApi.kDevils, MEPumkProtocol.kJson, response.statusCode);
        this.cancelRequestTimer();
        //objPumk.showServiceError();
    }
}

String.prototype.htmlEntities = function() {
    return this.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;");
}

pumk.prototype.showSuggestedFriends = function() {
    this.showPage(1);
    this.hidePreloader();
}

pumk.prototype.showPage = function(nPageIndex) {

    this._nCurrentPage = nPageIndex;

    var mainDiv = document.getElementById("d-pumk-main");
    this.addPageHtml(mainDiv, nPageIndex);
    this.showPagesIndicator();

    //mainDiv.innerHTML = sHtml;
    mainDiv.style.display = C_DISPLAY_BLOCK;
}

pumk.prototype.addPageHtml = function(mainDiv, nPage) {
    // First remove all the children of the previous page, but always keep the 'More added' div.
    var children = mainDiv.children;
    while (children.length) {
        mainDiv.removeChild(children[0]);
    }

    // Prepare url for asking devil images
    var sDevilsUrl = C_GET_DEVILS_URL;

    var nStart = (nPage - 1) * this._maxUsersPerPage;
    var nAdded = 0;

    var nIndex = nStart;
    var nAddedIndex = 0;
    while (nIndex >= 0 && nIndex < this._nSuggestions && nAdded < this._maxUsersPerPage && nAddedIndex >= 0) {
        nAddedIndex = this.addNextUser(mainDiv, nIndex);
        if (nAddedIndex >= 0) {
            nAdded++;
            nIndex = nAddedIndex + 1;

            sDevilsUrl += "&t=" + this._arrSortedSuggestedUsers[nAddedIndex].sn; // Continue building devils URL
        }
    }

    // If we can't fill the page, show the search more link
    if (nAdded < this._maxUsersPerPage) {
        this.addFindMoreFriends();
    }

    if (nAdded > 0) {
        this.requestDevils(sDevilsUrl);
    }
}

pumk.prototype.addNextUser = function(mainDiv, nIndex) {
    // If all users were already added, there are no more users for addition
    if (this._nSuggestions == this._nAddedUsers) {
        return -1;
    }

    while (nIndex < this._nSuggestions) {
        var sugUser = this._arrSortedSuggestedUsers[nIndex];
        // Add user only if it was not deleted, and if it is relevant to the current page
        if (sugUser.isAdded == false && (sugUser.onPage == 0 || sugUser.onPage >= this._nCurrentPage)) {
            var newDiv = mainDiv.appendChild(document.createElement("div"));
            newDiv.className = "d-pumk-result";
            newDiv.id = sugUser.sn;
            newDiv.innerHTML = this.getUserDiv(sugUser);
            sugUser.onPage = this._nCurrentPage;            // Update the user's page
            return nIndex;
        }
        else {
            nIndex++;
        }
    }

    return -1;
}

pumk.prototype.getUserDiv = function(sugUser) {
    var sHtml = '<img id="img_' + sugUser.sn + '" class="i-pumk-devil" onclick="javascript:objPumk.onOpenProfile(this);" sn="' + sugUser.sn +
                     '" src="http://c.icq.com/people/img/' + sugUser.gender + '100.jpg" onerror="this.src=\'http://c.icq.com/people/img/' + sugUser.gender + '100.jpg\'" />';
    // Add lines
    sHtml += '<div class="d-pumk-line1" onclick="javascript:objPumk.onOpenProfile(this);" sn="' + sugUser.sn + '">' + sugUser.firstLine + '</div>';
    if (sugUser.secondLine)
        sHtml += '<div class="d-pumk-line2">' + sugUser.secondLine + '</div>';
    // Add score
    if (sugUser.mutualFriends > 0)
        sHtml += '<div class="d-pumk-score">' + sugUser.mutualFriends + ' ' + this.getString("pumk.mutualFriends") + '</div>';
    else
        sHtml += '<div class="d-pumk-score">&nbsp;&nbsp;</div>';
    // Add add link
    sHtml += '<div class="d-pumk-addlink">' +
                    '<img class="i-pumk-addlink"' + getImageSrc("add_user_small.png") + '/>' +
                    '<div class="s-pumk-addlink" onclick="javascript:objPumk.onAddUser(this);" sn="' + sugUser.sn + '">' + this.getString("pumk.addAsFriend") + '</div>' +
             '</div>';
    return sHtml;
}

pumk.prototype.calcPages = function() {
    // If all users were already added, there are no more pages
    if (this._nSuggestions == this._nAddedUsers) {
        this._nTotalPages = 0;
        return;
    }

    var nRemainingUsers = this._nSuggestions - this._nAddedUsers;
    var nTempTotal = Math.floor(nRemainingUsers / this._maxUsersPerPage);
    if (nRemainingUsers % this._maxUsersPerPage)
        nTempTotal += 1;

    this._nTotalPages = nTempTotal < this._maxPages ? nTempTotal : this._maxPages;
}

pumk.prototype.showPagesIndicator = function() {
    // If the first page is full, calculate the number of pages and show it in the bottom
    var pagesDiv = document.getElementById("d-pumk-pages");
    var pagesSpan = document.getElementById("s-pumk-pages");

    if (this._nTotalPages > 1) {
        pagesDiv.style.display = C_DISPLAY_BLOCK;
        pagesSpan.innerHTML = '<b>' + this._nCurrentPage + '</b>/' + this._nTotalPages;

        // Disable left or right arrows
        var nextArrowId = this._dir == 'rtl' ? "id-left-arrow" : "id-right-arrow";
        var prevArrowId = this._dir == 'rtl' ? "id-right-arrow" : "id-left-arrow";
        document.getElementById(prevArrowId).className = this._nCurrentPage == 1 ? "d-pumk-disabled-arrow" : "d-pumk-arrow";
        document.getElementById(nextArrowId).className = this._nCurrentPage == this._nTotalPages ? "d-pumk-disabled-arrow" : "d-pumk-arrow";
    }
    else {
        pagesDiv.style.display = C_DISPLAY_NONE;
    }
}

pumk.prototype.showNoResults = function() {
    this.hidePreloader();

    this.setContainerHtml(
        '<div id="d-pumk-nofriends" class="d-pumk-nf">'
            + '<img' + getImageSrc("add_user_large.png") + 'class="i-pumk-nf ie6png" />'
	        + '<div class="d-pumk-nf-t">' + this.getString("pumk.noSuggestions") + '</div>'
	        + this.addButton("d-pumk-nf-b", "pumk.findOnIcq", "objPumk.searchIcq()")
	    + '</div>'
	);
}

pumk.prototype.showServiceError = function() {
    this.hidePreloader();

    this.setContainerHtml(
        '<div id="d-pumk-error" class="d-pumk-error">'
            + '<img' + getImageSrc("add_user_error.png") + 'class="i-pumk-error  ie6png"/>'
	        + '<div class="d-pumk-error-t">' + this.getString("pumk.serviceError") + '</div>'
	        + this.addButton("d-pumk-error-add", "pumk.findOnIcq", "objPumk.searchIcq()")
        + '</div>'
    )

    this._state = MEPumkState.kError;
}

pumk.prototype.addFindMoreFriends = function() {
    var mainDiv = document.getElementById("d-pumk-main");

    var moreDiv = mainDiv.appendChild(document.createElement("div"));
    moreDiv.id = "d-pumk-main-more";
    moreDiv.className = "d-pumk-more";
    moreDiv.innerHTML = '<img' + getImageSrc("add_user_large.png") + 'class="i-pumk-more" />'
                        + this.addButton("b-pumk-more", "pumk.findMoreFriends", "objPumk.searchIcq()");
}

pumk.prototype.addButton = function(className, resId, callbackName) {
    var strRet = '<div class="' + className + ' d-pumk-button"' +
            ' onClick="javascript:' + callbackName + ';"' +
            ' onMouseOver="btnMouseOver(this);" onMouseOut="btnMouseOut(this);" onMouseUp="btnMouseUp(this);" onMouseDown="btnMouseDown(this);">' +
            '<div>' + this.getString(resId) + '</div></div>';

    return strRet;
}

pumk.prototype.hidePreloader = function() {
    objPreloader = document.getElementById("d-pumk-preloader");
    objPreloader.style.display = C_DISPLAY_NONE;
}

// Class helper functions
pumk.prototype.runUrl = function(sUrl, callback, bSign) {
    if (bSign == true) {
        var vPseudo;
        sUrl = this._urlService.SignUrl(sUrl, vPseudo);
    }

	var oAjax = null;
	if (window.XMLHttpRequest) {
	  // If IE7, Mozilla, Safari, and so on: Use native object.
	  oAjax = new XMLHttpRequest();
	}
	else
	{
	  if (window.ActiveXObject) {
		 // ...otherwise, use the ActiveX control for IE5.x and IE6.
		 oAjax = new ActiveXObject('MSXML2.XMLHTTP.3.0');
	  }
	}
    oAjax.open("GET", sUrl, true);
    oAjax.send('');
    oAjax.onreadystatechange = makeAjaxCallback(oAjax, null, callback);

    // Activate request timeout timer (60 seconds of HTTP is too much)
    this.cancelRequestTimer();
    this._timeoutId = setTimeout("onRequestTimeout(objPumk);", C_REQUEST_TIMEOUT);
}

pumk.prototype.cancelRequestTimer = function() {
    if (this._timeoutId !== null) {
        clearTimeout(this._timeoutId);
        this._timeoutId = null;
    }
}


function onRequestTimeout(objPumk) {
    objPumk.onTimeout();
}

pumk.prototype.onTimeout = function() {
    this.cancelRequestTimer();
    var apiCode = '';

    if (this._state == MEPumkState.kWaitForFriender)
        apiCode = MEPumkApi.kJapi;
    else if (this._state == MEPumkState.kWaitForMemberDir)
        apiCode = MEPumkApi.kMdir;
    else if (this._state == MEPumkState.kWaitForDevils)
        apiCode = MEPumkApi.kDevils;

    if (apiCode.length)
        this.onError(apiCode, MEPumkProtocol.kHttp, kErrorTimeout);
}

pumk.prototype.getString = function(resId) {
    var ret = this._icqConnector.GetString(resId);
    return ret;
}

pumk.prototype.reportClick = function() {
    if (this._clickCallbackObj)
        this._clickCallbackObj.pumkClicked();
}

pumk.prototype.setContainerHtml = function(sInnerHtml) {
    var divContainer = document.getElementById("d-pumk-container");
    divContainer.innerHTML = sInnerHtml;
}

pumk.prototype.searchIcq = function() {
    icqEx.openSearch();
    this.reportClick();
}

// Actions
pumk.prototype.onGotoNextPage = function(bInner) {
    if (bInner == null && this._dir == 'rtl') {
        this.onGotoPrevPage(false);
        return;
    }

    if (this._nCurrentPage < this._nTotalPages) {
        this.showPage(this._nCurrentPage + 1);
    }
}

pumk.prototype.onGotoPrevPage = function(bInner) {
    if (bInner == null && this._dir == 'rtl') {
        this.onGotoNextPage(false);
        return;
    }

    if (this._nCurrentPage > 1) {
        this.showPage(this._nCurrentPage + -1);
    }
}

pumk.prototype.onOpenProfile = function(objLink) {
    icqEx.openUserProfile(objLink.sn, this._arrSuggestedUsers[objLink.sn].displayName);

    // Report open profile statistics
    this._statCollection.IncrementMapSecondDItem("CUsage", "SFSV", this._xtraHostName, "1");

    this.reportClick();
}

pumk.prototype.onAddUser = function(objLink) {
    icqEx.openAddUser(objLink.sn, this._arrSuggestedUsers[objLink.sn].displayName);
}

// Client events handlers
function OnXtraClosed(reason) {
    /* ---------------------------------------------------------
    Input:  none.
    Output: void();
    --------------------------------------------------------- */
    var objThis = arguments.callee.owner;
    objThis.OnXtraClosed(reason);

    icq = null;
    XtrazConnector.icqConnector = null;
}

pumk.prototype.OnXtraClosed = function(reason) {
    this.cancelRequestTimer();

    this._closing = true;
    this._icqConnector = null;
    this._clickCallbackObj = null;
    this._statCollection = null;
    this._skinChangedEvent = null;
    this._userAddedEvent = null;

    // Destroy global extended ICQ API object
    icqEx = null;

    // Release ajax objects if necessary
    // clean open ajax calls
    var nLength = g_aAjaxCalls.length;
    for (var i = nLength - 1; i >= 0; i--) {
        var oAjax = g_aAjaxCalls[i];
        if (oAjax) {
            oAjax.abort();
            g_aAjaxCalls[i] = null;
        }
    }

    g_aAjaxCalls = null; 
    g_aAjaxCalls = [];
}

function OnUserAdded(event) {
    var objThis = arguments.callee.owner;
    objThis.OnUserAdded(event);
}

pumk.prototype.OnUserAdded = function(event) {
    var addedUserScreenName = event.Params.pUser.ScreenName;
    var elementDiv = document.getElementById(addedUserScreenName);

    // Process the user added event only if the added user is currently in the UI
    if (elementDiv) {
        // Replace user with a new one if available
        this._arrSuggestedUsers[addedUserScreenName].isAdded = true; // mark previous user as added
        var mainDiv = document.getElementById("d-pumk-main");
        mainDiv.removeChild(elementDiv);

        // Calculate which user should go next
        for (var nStart = this._nCurrentPage * this._maxUsersPerPage; nStart < this._nSuggestions; ++nStart) {
            if (this._arrSortedSuggestedUsers[nStart].isAdded == false &&
                (this._arrSortedSuggestedUsers[nStart].onPage == 0 || this._arrSortedSuggestedUsers[nStart].onPage > this._nCurrentPage))
                break;
        }

        this._nAddedUsers++;
        this._statCollection.IncrementMapSecondDItem("CUsage", "SFA", this._xtraHostName, "1");
        this.reportClick();

        if (nStart < this._nSuggestions) {
            var nAddedIndex = this.addNextUser(mainDiv, nStart);
            // If a user was added, get its devil
            if (nAddedIndex >= 0) {
                var sDevilsUrl = C_GET_DEVILS_URL + "&t=" + this._arrSortedSuggestedUsers[nAddedIndex].sn;
                this.requestDevils(sDevilsUrl);
            }
        }

        this.calcPages();
        this.showPagesIndicator();

        // If the current page is empty, move to the last page, or show the no more results page if no results are left
        // The 'No more friends' div should always remain
        var nRemainingChildren = mainDiv.childNodes.length;
        if (nRemainingChildren == 1 && this._nTotalPages > 0) {
            this.showPage(this._nTotalPages);
        }
        else if ((nRemainingChildren == (this._maxUsersPerPage - 1)) && (mainDiv.lastChild.id != "d-pumk-main-more")) {
            this.addFindMoreFriends();
        }
    }
}

pumk.prototype.reportError = function(api, protocol, error) {
    var srpItem = api + ':' + protocol + ':' + error;
    this._statCollection.IncrementMapSecondDItem("CUsage", "SFE", srpItem, "1");
}

function OnSkinChanged() {
    buttonHoverColor = icqEx.getClientColor(MEClientColor.kColor_ButtonHover, '#75DE72');
}

// Global helper functions
function isSet(v) {
    if (typeof (v) != 'undefined') return true;
    return false;
}

function getImageSrc(sFileName) {
    var src = ' src="' + C_IMAGE_PREFIX + sFileName + '" ';
    return src;
}

function trim(str) {
    return str.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
}

// Text buttons effects handlers
function btnMouseOver(btn) {
    btn.blur();
    btn.style.filter = 'progid:DXImageTransform.Microsoft.gradient(gradientType=0,enabled="true", startColorstr=#FFFFFF, endColorstr=' + buttonHoverColor + ')';
}
function btnMouseOut(btn) {
    btn.blur();
    btn.style.filter = 'progid:DXImageTransform.Microsoft.gradient(gradientType=0,enabled="true", startColorstr=#FFFFFF, endColorstr=#E9E9DF)';
    btn.style.border = '1px solid #9F9F9F';
}
function btnMouseUp(btn) {
    btn.blur();
    btn.style.filter = 'progid:DXImageTransform.Microsoft.gradient(gradientType=0,enabled="true", startColorstr=#FFFFFF, endColorstr=' + buttonHoverColor + ')';
    btn.style.border = '1px solid #9F9F9F';
}
function btnMouseDown(btn) {
    btn.blur();
    btn.style.filter = 'progid:DXImageTransform.Microsoft.gradient(gradientType=0,enabled="true", startColorstr=' + buttonHoverColor + ', endColorstr=#FFFFFF)';
    btn.style.border = '1px solid #BBBBBB';
}

///////////////////////////////////////////////////////////////////////////////
// Json and Ajax helper functions

pumk.prototype.parseAjax = function(oAjax, apiCode) {
    var response = null;
    if (oAjax.readyState == 4) {

        if (oAjax.status != 200) {
            this.onError(apiCode, MEPumkProtocol.kHttp, oAjax.status);
        }
        else {
            response = parseIcqProtocol(oAjax.responseText);
            if (response == null) {
                this.onError(apiCode, MEPumkProtocol.kHttp, MEPumkError.kInvalidJsonFormat);
            }
        }
    }

    return response;
}

pumk.prototype.onError = function(api, protocol, error) {
    this.cancelRequestTimer();
    this.showServiceError();

    this.reportError(api, protocol, error);
}

function parseIcqProtocol(sJson) {
    var oResponse = null;
    try {
        oResponse = json_parse(sJson);
        oResponse = oResponse ? oResponse.response : null;
    }
    catch (error) {
        return null;
    }

    return oResponse;
}

function makeAjaxCallback(oAjax, thisArg, funcCallback, nRequestId) {

    g_aAjaxCalls.push(oAjax);

    return function() {
        funcCallback.call(thisArg, oAjax, nRequestId);
        if (oAjax.readyState === 4) {
            // remove from live list
            g_aAjaxCalls.find(oAjax, true);

            oAjax.abort();
            oAjax.onreadystatechange = null;
            oAjax = null;
        }
    }
}

Array.prototype.find = function(match, bRemove) {
    var bIsFunction = typeof (match) === "function";
    var nLength = this.length;

    var ret = null;

    for (var i = 0; i < nLength; i++) {
        var item = this[i];

        if (bIsFunction) {
            if (match(item)) {
                ret = item;
                break;
            }
        }
        else if (match === item) {
            ret = item;
            break;
        }
    }

    if (bRemove && (i < nLength))
        this.splice(i, 1); // remove found item

    return ret;
}

// Json parser function was copied as is from www.json.org
var json_parse = (function() {
    var at,     // The index of the current character
        ch,     // The current character
        escapee = { '"': '"','\\': '\\','/': '/',b: '\b',f: '\f',n: '\n',r: '\r',t: '\t'},
        text,

        error = function(m) {
            throw {name: 'SyntaxError',message: m,at: at,text: text};
        },

        next = function(c) {
            if (c && c !== ch) {
                error("Expected '" + c + "' instead of '" + ch + "'");
            }
            ch = text.charAt(at);
            at += 1;
            return ch;
        },

        number = function() {
            var number,string = '';

            if (ch === '-') {
                string = '-';
                next('-');
            }
            while (ch >= '0' && ch <= '9') {
                string += ch;
                next();
            }
            if (ch === '.') {
                string += '.';
                while (next() && ch >= '0' && ch <= '9') {
                    string += ch;
                }
            }
            if (ch === 'e' || ch === 'E') {
                string += ch;
                next();
                if (ch === '-' || ch === '+') {
                    string += ch;
                    next();
                }
                while (ch >= '0' && ch <= '9') {
                    string += ch;
                    next();
                }
            }
            number = +string;
            if (isNaN(number)) {
                error("Bad number");
            } else {
                return number;
            }
        },

        string = function() {
            // Parse a string value.
            var hex,i,string = '',uffff;
            if (ch === '"') {
                while (next()) {
                    if (ch === '"') {
                        next();
                        return string;
                    } else if (ch === '\\') {
                        next();
                        if (ch === 'u') {
                            uffff = 0;
                            for (i = 0; i < 4; i += 1) {
                                hex = parseInt(next(), 16);
                                if (!isFinite(hex)) {
                                    break;
                                }
                                uffff = uffff * 16 + hex;
                            }
                            string += String.fromCharCode(uffff);
                        } else if (typeof escapee[ch] === 'string') {
                            string += escapee[ch];
                        } else {
                            break;
                        }
                    } else {
                        string += ch;
                    }
                }
            }
            error("Bad string");
        },

        white = function() {
            while (ch && ch <= ' ') { next(); }
        },

        word = function() {
            // true, false, or null.
            switch (ch) {
                case 't': next('t');next('r');next('u');next('e');
                    return true;
                case 'f': next('f');next('a');next('l');next('s');next('e');
                    return false;
                case 'n': next('n');next('u');next('l');next('l');
                    return null;
            }
            error("Unexpected '" + ch + "'");
        },

        value,  // Place holder for the value function.

        array = function() {
            var array = [];

            if (ch === '[') {
                next('[');
                white();
                if (ch === ']') {
                    next(']');
                    return array;   // empty array
                }
                while (ch) {
                    array.push(value());
                    white();
                    if (ch === ']') {
                        next(']');
                        return array;
                    }
                    next(',');
                    white();
                }
            }
            error("Bad array");
        },

        object = function() {
            var key,
                object = {};

            if (ch === '{') {
                next('{');
                white();
                if (ch === '}') {
                    next('}');
                    return object;   // empty object
                }
                while (ch) {
                    key = string();
                    white();
                    next(':');
                    if (Object.hasOwnProperty.call(object, key)) {
                        error('Duplicate key "' + key + '"');
                    }
                    object[key] = value();
                    white();
                    if (ch === '}') {
                        next('}');
                        return object;
                    }
                    next(',');
                    white();
                }
            }
            error("Bad object");
        };

    value = function() {
        white();
        switch (ch) {
            case '{': return object();
            case '[': return array();
            case '"': return string();
            case '-': return number();
            default: return ch >= '0' && ch <= '9' ? number() : word();
        }
    };

    return function(source, reviver) {
        var result;
        text = source;
        at = 0;
        ch = ' ';
        result = value();
        white();
        if (ch) {
            error("Syntax error");
        }

        return typeof reviver === 'function' ? (function walk(holder, key) {
            var k, v, value = holder[key];
            if (value && typeof value === 'object') {
                for (k in value) {
                    if (Object.hasOwnProperty.call(value, k)) {
                        v = walk(value, k);
                        if (v !== undefined) {
                            value[k] = v;
                        } else {
                            delete value[k];
                        }
                    }
                }
            }
            return reviver.call(holder, key, value);
        } ({ '': result }, '')) : result;
    };
} ())