/*
// XtrazSocket
// A singleton object used to serialize incoming and outgoing Plugin notifications
// sent and received through SendNotifyPlugin and onRemoteNotification
// The default operation is to send requests in a strict order. this means that a notification
// is not sent until a confirmation is received for the previous notification
// protocol:
// transactions always start at 1 and increment tightly
*/

if (top.XtrazSocket) {
	XtrazSocket = top.XtrazSocket;
}
else {

	XtrazSocket = {
		bInited : false,
		nextConnection : 0,
		maxResend : 6,
	
		randSeed : 1,
	
	// messages for broadcasting 
		msgDisconnect : 'dsCon',
		msgInError : 'ntfyInEr',
		msgConnectionDied : 'xsDead',

// Connection types
		eDirect : 'dir', eServer: 'srvr',
	
		mOwner : null
	}
	

	XtrazSocket.init = function() {
		if (! XtrazSocket.bInited) {
			var dt = new Date();
			var d = String(dt.getTime());
			d = d.replace(/[^0-9]/g, '');
			if (d.length > 9)
				d = d.substring(d.length -  9);
			XtrazSocket.randSeed = d;

			XtrazSocket.bInited = true;
			XtrazSocket.mTransTable = new Object;
//			XtrazSocket.mClients = new Array;
			XtrazSocket.mConnections = new Object;

			XtrazBroadcaster.addListener(this, XtrazConnector.msgShutdown);
		}
	}

// the owner of XtrazSocket is responsible for deciding what to do with handshake requests
// in theory, the xtra can open several concurrent games so more than one handshake is allowed
	XtrazSocket.setOwner = function(o) {
		this.init();
		this.mOwner = o;
	}
	
	XtrazSocket.listenToMessage = function(msg, p1, p2) {
		XtrazDebug.trace("XtrazSocket listentomessage", msg);

		var connector = XtrazConnector.getIcqConnector();

		switch (msg) {
			case XtrazConnector.msgShutdown:
				var connection, rec;
				var uXml, qXml, dataXml;
				var doc = XMLUtils.getScratchDocument();

// create data node	
// create recipient node

				for (var key in this.mConnections) {
					try {
						rec = this.mConnections[key];
						if (rec) {
							var outData = this.makeSendData(rec, {XS: XtrazSocket.msgDisconnect}, false /* don't use transid */);
	
							XtrazDebug.trace("shutdown sending", uXml, qXml, dataXml);
							if (rec.mSocketType == XtrazSocket.eServer)
								connector.SendNotifyPlugin(1000, outData.to, outData.props, outData.data);
							else if (rec.mSocketType == XtrazSocket.eDirect)
								connector.SendPluginMessage( outData.data, 1000);
						}
						else {
							XtrazDebug.trace("msgshutdown: null connection");
						}
					}
					catch (e) {
						XtrazDebug.trace("xtraz notify shutdown", e);
					}
				}
				break;
			default:
				XtrazDebug.trace("XtrazSocket.listenToMessage", msg);
		}
	}

	XtrazSocket.ClientRecord = function(c) {
		this.client = c; // notification client
		this.nextTrans =  1 + XtrazSocket.nextConnection * 1000000; // next transaction id, always incremented
		this.lastTransOut =  this.nextTrans - 1; // last transaction sent 
		this.lastTransIn =  0; // last transaction received
		this.mOutQueue =  new Object; // queue of pending transactions to send
		this.mInQueue =  new Object;  // queue of pending transactions received
		this.mResendTimer =  null; // timer of resend operation
		this.connectionID = -999;
		this.mRetries = 0;
		this.mPluginProps = { PluginID: XtrazConnector.PluginID };
		this.mBuddy = '';
		this.maxResend = XtrazSocket.maxResend;
		this.mSocketType = XtrazSocket.eServer;
		this.bRawSocket = false; // if true, incoming messages are not processed at all, just passed to the client 
	//$$ todo: change this to an array of buddies
		return this;
	}
	
	XtrazSocket.ClientRecord.prototype = new Object;
	
	XtrazSocket.ClientRecord.prototype.stopTimer = function() {
		if (this.mResendTimer) {
			clearInterval(this.mResendTimer);
			this.mResendTimer = null;
			this.mRetries = 0;
		}
	}

//========= Client Record accessors ===========

	XtrazSocket.getConnectionNextTransID = function(cID) {
		var ret = -1;
		
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	getConnectionNextTransID: bad id", cID);
		if (rec) {
			ret = rec.nextTrans;
		}
		return ret;
	}

	XtrazSocket.setConnectionNextTransID = function(cID, tID) {
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	getConnectionNextTransID: bad id", cID);
		XtrazDebug.assert(! isNaN(tID), "setConnectionNextTransID: bad data", cID, tID);
		tID = Number(tID);
		rec.lastTransIn = tID - 1;
	}

	XtrazSocket.addPluginProperty = function(cID, key, value) {
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	addPluginProperty: bad id", cID);
		if (rec) {
			rec.mPluginProps[key] = value;
		}
	}

	XtrazSocket.removePluginProperties = function(cID) {
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	removePluginProperties: bad id", cID);
		if (rec) {
			rec.mPluginProps = {};
		}
	}

	XtrazSocket.setSocketBuddy = function(cID, sBuddy) {
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	setSocketBuddy: bad id", cID);
		if (rec) {
			rec.mBuddy = sBuddy;
		}
	}

	XtrazSocket.getSocketBuddy = function(cID) {
		var rec = this.findConnection(cID);
		var ret = '';
		XtrazDebug.assert(rec, "getSocketBuddy: bad id", cID);
		if (rec) {
			ret = rec.mBuddy;
		}
		return ret;
	}

	XtrazSocket.setSocketType = function(cID, sType) {
		XtrazDebug.assert(sType == XtrazSocket.eDirect || sType == XtrazSocket.eServer, "setSocketType: bad type", sType);
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	setSocketType: bad id", cID);
		if (rec) {
			rec.mSocketType = sType;
		}
	}

	XtrazSocket.setRaw = function(cID, bRaw) {
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	setRaw: bad id", cID);
		if (rec) {
			rec.bRawSocket = bRaw;
		}
	}
	
	XtrazSocket.setConnectionOwner = function(cID, owner) {
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	getConnectionNextTransID: bad id", cID);
		if (rec) {
			rec.client = owner;
		}
	}

	XtrazSocket.setSocketRetries = function(cID, nRetries	) {
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	setSocketRetries: bad id", cID);
		if (rec) {
			if (nRetries && (! isNaN(nRetries))) {
				nRetries = Number(nRetries);
				if (nRetries < 10) {
					rec.maxResend = nRetries;
				}
				else {
					XtrazDebug.assert(false, "setSocketRetries: bad data", nRetries);
				}
			}
			else {
				XtrazDebug.assert(false, "setSocketRetries: bad data", nRetries);
			}
		}
	}

//========= internal transaction and connection methodsd ========
	XtrazSocket.findConnection = function(id) {
		return XtrazSocket.mConnections[String(id)];
	}

	XtrazSocket.findConnectionByBuddy = function(sBuddy) {
		var con;
		for (var key in this.mConnections) {
			con = this.mConnections[key];
			if (con && (con.mBuddy == sBuddy))
				return con;
		}
		return null;
	}

	XtrazSocket.pushTransaction = function(id, connectionID, cookie) {
		XtrazDebug.trace("XtrazSocket pushing transaction", id);
		this.mTransTable[String(id)] = {connection: connectionID, cookie: cookie};
	}
	
	XtrazSocket.popTransaction = function(id) {
		id = String(id);
		var trans = this.mTransTable[id];
		if (trans) {
			delete this.mTransTable[id];
		}
		return trans;
	}

	XtrazSocket.reset = function() {
		var rec;
		for (var key in this.mConnections) {
			rec = this.mConnections[key];
			rec.stopTimer();
			this.mConnections[key] = new XtrazSocket.ClientRecord(rec.client);
			this.mConnections[key].connectionID = rec.connectionID;
		}
	}
	
	
	XtrazSocket.sendTransaction = function(rec, cookie, uXml, qXml, dataXml, socketData) {
		var t = rec.nextTrans++;
		XtrazDebug.assert(! rec.mResendTimer, "sendTransaction: tried to send while timer was active");
		var ret = false;
		var trec = { to: uXml, props: qXml, data: dataXml, cookie: cookie, ack: false, socketData: socketData };
		rec.mOutQueue[t] = trec;
		if (t == (rec.lastTransOut + 1)) { // no pending transactions
			XtrazDebug.trace("sendTransaction: lastTransOut is", rec.lastTransOut, "sending", t);
			ret = this.doSendTransaction(rec, t, trec);
		}
		else {
			XtrazDebug.trace("sendTransaction: deferring transaction", t);
		}
		return ret
	}
	
	XtrazSocket.doSendTransaction = function(rec, transID, transRec) {
		var str = 'XtrazSocket.resend("' + rec.connectionID + '",' + transID + ')';
//		XtrazDebug.trace("setting interval to", str);
		rec.mResendTimer = setInterval(str, 2500);
//		XtrazDebug.trace("set timer to" , str);
// must push BEFORE we send, because the return event can interrupt execution of this code
		this.pushTransaction(transID, rec.connectionID, transRec.cookie);
		var ret = this.sendData(rec, transID, transRec);
		if (! ret) {
			XtrazDebug.trace("failed to send, popping transaction");
			this.popTransaction(transID);
			// for now we don't update any record data, we just wait for the reply
		}
		return ret;
	}
	
	XtrazSocket.sendData = function(rec, t, transRec) {
		var connector = XtrazConnector.getIcqConnector();
		var ret = true;
		try {
			if (rec.mSocketType == XtrazSocket.eServer) {
				XtrazDebug.trace("socket sending data by notification");
				connector.SendNotifyPlugin(t, transRec.to, transRec.props, transRec.data);
			}
			else if (rec.mSocketType == XtrazSocket.eDirect){
				XtrazDebug.trace("socket sending data by pluginmessage");
				connector.SendPluginMessage(transRec.data, t);
			}
			else {
				XtrazDebug.trace("socket senddata: illegal socket type", rec.mSocketType);
			}
		}
		catch (e) {
			XtrazDebug.trace("socket senddata", e);
			ret = false;
		}
		return ret;
	}


	XtrazSocket.resend = function(connectionID, transID) {
// $$ todo: limit retries
		XtrazDebug.trace("notify resend", connectionID, transID);
		var rec = XtrazSocket.findConnection(connectionID);
		XtrazDebug.assert(rec, "XtrazSocket.resend: bad connection id", connectionID);
		XtrazDebug.trace("resend: connectionid", connectionID, "rec", String(rec), typeof(rec), "type", rec.mSocketType);
		if (rec) {
			rec.mRetries++;
			if (rec.mRetries > rec.maxResend) {
				rec.stopTimer();
				XtrazBroadcaster.broadcast(XtrazSocket.msgConnectionDied, connectionID);
				this.closeConnection(connectionID);
			}
			else {
				var o = rec.mOutQueue[transID];
				XtrazDebug.assert(o, "resend: null out data for transid", transID);
			
				this.pushTransaction(transID, rec.connectionID, o.cookie); // before send, because the ack can interrupt this code
				var success = this.sendData(rec, transID, o);
				if (! success) {
					XtrazDebug.trace("failed to resend, popping transaction");
					this.popTransaction(transID);
					// for now we don't update any record data, we just wait for the reply
				}
			}
		}
	}
	
	XtrazSocket.getRandomConnectionID = function() {
		this.init();
		var ret = XtrazSocket.randSeed;
		do {
			ret += "_" + XtrazSocket.nextConnection; 
		}
		while (this.mConnections[ret]);

		XtrazSocket.nextConnection++;
		return ret;
	}
	
	XtrazSocket.openConnection = function(inClient, id) {
		XtrazDebug.trace("openconnection with id", id, typeof(id));
		XtrazSocket.init();
		if (! id) {
			id = this.getRandomConnectionID();
			XtrazDebug.trace("generated random id", id, typeof(id));
		}
		id = String(id);

		var rec = this.findConnection(id);
		if (rec) {
			XtrazDebug.trace("openconnection: switching connection owner for", id);
			rec.client = inClient;
		}
		else {
			rec = new XtrazSocket.ClientRecord(inClient);
			XtrazSocket.mConnections[id] = rec;
			rec.connectionID = id;
		}
		return id;	
	}
	
	XtrazSocket.onSocketBuddyLeft = function(buddy) {
		XtrazSocket.init();
		var rec = this.findConnectionByBuddy(buddy);
		XtrazDebug.assert(rec, "onSocketBuddyLeft: no socket for buddy", buddy);
		if (rec) {
			XtrazBroadcaster.broadcast(XtrazSocket.msgConnectionDied, rec.connectionID);
			this.closeConnection(rec.connectionID);
		}
	}
	
	XtrazSocket.closeConnection = function(cID) {
		cID = String(cID);
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "	getConnectionNextTransID: bad id", cID);
		if (rec) {
			rec.stopTimer();
			delete this.mConnections[cID];
		}
	}
	
	XtrazSocket.makeSendData = function(rec, data, bTrans) {
		var ret = new Object;
		ret.to = rec.mBuddy;
		
		var doc = XMLUtils.getScratchDocument();
		var props = rec.mPluginProps;
		var node;

	// create plugin query xml
		node = doc.createElement("Q");
		XMLUtils.objectToXML(props, node);
		doc.appendChild(node);
		ret.props = doc.xml;
	// create message xml
		doc.removeChild(node);
		
		node = doc.createElement("MSG");
		doc.appendChild(node);
		XMLUtils.objectToXML(data, node);
		XMLUtils.addTextNode(node, "_ConID_", rec.connectionID);
		if (bTrans)
			XMLUtils.addTextNode(node, "_TransID_", rec.nextTrans);

		XtrazDebug.trace("XtrazSocket.notify added trans id");
		
		ret.data = doc.xml;

		XtrazDebug.trace("XtrazSocket will send", ret.to, ret.props, ret.data);
		
		return ret;
	}
	
	
	XtrazSocket.attemptHandShake = function(cID, cookie, listenSocketID) {
		this.notify(cID, cookie, 
		{	
			XS: "GameHandShake", 
			FROMUIN: XtrazConnector.Owner.ScreenName,
			gameid: escape(XtrazConnector.PluginID),
			SEQ: 2,
			ConnectionID: listenSocketID,
			nextTransID: XtrazSocket.getConnectionNextTransID(listenSocketID)
		});
	}

	XtrazSocket.notify = function(connection, cookie, data) {
		XtrazDebug.trace("XtrazSocket.notify", connection, cookie, data);
		XtrazDebug.assert(typeof(data)=='object', "XtrazSocket.notify: bad data", data);
		var rec = XtrazSocket.findConnection(connection);
		XtrazDebug.assert(rec, "Xtraz notify: null client");
		var ret = false;
		if (rec) {
			var outData = this.makeSendData(rec, data, true /* user trans id */);
			ret =  this.sendTransaction(rec, cookie, outData.to, outData.props, outData.data, data.XS);
		}
		return ret;
	}


	XtrazSocket.onMessageAck = function(cID, nTransID) {
		XtrazDebug.trace('socket onmsgack');
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "onMessageAck: no client for connectionid", cID);
		if (rec)
			this.onMessageResult(nTransID, rec.mBuddy, sbRC_OkCode, '');
	}

// onMessageResult: handles replies for messages sent thru Notify
	XtrazSocket.onMessageResult = function(nTransID, sUserID, eCode, sData) {
		// handshake answer
		var trans = this.popTransaction(nTransID);
		XtrazDebug.trace("XtrazSocket.onMessageResult for trans", nTransID, "error", eCode, typeof(eCode));
		if (trans) {
			var rec = this.findConnection(trans.connection);
			XtrazDebug.assert(rec, "onMessageResult: no client for trans", trans);
			if (! rec)
				return;

			nTransID = Number(nTransID);

// Handle all outgoing messages
			var queue = rec.mOutQueue;
			var nextRec = queue[nTransID];
			nextRec.user = sUserID;
			nextRec.inData = sData;
			nextRec.inCode = eCode;
			nextRec.ack = (eCode == sbRC_OkCode);
			nextRec.cookie = trans.cookie;

			var tid;
			var queue = rec.mOutQueue;
			var nProcessed = 0;
			while (1) {
				tid = rec.lastTransOut + 1;// this will ignore duplicate replies, since
				// lastTransOut incremenets when  receive a reply
				nextRec = queue[tid]; // look for next transaction
//				XtrazDebug.trace("XtrazSocket.onMessageResult testing", nextRec);
				if (nextRec && nextRec.ack) {
					nProcessed++;
					rec.stopTimer();
					delete queue[tid];
					if (nextRec.socketData) {
						XtrazDebug.trace("XtrazSocket.onMessageResult: transaction", tid, "is a socket message");
						this.handleSocketResult(rec, nextRec);
					}
					else {
						rec.client.onNotificationResult(nextRec.cookie, nextRec.user, nextRec.inData, nextRec.inCode);
					}
					rec.lastTransOut++;
				}
				else { // no next record;
					break;
				}
			}
			tid = rec.lastTransOut + 1;
			if (nProcessed && (! rec.mResendTimer)) {
				nextRec = queue[tid];
				if (nextRec && ! nextRec.ack) {
					XtrazDebug.trace("onMessageResult: sending out deferred notification", tid);
					this.doSendTransaction(rec, tid, nextRec);
				}
			}
		}
// Handle all incoming messages
		else {
			XtrazDebug.trace("XtrazSocket.onMessageResult: no transaction for id", nTransID);
		}
	}
	
	XtrazSocket.handleSocketResult = function(clientRec, dataRec) {
		switch (dataRec.socketData) {
			case "GameHandShake":
				var success = false;
				if (dataRec.inCode == sbRC_OkCode) {
					var doc = XMLUtils.getScratchDocument();
					doc.loadXML(dataRec.inData);
					var err = XMLUtils.getParseError(doc);
					if (err == 0) {
						var reply = XMLUtils.getValueByNodeName(doc, "//RES", "");
						if (reply) {
							doc.loadXML(reply);
							err = XMLUtils.getParseError(doc);
							if (err == 0) {
								reply = XMLUtils.getValueByNodeName(doc, "//XS", "");
								XtrazDebug.trace("handleSocketResult: XS reply is", reply);
								if (reply == "GameHandShake_Ok") {
									success = true;
								}
							}
							else {
								XtrazDebug.trace("handleSocketResult: bad RES xml status", err);
							}
						}
						else {
							XtrazDebug.trace("handleSocketResult: no reply node found");
						}
					}
					else {
						XtrazDebug.trace("handleSocketResult: bad message xml status", err);
					}
				}
				else {
					XtrazDebug.trace("handleSocketResult: bad code", dataRec.inCode);
				}
				clientRec.client.onHandshakeResult(success, dataRec.user);
				break;
			default:
				XtrazDebug.assert(false, "handleSocketResult: bad socket data in", dataRec);
		}
	}
	
	XtrazSocket.onPluginMessage = function(cID, sMessage) {
		XtrazSocket.init();
		var ret = false;
		var rec = this.findConnection(cID);
		XtrazDebug.assert(rec, "socket onpluginmessage: bad id", cID);

		if (! rec)
			return false;

		if (rec.bRawSocket) {
			try {
				ret = rec.client.onPluginMessage(sMessage);
			}
			catch(e) {
				XtrazDebug.trace("XtrazSocket.onPluginMessage: raw client onpluginmessage", e);
			}
		}
		else {
			var xmlDoc = XMLUtils.getScratchDocument();
			xmlDoc.loadXML(sMessage);
			var status = XMLUtils.getParseError(xmlDoc);
	
			if (status != 0) { // parse error
				XtrazDebug.trace("on remote notification parse error", obj.status);
				XtrazBroadcaster.broadcast(XtrazSocket.msgInError, obj.status);
			}
			else {
				try {
					ret = this.onRemoteNotification(rec.mBuddy, sMessage, xmlDoc, cID);
				}
				catch (e) {
					XtrazDebug.trace("XtrazSocket.onPluginMessage: client onpluginmessage", e);
				}
			}
			
		}
	}

	XtrazSocket.onRemoteNotification = function(sBuddy, sNotify, xmlDoc, connection) {
		XtrazDebug.trace("XtrazSocket.onRemoteNotification",sBuddy, sNotify);
		var icqConnector = XtrazConnector.getIcqConnector()

		if (! xmlDoc) {
			xmlDoc = XMLUtils.getScratchDocument();
			xmlDoc.loadXML(sNotify);
		}

		var obj = new Object;
		obj.doc = xmlDoc;
		obj.buddy = sBuddy;
		obj.rawMessage = sNotify;
		obj.status = XMLUtils.getParseError(xmlDoc);
		
		var ret = true;
		
		if (obj.status != 0) {// parse error
			XtrazDebug.trace("on remote notification parse error", obj.status);
			XtrazBroadcaster.broadcast(XtrazSocket.msgInError, obj.status);
			return false;
		}
		
		obj.msg = XMLUtils.getValueByNodeName(xmlDoc, '//MSG', "");
		XtrazDebug.trace("type of msg is", typeof(obj.msg));

		if (! connection)
			connection = XMLUtils.getValueByNodeName(xmlDoc, '//_ConID_', -1);
		XtrazDebug.trace("onRemoteNotification: msg", typeof(obj.msg), "connection", connection);
		var rec;
		if (connection != -1) { 
			rec = this.findConnection(connection);
		}
		else
			rec = null;
		if (rec) {// in notification contains trans id
			var socketMsg = XMLUtils.getValueByNodeName(obj.msg, "//XS", '');
			if (socketMsg != '') { // reserved socket message
				switch (socketMsg) {
					case XtrazSocket.msgDisconnect:
						XtrazBroadcaster.broadcast(XtrazSocket.msgDisconnect, connection);
						break;
					default:
						XtrazDebug.assert(false, "unknown socket message", socketMsg);
						break;
				}
			}

			else { // not a reserved socket msg
				var transID = XMLUtils.getValueByNodeName(xmlDoc, '//_TransID_', -1);
				var queue = rec.mInQueue;
				XtrazDebug.assert(! queue[transID], "XtrazSocket.onRemoteNotification: duplicate trans id", transID);
				if (transID == -1) { // transactionless message, process immediately
					try {
						rec.client.onRemoteNotification(obj);
					}
					catch (e) {
						XtrazDebug.trace("socket onremotenotifcation: failed to notify client", e);
					}
				}
				else if (transID <= rec.lastTransIn) {
					XtrazDebug.trace("ignoring duplicate transaction", transID);
				}
				else {
		// push the incoming message into the queue, then try to flush it
					queue[transID] = obj;
					var nextRec;
					var tid;
					var status;
					while (1) {
// hack to avoid a complex handshake for direct connections
// the first message determines the first trans number
//						if (rec.mSocketType == XtrazSocket.eDirect) {
							if (rec.lastTransIn == 0)
								rec.lastTransIn = transID - 1;
//						}
						tid = rec.lastTransIn + 1;
						XtrazDebug.trace("XtrazSocket.onRemoteNotification processing trans", tid, "incoming trans", transID);
						nextRec = queue[tid]; // look for next transaction
						if (nextRec) { // have a pending in message
							XtrazDebug.trace("XtrazSocket.onRemoteNotification can send transaction to connection owner");
							delete queue[tid];
							status = rec.client.onRemoteNotification(obj);
							if (status)
								rec.lastTransIn++;
						}
						else { // no next record;
							break;
						}
					}
				}
			}
		}
		else { // no connection
//			XtrazDebug.trace("XtrazSocket.onRemoteNotification, no connection, message", obj.msg);
			var msg = unescape(XMLUtils.getValueByNodeName(obj.msg, "//XS", ''));
			switch (msg) {
				case 'GameHandShake':
					XtrazDebug.assert(connection != -1, "onRemoteNotification handshake: bad connection", connection);
					var wantHS = false;
					try {
						wantHS = XtrazSocket.mOwner.onHandshake(sBuddy, xmlDoc);
					}
					catch(e) {}
					if (wantHS) {
						var doc = XMLUtils.getScratchDocument();
						var node = doc.createElement("REPLY");
						doc.appendChild(node);
						XMLUtils.addTextNode(node, "XS", "GameHandShake_Ok");
						XMLUtils.addTextNode(node, "_ConID_", connection);
						ret = doc.xml;
					}
					else
						ret = false;
					break;
				default:
					XtrazDebug.assert(false, "socket onRemoteNotification: unknown message", msg);
					break;

			}
		}
		return ret;
	}

//========= End Class definition ============
} // end conditional class definition
