/**
 * Category button's constructor.
 *
 * @param  _gadget the button gadget
 */
function component (_gadget) 
{ 
	this.gadget = _gadget; 
	this.eventListener = null;
}

/**
 * 
 */
component.prototype.onInitialized = function ()
{
	this.categoryID = this.gadget.getAttribute("idVal");
}

/**
 * Returns the button's category ID: a string taken from the idVal attribute. 
 *
 * @return      ID (string)
 */
component.prototype.getCategoryID = function ()
{
	return this.categoryID;
}

/**
 * Calls eventListener's function when mouse over the button.
 *
 */
component.prototype.onMouseOver = function()
{
	if (this.eventListener) {
		this.eventListener.onMouseOverCategoryButton(this.gadget);
	}
}

/**
 * Calls eventListener's function when user clicks on the button.
 *
 */
component.prototype.onClick = function()
{
	
	var eventObj = shell.currentEvent;
    if (eventObj)
    {
		// This prevents the mouseDown event from causing the freshly opened popup to immediately close!
		eventObj.preventDefault();
		eventObj.stopPropagation(); 
	}              
	
	if (this.eventListener) {
		this.eventListener.onClickCategoryButton(this.gadget);
	}
}

/**
 * Sets the button's eventListener.
 *
 * @param  listener a gadget that will respond to button events
 */
component.prototype.setEventListener = function(listener)
{
	this.eventListener = listener;
}
