/**
 * Popup Tab's constructor.
 *
 * @param  _gadget the popup tab gadget
 */
function component (_gadget) 
{ 
	this.gadget = _gadget; 
	this.category = null;
	this.location = null;
}

var TAB_CONTENTS_OFFSET = 52;
var TAB_CONTENTS_OFFSET_Y = 6;

/**
 * Sets the backing image for the tab, which can be up, down, or detached.
 *
 * @param  shape the name of the bitmap for the image.
 */
component.prototype.setShape = function (shape)
{
	this.gadget.getPartById("popup_tab").src = shape;
}

component.prototype.initializeImageStack = function (composeCategoryList, readyMessagesCategoryList, extraCategoryList)
{
	var imageStack = this.gadget.getPartById("popup_tab_contents").getBoxByTag("imageStack");
	if(!imageStack.alreadyInitialized())
	{
		imageStack.addCategoryImages (composeCategoryList);
		imageStack.addCategoryImages (readyMessagesCategoryList);
		imageStack.addCategoryImages (extraCategoryList);
	}
	
}

/**
 * Sets the category and location values for the popup tab.
 *
 * @param  cat category data.
 * @param  location location data.
 */
component.prototype.setUpPopupTab = function (cat, location)
{	
	this.category = cat;
	this.location = location;

	var buttonX = this.location.buttonX - this.gadget.parent.frameX - 39;

	this.gadget.left = buttonX;
		
	var catImage = this.category.imagePath;
	var catName = this.category.label;
	
	this.gadget.setAttribute("txt", catName);
	//this.gadget.setAttribute("img", catImage);
	this.gadget.getPartById("popup_tab_contents").getBoxByTag("imageStack").setCurrentImage(catImage);
	this.gadget.setStyle("position", "fixed");
	this.gadget.setStyle("margin", "0");
	this.gadget.setStyle("padding", "0");

	this.gadget.getPartById("popup_tab_contents").left = TAB_CONTENTS_OFFSET - 19;
	this.gadget.getPartById("popup_tab_contents").top = TAB_CONTENTS_OFFSET_Y;
}
