/**
 * Ready Messages Panel's constructor.
 *
 * @param  _gadget the ready messages panel gadget
 */
function component (_gadget) 
{ 
	this.gadget = _gadget; 
	this.lastMessageItemOnPage = null;
	this.firstMessageItemOnPage = null;
	this.selectedMessageItem = null;
	this.selectedMessageWords = Array();
	this.selectedMessageIds = Array();
	this.category = null;
	this.currentPage = 0;
}

var MAX_MESSAGES_PER_PAGE = 5;

/**
 * 
 *
 */
component.prototype.onInitialized = function()
{
	this.zlangoRootBox = this.gadget.parent.gadgetParent.gadgetParent;
	/*
	if (this.zlangoRootBox)
	{
		this.scene.alert(this.zlangoRootBox.tag, "readyMessagesPanel.js", 1);
	}
	*/
}


/**
 * Sets the category, and calls setUpPopup to continue setup.
 *
 * @param  cat the category object.
 */
component.prototype.setCategory = function (cat)
{
	this.category = cat;
	this.gadget.setUpPopup();
}

/**
 * Sets up the popup's data and visual layout.
 *
 */
component.prototype.setUpPopup = function ()
{
	this.cat = this.category.label;
	this.catID = this.category.id;
	this.catIndex = this.category.index;
	this.gadget.setAttribute("Category", this.cat);
	
	var oldList = this.gadget.getPartById("readyMessageList");
	var temp;
	while (oldList.firstChild != null) {
		temp = oldList.firstChild
		oldList.removeChild(temp);
		this.scene.destroyBox(temp);
	}

	//this.gadget.getPartById("nextButton").setAttribute("label", shell.zlangoGlobals.strings.next + " " + MAX_MESSAGES_PER_PAGE);
	//this.gadget.getPartById("previousButton").setAttribute("label", shell.zlangoGlobals.strings.previous + " " + MAX_MESSAGES_PER_PAGE);
	
	var readyMessagesNodes = this.gadget.parent.gadgetParent.gadgetParent.getReadyMessagesDomObject().documentElement.selectNodes("category[@id='" + this.catID + "']/msg");
	var textBox;
	var firstTextBox = null;
	var separator;
	separator = this.scene.createBox("readyMessageListSeparator");
	separator.setStyle("position", "fixed");
	separator.left = 0;
	separator.top = 0;
	this.gadget.getPartById("readyMessageList").addChild(separator);

	var totalNodes = readyMessagesNodes.length;

	var messageOffset = this.currentPage * MAX_MESSAGES_PER_PAGE;
	
	if (messageOffset > totalNodes) {
		this.currentPage = 0;
		messageOffset = this.currentPage * MAX_MESSAGES_PER_PAGE;
	}
	else if (messageOffset < 0) {
		var numPages = Math.ceil(totalNodes / MAX_MESSAGES_PER_PAGE);
		this.currentPage = Math.max(numPages - 1, 0);
		messageOffset = this.currentPage * MAX_MESSAGES_PER_PAGE;
	}
	
	var pageText = "(";
	pageText += this.currentPage * MAX_MESSAGES_PER_PAGE + 1 + "-";
	if ((this.currentPage + 1) * MAX_MESSAGES_PER_PAGE > totalNodes) {
		pageText += totalNodes;
	} else {
		pageText += (this.currentPage + 1) * MAX_MESSAGES_PER_PAGE;
	}
	pageText += " / " + totalNodes + ")";
	this.gadget.getPartById("pageText").value = pageText;
	
	for (i = 0; i + messageOffset < totalNodes && i < MAX_MESSAGES_PER_PAGE; i++) {
		textBox = this.scene.createBox("readyMessageItem");
		textBox.setAttribute("id", "textLine" + i);
		textBox.setStyle("position", "fixed");
		textBox.setStyle("left", "0");
		textBox.setStyle("top", i * 20 + 2);
		textBox.setAttribute("label", readyMessagesNodes[i + messageOffset].text);
		this.gadget.getPartById("readyMessageList").addChild(textBox);

		separator = this.scene.createBox("readyMessageListSeparator");
		separator.setStyle("position", "fixed");
		separator.left = 0;
		separator.top = (i + 1) * 20;
		this.gadget.getPartById("readyMessageList").addChild(separator);
		if (i == 0) {
			firstTextBox = textBox;
		}
	}
	this.lastMessageItemOnPage = textBox;

	if (firstTextBox) {
		this.firstMessageItemOnPage = firstTextBox;
		this.gadget.selectReadyMessageItem(firstTextBox);
	}
}

/**
 * Handles keyboard input.
 *
 * @param  key the keyboard button that was pressed.
 */
component.prototype.onReceivedKey = function(key)
{
	var LEFT_KEY = 37;
	var UP_KEY = 38;
	var RIGHT_KEY = 39;
	var DOWN_KEY = 40;
	var ENTER_KEY = 13;
	switch (key.keyCode) {
		case 0:
			switch(key.virtualKeyCode) {
				case UP_KEY:
					if (this.selectedMessageItem) {
						if (this.selectedMessageItem.previousSibling && this.selectedMessageItem.previousSibling.previousSibling) {
							this.selectReadyMessageItem(this.selectedMessageItem.previousSibling.previousSibling);
						} else {
							this.selectReadyMessageItem(this.lastMessageItemOnPage);							
						}
					}
					break;
				case DOWN_KEY:
					if (this.selectedMessageItem) {
						if (this.selectedMessageItem.nextSibling && this.selectedMessageItem.nextSibling.nextSibling) {
							this.selectReadyMessageItem(this.selectedMessageItem.nextSibling.nextSibling);
						} else {
							this.selectReadyMessageItem(this.firstMessageItemOnPage);							
						}
					}
					break;
				case RIGHT_KEY:
					if (shell.zlangoGlobals.rtl)
					{
						this.gadget.previousPage();
					}
					else
					{
						this.gadget.nextPage();
					}
					break;
				case LEFT_KEY:
					if (shell.zlangoGlobals.rtl)
					{
						this.gadget.nextPage();
					}
					else
					{
						this.gadget.previousPage();
					}
					break;
			}
			break;
		case ENTER_KEY:
			this.gadget.insertMessage();
			break;
	}
}

/**
 * Inserts the selected message, word by word, and closes the popup.
 *
 */
component.prototype.insertMessage = function ()
{
	for (i = 0; i < this.selectedMessageWords.length; i++) {
		this.zlangoRootBox.sendWordBySynonym(this.selectedMessageWords[i]);
	}
	this.zlangoRootBox.logReadyMessage (this.selectedMessageIds.join("+"));
	this.gadget.parent.closePopup();
}

/**
 * When the Insert button is pressed, calls insertMessage().
 *
 */
component.prototype.insertButton = function ()
{
	this.gadget.insertMessage();
}

/**
 * Goes to the next page of ready messages.
 *
 */
component.prototype.nextPage = function ()
{
	this.selectedMessageItem = null;
	this.currentPage++;
	this.gadget.setUpPopup();	
}

/**
 * Goes to the previous page of ready messages.
 *
 */
component.prototype.previousPage = function ()
{
	this.selectedMessageItem = null;
	this.currentPage--;
	this.gadget.setUpPopup();	
}

/**
 * When the Next button is pressed, calls nextPage().
 *
 */
component.prototype.nextButton = function ()
{
	this.gadget.nextPage();
}

/**
 * When the Previous button is pressed, calls previousPage().
 *
 */
component.prototype.previousButton = function ()
{
	this.gadget.previousPage();
}

/**
 * Selects a new ready message to be displayed as the current
 * selection.
 *
 * @param  item the ready message to be selected.
 */
component.prototype.selectReadyMessageItem = function (item)
{
	if (item == this.selectedMessageItem) {
		return;
	}
	delete this.selectedMessageWords;
	this.selectedMessageWords = Array();
	delete this.selectedMessageIds;
	this.selectedMessageIds = Array();
	
	if (this.selectedMessageItem) {
		// make this a function call
		this.selectedMessageItem.getPartById("text").setStyle("fontBold", "false");	
		this.selectedMessageItem.removeStyle("fill");
		this.selectedMessageItem.setAttribute("bullet", "#messagesBulletUnselected")
	}
	// this should also be in the function
	item.getPartById("text").setStyle("fontBold", "true");	
	item.setStyle("fill", "#E7F3DB");	
	item.setAttribute("bullet", "#messagesBulletSelected")
	this.selectedMessageItem = item;
	this.gadget.getPartById("messageText").value = item.getPartById("text").value;
	
	var message = item.getAttribute("label");
	var tokens = this.gadget.tokenizeReadyMessage(message);
	var container = this.gadget.getPartById("messagePreviewBox").getPartById("container");
	// remove previous message from the display
	var temp;
	while (container.firstChild != null) {
		temp = container.firstChild;
		container.removeChild(temp);
		this.scene.destroyBox(temp);
	}
	for (i = 0; i < tokens.length; i++) {
		// To ignore spaces, uncomment this block:
		if (tokens[i] == " ")
		{
			continue;
		}
	
		if (i==0)
		{
			//this.scene.alert(tokens[i], "tokens[i]", 1);
		}
		
		// For some reason this.zlangoRootBox doesn't work here so we have to use the full hierarchy path...
		var word = this.gadget.parent.gadgetParent.gadgetParent.getWordBySynonym(tokens[i]);
		this.selectedMessageWords.push(tokens[i]);
		var newIcon = this.scene.createBox("wordIcon");
		newIcon.setAttribute("labelVal", tokens[i]);
		if (word) {
			this.selectedMessageIds.push(word.Id);
			newIcon.setAttribute("wordID", word.Id);
			newIcon.setAttribute("iconVal", word.Image);	
		} else {
			newIcon.setAttribute("wordID", "");
			newIcon.setAttribute("iconVal", "");	
		}
		container.addChild(newIcon);
	}
}

/**
 * Highlights a ready message in the list without seleceting it as the current selection.
 *
 * @param  item the ready message to be selected.
 */
component.prototype.mouseOverReadyMessageItem = function (item)
{
	if (item == this.selectedMessageItem) {
		return;
	}
	
	item.setStyle("fill", "#E7F3DB");	
}

/**
 * Un-highlights a ready message in the list.
 *
 * @param  item the ready message to be unhighlighted.
 */
component.prototype.mouseOutReadyMessageItem = function (item)
{
	if (item == this.selectedMessageItem) {
		return;
	}
	
	item.removeStyle("fill");	
}


/**
 * Splits a string into tokens.
 *
 */
component.prototype.tokenizeReadyMessage = function (message)
{
	var TEMP_DELIM = String.fromCharCode(0x007C);
	// Surround all delimiters with a temporary delimiter
	var temp = message.replace(/([^a-zA-Z0-9\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF\u05D0-\u05EA\u0400-\u045F\u0100-\u017F\'-])/g, TEMP_DELIM + "$1" + TEMP_DELIM);
	
	// If there are two delimiters in a row, remove one of the temporary delimiters
	temp = temp.replace(/\u007C\u007C/g, TEMP_DELIM);
	
	// Split the message on the temporary delimiters
	var tokens = temp.split(TEMP_DELIM);
	if (tokens[tokens.length-1] == "") {
		tokens.pop();
	}
	return tokens;
}