/**
 * Tab Panel Set's constructor.
 *
 * @param  _gadget the tab panel set gadget
 */
function component (_gadget) 
{ 
	this.gadget = _gadget; 
	this.showingPanel = null;
}

/**
 * Set up the tab panel set when instantiated.
 *
 */
component.prototype.onInitialized = function ()
{
	this.gadget.parent.setTabPanelSet(this.gadget);
	
	var tabSet = this.gadget.parent.getTabSet();
	this.gadget.top = tabSet.boxHeight;
}

/**
 * Show a specific panel, by index.
 *
 * @param  tabIndex which panel to show.
 */
component.prototype.showPanel = function (tabIndex)
{
	if (this.showingPanel) {
		this.showingPanel.setVisible(false);
	}
	
	var currentPanel = this.gadget.getChildAt(tabIndex);

	if (currentPanel) {
		currentPanel.setVisible(true);
		this.gadget.setShowingPanel(currentPanel);
		var categoryPanel = currentPanel.getCategoryPanel();
		if (categoryPanel)
		{
			categoryPanel.onSelected();
		}
	}
}

/**
 * Get the panel which is currently showing.
 *
 * @return the tabPanel object
 */
component.prototype.getShowingPanel = function ()
{
	return this.showingPanel;
}

/**
 * Set the panel which is currently showing.
 *
 * @param  panel which panel to show.
 */
component.prototype.setShowingPanel = function (panel)
{
	this.showingPanel = panel;
}