
var ROW_HEIGHT = 59;
var NUM_ICONS_PER_ROW = 9;
var NUM_VISIBLE_ROWS = 3;
var MAXIMUM_ICONS_PER_CATEGORY = 54; // more than this causes Boxely to freeze
									// perhaps a memory or resources problem

/**
 * Word Icons Panel's constructor.
 *
 * @param  _gadget the word icons panel gadget
 */
function component (_gadget) { 
	this.gadget = _gadget;
	this.liveAnimations = 0;
	this.numRows = 0;
	this.topRowIndex = 0;
	this.rowArray = Array();
	this.category = null;
	this.currentButton = null;
	this.currentRow = null;
}

/**
 * 
 *
 */
component.prototype.onInitialized = function()
{
	this.zlangoRootBox = this.gadget.parent.gadgetParent.gadgetParent;
	/*
	if (this.zlangoRootBox)
	{
		this.scene.alert(this.zlangoRootBox.tag, "wordIconsPanel.js", 1);
	}
	*/
}

/**
 * Sets a new category and calls setUpWordIconsPanel().
 *
 * @param  cat the category object to select.
 */
component.prototype.setCategory = function (cat)
{
	this.category = cat;
	this.gadget.setUpWordIconsPanel();
}

/**
 * When a button is clicked, pass its label on to sendWord().
 *
 * @param  button the word icon button that was clicked.
 */
component.prototype.clickWordIcon = function (button)
{
	var rowIndex = button.parent.id.substr(4);
	if (rowIndex < this.topRowIndex || rowIndex >= this.topRowIndex + NUM_VISIBLE_ROWS) {
		return;
	}
	
	if (this.currentButton) {
		this.currentButton.setSelected(false);
	}
	this.currentButton = button;
	this.currentButton.setSelected(true);

	this.zlangoRootBox.sendWord(button.getAttribute("labelVal"));
	
	this.zlangoRootBox.logWord(button.getAttribute("wordID"));
}

/**
 * Sets the currently selected button based on mouse movement.
 *
 * @param  button the button that was selected by the mouse (mouseOver).
 */
component.prototype.onMouseSelect = function(button) {
	var rowIndex = button.parent.id.substr(4);
	if (rowIndex < this.topRowIndex || rowIndex >= this.topRowIndex + NUM_VISIBLE_ROWS) {
		return;
	}
	if (this.currentButton) {
		this.currentButton.setSelected(false);
	}
	this.currentButton = button;
	this.currentButton.setSelected(true);
}

/**
 * Responds to keyboard buttons by moving the selection indicator or sending
 * a word.
 *
 * @param  key the keyboard button that was pressed.
 */
component.prototype.onReceivedKey = function(key){
	if (this.currentButton == null) {
		this.currentButton = this.gadget.getPartById("hbox0").firstChild;
	}
	this.currentRow = this.currentButton.parent;
	var currentRowIndex = this.currentRow.id.substr(4); //hboxN
	//var currentColNumber = this.currentButton.id.substr(10); //wordButtonN
	
	var LEFT_KEY = 37;
	var UP_KEY = 38;
	var RIGHT_KEY = 39;
	var DOWN_KEY = 40;
	var ENTER_KEY = 13;
	
	var SPACE_KEY = 32;
	var SUBMIT_KEY = SPACE_KEY;
	
	switch (key.keyCode) {
		case 0:
			switch(key.virtualKeyCode) {
				case LEFT_KEY:
					this.currentButton.setSelected(false);
					if (shell.zlangoGlobals.rtl)
					{
						if (this.currentButton.nextSibling) {
							this.currentButton = this.currentButton.nextSibling;
						}
						else
						{
							this.currentButton = this.currentRow.firstChild;
						}
					}
					else
					{
						if (this.currentButton.previousSibling) {
							this.currentButton = this.currentButton.previousSibling;
						}
						else
						{
							this.currentButton = this.currentRow.lastChild;
						}
					}
					this.currentButton.setSelected(true);
					break;
					
				case RIGHT_KEY:
					this.currentButton.setSelected(false);
					if (shell.zlangoGlobals.rtl)
					{
						if (this.currentButton.previousSibling) {
							this.currentButton = this.currentButton.previousSibling;
						}
						else
						{
							this.currentButton = this.currentRow.lastChild;
						}
					}
					else
					{
						if (this.currentButton.nextSibling) {
							this.currentButton = this.currentButton.nextSibling;
						}
						else
						{
							this.currentButton = this.currentRow.firstChild;
						}
					}
					this.currentButton.setSelected(true);
					break;
					
					
					
					
				case UP_KEY:
					this.currentButton.setSelected(false);
					var id = this.currentButton.id;
					if (currentRowIndex == this.topRowIndex) {
						if (this.moveOneRowUpOrDown(-1)) {
							this.currentRow = this.currentRow.previousSibling;
							this.currentButton = this.currentRow.getPartById(id);
						}
					}
					else if (this.currentRow.previousSibling) {
						this.currentRow = this.currentRow.previousSibling;
						this.currentButton = this.currentRow.getPartById(id);
					}
					this.currentButton.setSelected(true);					
					break;
					
				case DOWN_KEY:
					this.currentButton.setSelected(false);
					var id = this.currentButton.id;
					if (currentRowIndex == this.topRowIndex + NUM_VISIBLE_ROWS - 1) {
						if (this.moveOneRowUpOrDown(1)) {
							this.currentRow = this.currentRow.nextSibling;
							this.currentButton = this.currentRow.getPartById(id);
							if (!this.currentButton)
							{
								this.currentRow = this.currentRow.previousSibling;
								this.currentButton = this.currentRow.getPartById(id);
							}
						}
					}
					else if (this.currentRow.nextSibling && this.currentRow.nextSibling.getPartById(id)) {
						this.currentRow = this.currentRow.nextSibling;
						this.currentButton = this.currentRow.getPartById(id);
						/*
						if (!this.currentButton)
						{
							this.currentButton = this.currentRow.firstChild;
						}
						*/
					}
					this.currentButton.setSelected(true);					
					break;
					
			}
			break;
			
		case ENTER_KEY:
		case SUBMIT_KEY:
			if (this.currentButton) {
				this.currentButton.onClick();
			}
			break;
	}
}

/**
 * Sets up the word icons panel based on category data.
 *
 */
component.prototype.setUpWordIconsPanel = function()
{
	this.gadget.hidden = true;
	this.cat = this.category.label;
	this.catID = this.category.id;
	this.catIndex = this.category.index;
	this.gadget.setAttribute("Category", this.cat);

	var categoryWordObjects = this.category.wordObjects;

	if (categoryWordObjects.length <= NUM_VISIBLE_ROWS * NUM_ICONS_PER_ROW) {
		this.scene.destroyBox(this.gadget.getPartById("moreButton"));
	}
	var hbox;
	var wordButtonIndex = 0;
	var numWords = Math.min(MAXIMUM_ICONS_PER_CATEGORY, categoryWordObjects.length);
	for (i = 0; i < numWords; i++) {
		if ( i % NUM_ICONS_PER_ROW == 0) {
			wordButtonIndex = 0;
			hbox = this.scene.createBox("wordRow");
			hbox.setAttribute("id", "hbox" + this.numRows);
			hbox.setStyle("position", "fixed");
			hbox.setStyle("left", "0");
			hbox.setStyle("top", this.numRows * ROW_HEIGHT);
			hbox.wordIconsPanel = this.gadget;
			this.gadget.getPartById("iconMenu").addChild(hbox);
			this.rowArray[this.numRows] = hbox;
			hbox.setAttribute("originalPosition", hbox.top);
			this.numRows++;
			hideWordRow (hbox);
		}
		var newBut = this.scene.createBox("wordButton");
		var word = categoryWordObjects[i];
		newBut.setAttribute("id", "wordButton" + wordButtonIndex);
		wordButtonIndex++;
		newBut.setAttribute("wordID", word.Id);
		newBut.setAttribute("iconVal", word.Image);	
		newBut.setAttribute("labelVal", word.Label);
		var tooltip = word.Label;
		if (word.Shortcut != "")
		{
			tooltip += " (" + word.Shortcut + ")";
		}
		newBut.setAttribute("tooltipVal", tooltip);
		newBut.panel = this.gadget;
		hbox.addChild(newBut);
	}
	
	// unblock initial rows
	for (i = 0; i < NUM_VISIBLE_ROWS && i < this.numRows; i++) {
		hbox = this.gadget.getPartById("hbox" + i);
		showWordRow (hbox);
	}
	this.gadget.hidden = false;
	
	// Automatically select the first word in the first row:
	this.currentButton = this.gadget.getPartById("hbox0").firstChild;
	this.currentButton.setSelected(true);
}

/**
 * When the more button is clicked, calls doRowTransition().
 *
 */
component.prototype.moreButton = function ()
{
	this.gadget.doRowTransition();
}

component.prototype.moveOneRowUpOrDown = function (direction)
{
	var success = false;
	// direction: -1 is up, 1 is down
	
	this.gadget.showAllRows();

	if (this.topRowIndex + direction >= 0 && this.topRowIndex + direction < this.numRows - 2) {
		var rowsToMove = direction;
		for (rowIndex = 0; rowIndex < this.numRows; rowIndex++) {
			// Commented out to remove single-row animation:
			//this.rowArray[rowIndex].setAttribute("upPosition", this.rowArray[rowIndex].top - (ROW_HEIGHT * rowsToMove));
			// Replaced above line with this line to set actual top, instead of triggering animation:
			this.rowArray[rowIndex].setStyle("top", this.rowArray[rowIndex].top - (ROW_HEIGHT * rowsToMove));
		}
		// Commented out block to remove single-row animation:
		/*
		for (rowIndex = 0; rowIndex < this.numRows; rowIndex++) {
			this.rowArray[rowIndex].setAttribute("moveRowUpAnimation", "true");
			this.liveAnimations++;				
		}
		*/
		this.topRowIndex += rowsToMove;
		success = true;
	} 
	
	// Added in to remove single-row animation:
	// -----------------------------------------------------------------------------------------
	// hide clipped rows so they don't trap events:
	for (i = 0; i < this.topRowIndex; i++) {
		hbox = this.gadget.getPartById("hbox" + i);
		hideWordRow (hbox);
	}
	for (i = this.topRowIndex + NUM_VISIBLE_ROWS; i < this.numRows; i++) {
		hbox = this.gadget.getPartById("hbox" + i);
		hideWordRow (hbox);
	}
	// -----------------------------------------------------------------------------------------
	
	return success;
}

function showWordRow (hbox)
{
	hbox.hidden = false;
}

function hideWordRow (hbox)
{
	hbox.hidden = true;
}

component.prototype.showAllRows = function ()
{
	for (i = 0; i < this.numRows; i++) {
		hbox = this.gadget.getPartById("hbox" + i);
		showWordRow (hbox);
	}
}

/**
 * Does a row transition animation.
 *
 */
component.prototype.doRowTransition = function ()
{
	this.gadget.showAllRows();
	
	/* IF AT BOTTOM, FLIP TO TOP */
	if (this.topRowIndex == this.numRows - NUM_VISIBLE_ROWS) {
		for (i = 0; i < this.numRows; i++) {
			this.rowArray[i].setAttribute("upPosition", this.rowArray[i].getAttribute("originalPosition"));
		}
		for (i = 0; i < this.numRows; i++) {
			this.rowArray[i].setAttribute("moveBackToTopAnimation", "true");
			this.liveAnimations++;				
		}
		this.topRowIndex = 0;
		
		// Set the more button to show a down arrow:
			this.gadget.getPartById("moreButton").setAttribute("imgSrc","#MORE_ARROW_DOWN");
		
	/* OTHERWISE, IF CAN MOVE THREE ROWS, DO SO */
	} else {
		var rowsToMove;
		if (this.topRowIndex < this.numRows - ((2 * NUM_VISIBLE_ROWS) - 1)) {
			rowsToMove = NUM_VISIBLE_ROWS;
		} else {
			var rowsToMove = this.numRows - this.topRowIndex - NUM_VISIBLE_ROWS;
		}

		/* SET FINAL POSITIONS FOR ROW TOPS FOR ANIMATION */
		for (i = 0; i < this.numRows; i++) {

			this.rowArray[i].setAttribute("upPosition", this.rowArray[i].top - (ROW_HEIGHT * rowsToMove));
		}
		/* TELL EACH ROW TO ANIMATE */
		for (i = 0; i < this.numRows; i++) {
			this.rowArray[i].setAttribute("moveRowUpAnimation", "true");
			this.liveAnimations++;				
		}
		this.topRowIndex += rowsToMove;
		
		var bWillBeShowingLastRow = this.topRowIndex + NUM_VISIBLE_ROWS >= this.numRows - 1;
		//this.scene.alert("this.topRowIndex: " + this.topRowIndex + " this.numRows: " + this.numRows, "bWillBeShowingLastRow: " + bWillBeShowingLastRow, 1);
		if (bWillBeShowingLastRow)
		{
			// Set the more button to show a down arrow:
			this.gadget.getPartById("moreButton").setAttribute("imgSrc","#MORE_ARROW_UP");
		}
		
	}
}

/**
 * When the close button is pressed, close the popup.
 *
 */
component.prototype.closeButton = function ()
{
	this.gadget.parentGadget.opened = false;
}

/**
 * When the animation is stopped, clear the animation attributes
 * so they can run again.
 *
 */
component.prototype.finishedScrolling = function ()
{
	this.liveAnimations--;
	/* CLEAR ANIMATION ATTRIBUTES SO THEY CAN RUN AGAIN */
	if (this.liveAnimations == 0) {
		for (i = 0; i < this.numRows; i++) {
			this.rowArray[i].removeAttribute("moveRowUpAnimation","false");
			this.rowArray[i].removeAttribute("moveBackToTopAnimation","false");
		}
	}
	
	// hide clipped rows so they don't trap events:
	for (i = 0; i < this.topRowIndex; i++) {
		hbox = this.gadget.getPartById("hbox" + i);
		hideWordRow (hbox);
	}
	for (i = this.topRowIndex + NUM_VISIBLE_ROWS; i < this.numRows; i++) {
		hbox = this.gadget.getPartById("hbox" + i);
		hideWordRow (hbox);
	}

}

