/**
 * Category Panel's constructor.
 *
 * @param  _gadget the panel gadget
 */
function component (_gadget) 
{
	this.gadget = _gadget; 
	this.type = null;
	this.numberOfButtons = 0;
	this.buttons = Array();
}

/**
 * When this gadget is first painted:
 *
 */
component.prototype.onPresented = function()
{
	this.gadget.onSelected();
}

/**
 * When this tab panel is selected, tells zlangoInterface that it is the current panel.
 *
 */
component.prototype.onSelected = function()
{
	this.zlangoInterfaceBox.setCategoryPanel(this.gadget);
}

/**
 * When instantiated, get the category list and create and set up
 * all necessary buttons.
 *
 */
component.prototype.onInitialized = function()
{
	this.zlangoRootBox = this.gadget.parent.parent.parent.gadgetParent.gadgetParent;
	this.zlangoInterfaceBox = this.gadget.parent.parent.parent.gadgetParent;
	/*
	if (this.zlangoInterfaceBox)
	{
		this.scene.alert(this.zlangoInterfaceBox.tag, "categoryPanel.js", 1);
	}
	*/
	
	//this.scene.alert("onInitialized", "onInitialized", 1);
	this.gadget.createButtons();
}


/**
 * When instantiated, get the category list and create and set up
 * all necessary buttons.
 *
 */
component.prototype.createButtons = function()
{
	this.type = this.gadget.getAttribute("type");
	var categoryList = null;
	switch (this.type) {
		case "COMPOSE":
			categoryList = this.zlangoRootBox.getComposeCategoryList();
			break;
		case "READY_MESSAGES":
			categoryList = this.zlangoRootBox.getReadyMessagesCategoryList();
			break;
		case "EXTRA":
			categoryList = this.zlangoRootBox.getExtraCategoryList();
			break;
	}
	
	this.numberOfButtons = categoryList.length;
	for (var i = 0; i < this.numberOfButtons; i++) {
		var category = categoryList[i];
		var newBut = this.scene.createBox("categoryButton");
		newBut.setAttribute("index", category.index);
		newBut.setAttribute("idVal", category.id);
		newBut.setAttribute("iconVal", category.imagePath);
		newBut.setAttribute("labelVal", category.label);
		newBut.setEventListener(this.gadget);
		this.gadget.addChild(newBut);
		this.buttons[i] = newBut;
	}
}

/**
 * Tells zlangoInterface that a button was rolled over. 
 *
 * @param  categoryButton the button which was selected
 */
component.prototype.onMouseOverCategoryButton = function(categoryButton)
{
	this.zlangoInterfaceBox.onMouseOverCategoryButton(categoryButton, this.type);
}


/**
 * Tells zlangoInterface that a button was selected.
 *
 * @param  categoryButton the button which was selected
 */
component.prototype.onClickCategoryButton = function(categoryButton)
{
	this.zlangoInterfaceBox.onClickCategoryButton(categoryButton, this.type);
}

/**
 * If user pressed 1-9, and that category exists, tells zlangoInterface that a
 * button was selected.
 *
 * @param  key the keyboard button which was pressed
 */
component.prototype.onReceivedKey = function(key)
{
	var keyCode = key.keyCode;
	var NUMBER_KEY_OFFSET = 48;
	var numberKey = keyCode - NUMBER_KEY_OFFSET;
	if ((numberKey >= 1) && (numberKey < this.numberOfButtons + 1) && (numberKey <= 9)) {
		this.zlangoInterfaceBox.onClickCategoryButton(this.buttons[numberKey-1], this.type);
		return true;
	}
	return false;
}