/**
 * Category Popup Gadget's constructor.
 *
 * @param  _gadget the popup gadget
 */

function component (_gadget) 
{ 
	this.gadget = _gadget; 
	this.category = null;
	this.location = null;
	this.contentGadgetName = null;
	this.alreadyUp = false;
	this.popupTabDownLocation = null;
	this.contentPanel = null;
	this.GADGET_HEIGHT = 270;
	this.BORDER_HEIGHT = 225;
	
	this.dragging = false;
}

/**
 * Time in milliseconds from initial upward display until the popup breaks
 * away from its tab.
 *
 */
var UPWARD_TAB_BREAKAWAY_DELAY_MS = 500;

/**
 * When instantiated, hide the gadget until it's set up.
 *
 */
component.prototype.onInitialized = function()
{
	this.zlangoInterfaceBox = this.gadget.gadgetParent;
	this.zlangoRootBox = this.zlangoInterfaceBox.gadgetParent;
	//this.gadget.hidden = true; // Commented this out when we made the menu popup on click instead of mouseOver. 20070827
}

/**
 * When close button is pressed, close the popup.
 *
 */
component.prototype.closeButton = function ()
{
	this.gadget.closePopup();
}

/**
 * When the popup is shut (no longer popped), hide it so it can be
 * set up again before next pop, and hide the separated tab.
 *
 */
component.prototype.onShut = function()
{
	this.gadget.hidden = true;
	this.zlangoInterfaceBox.hidePopupTab();

}

/**
 * Function which closes the popup.
 *
 */
component.prototype.closePopup = function ()
{
	this.gadget.opened = false;
}

/**
 * Handle key down events. If ESC, close the popup. If 1-9, pass
 * to zlangoInterface, otherwise pass to content panel
 *
 */
component.prototype.onKeyDown = function()
{
	var ESC_CODE = 27;
	var NUMBER_KEY_OFFSET = 48;
	var ENTER_KEY = 13;
	var eventObj = shell.currentEvent;
    if (eventObj)
    {
		var keyCode = eventObj.keyCode;
		if (keyCode == ESC_CODE) {
			this.gadget.closePopup();
		} else {
			var numberKey = keyCode - NUMBER_KEY_OFFSET;
			if ((numberKey >= 1) && (numberKey <= 9)) {
				this.zlangoInterfaceBox.onKeyDown();
			} else {
				this.contentPanel.onReceivedKey(eventObj);
			}
		}
	}
}

/**
 * When popped up, hide the gadget and set it up.
 *
 */
component.prototype.onPopped = function()
{
	this.gadget.hidden = true;
	this.gadget.displayCategoryPopup();
}

/**
 * Sets up the popup visually. Destroys the content
 * panel if it exists, creates a new one, sets it's location and category, calls
 * setUpPopup, positions the popup tab, and stops hiding the popup.
 *
 */
component.prototype.displayCategoryPopup = function()
{
	if (this.contentPanel) {
		this.scene.destroyBox(this.contentPanel);
	}
	this.contentPanel = this.scene.createBox(this.contentGadgetName);
	this.contentPanel.setAttribute("id", this.contentGadgetName);
	this.contentPanel.setStyle("position", "fixed");
	this.contentPanel.left = 14;
	this.contentPanel.top = 50;
	this.gadget.addChild(this.contentPanel);
	this.contentPanel.setCategory(this.category);
	this.gadget.setUpPopup();
	this.popupTabDownLocation = this.gadget.getPartById("popupTab").top;
	this.gadget.hidden = false;

}

/**
 * Sets the variable data values for the popup, including its border height,
 * gadget height, category, location, gadget name (readyMessagesPanel or 
 * wordIconsPanel). Also sets up the tab and the topline for upward position.
 *
 * @param  cat A Category object
 * @param  location XY coords of the category popup tab
 * @param  contentGadgetName The gadget type to use for the popup's content: either "wordIconsPanel" or "readyMessagesPanel"
 * @param  borderHeight Height of the visible border
 */
component.prototype.setValues = function (cat, location, contentGadgetName, borderHeight)
{
	this.BORDER_HEIGHT = borderHeight;
	this.GADGET_HEIGHT = borderHeight + 45;
	
	this.gadget.setStyle("height", this.GADGET_HEIGHT);
	this.gadget.getPartById("popupBorder").setStyle("height", this.BORDER_HEIGHT);	

	this.category = cat;
	this.location = location;
	this.contentGadgetName = contentGadgetName;
	
	var popupTab = this.gadget.getPartById("popupTab");
	if (popupTab) {
		popupTab.initializeImageStack (this.gadget.gadgetParent.gadgetParent.getComposeCategoryList(), this.gadget.gadgetParent.gadgetParent.getReadyMessagesCategoryList(), this.gadget.gadgetParent.gadgetParent.getExtraCategoryList());
		popupTab.setUpPopupTab(cat, location);
	}
	
	var topLine = this.gadget.getPartById("topLine");	
	topLine.setAttribute("txt", this.category.label);
	this.gadget.setTopLineImage(this.category.imagePath);
	//topLine.setAttribute("img", this.category.imagePath);

}

component.prototype.setTopLineImage = function (src)
{	
	var topLine = this.gadget.getPartById("topLine");
	var topLineImageStack = topLine.getBoxByTag("imageStack");
	if(!topLineImageStack.alreadyInitialized())
	{
		topLineImageStack.addCategoryImages (this.gadget.gadgetParent.gadgetParent.getComposeCategoryList());
		topLineImageStack.addCategoryImages (this.gadget.gadgetParent.gadgetParent.getReadyMessagesCategoryList());
		topLineImageStack.addCategoryImages (this.gadget.gadgetParent.gadgetParent.getExtraCategoryList());
	}
	topLineImageStack.setCurrentImage(src);
}

/**
 * Determines whether to pop upwards or downwards, repositions things
 * if necessary, and makes a delayed call to run the popup animation
 * if necessary.
 *
 */
component.prototype.setUpPopup = function ()
{	
	var parentX = this.location.parentX;
	var buttonX = this.location.buttonX;
	var buttonY = this.location.buttonY;
	
	if(shell.gConnector.GetIMClientData("LANG_ID")!= "he-il")
		this.gadget.left = parentX - 15;
	else this.gadget.left = parentX - 32;
	
	this.gadget.top = buttonY - this.gadget.parent.screenY;
		
	// Check if the popup menu reaches below the bottom of the screen work area:
	this.gadget.height = this.GADGET_HEIGHT;
	
    var screenBottom = appUtils.workAreaY + appUtils.workAreaHeight;
	var windowBottom = this.gadget.top + this.gadget.parent.screenY + this.gadget.boxHeight;
	var POP_UPWARD = (windowBottom > screenBottom);
	if (POP_UPWARD) {	
		// Position the popup UPWARD:
		var SHADOW_HEIGHT = 11;
		this.gadget.top -= (this.gadget.boxHeight - SHADOW_HEIGHT);
		if (this.alreadyUp || shell.zlangoGlobals.savedUserPopupPosition) { 
			var topLine = this.gadget.getPartById("topLine");
			topLine.hidden = false;
			this.gadget.getPartById("popupTab").hidden = true;
			this.gadget.resizeForUpwardMode(false);
			
			// If the user previously dragged the popup to a custom location, use that location:
			if (shell.zlangoGlobals.savedUserPopupPosition)
			{
			
				// If the user moved the ICQ box since last time Xtra was open, then reset our popup position to default:
				var savedICQX = this.zlangoRootBox.getPref("icq_x");
				var savedICQY = this.zlangoRootBox.getPref("icq_y");
				var icqBox = this.scene.rootBox;
				if (Number(savedICQX) != icqBox.screenX || Number(savedICQY) != icqBox.screenY)
				{
					this.zlangoRootBox.setPref("popup_x", "");
					this.zlangoRootBox.setPref("popup_y", "");
					
					this.gadget.top = -this.gadget.parent.frameY;
				}
				else
				{
					this.gadget.left = shell.zlangoGlobals.savedUserPopupPosition.x;
					this.gadget.top = shell.zlangoGlobals.savedUserPopupPosition.y;
				}
			}
			else
			{
				this.gadget.top = -this.gadget.parent.frameY;
			}
			this.zlangoInterfaceBox.showPopupTab(this.category, this.location);
		} else {
			this.gadget.height = this.GADGET_HEIGHT + 70;
			this.gadget.getPartById("popupTab").top = this.GADGET_HEIGHT - 8;
			this.gadget.getPartById("popupTab").getPartById("popup_tab").src = "#POPUP_TAB_DOWN";
			// delay the slideup and breakaway animation
			var dateObj = new Date();
			dateObj.setMilliseconds(dateObj.getMilliseconds() + UPWARD_TAB_BREAKAWAY_DELAY_MS);
			//debugger;
			this.timerObj = setTimeoutObject(this.gadget, "beginBreakawayAnimation", UPWARD_TAB_BREAKAWAY_DELAY_MS);
		}
	} else {
		// NOT POP_UPWARD
		var topLine = this.gadget.getPartById("topLine");
		topLine.hidden = true;
		var popupTab = this.gadget.getPartById("popupTab");
		popupTab.top = 0;
		popupTab.setShape("#POPUP_TAB");
		popupTab.hidden = false;
		
		this.zlangoInterfaceBox.hidePopupTab();
	}
}

/**
 * Called on a delay, this function separates the popup tab and begins
 * the upward moving animation for the popup.
 *
 */
component.prototype.beginBreakawayAnimation = function ()
{
	this.alreadyUp = true;
	this.zlangoInterfaceBox.showPopupTab(this.category, this.location);
	this.gadget.getPartById("popupTab").hidden = true;
	var endLocation = -this.gadget.parent.frameY;
	this.gadget.setAttribute("endLocation", endLocation);
	this.gadget.setAttribute("popUpAnimation", "true");
}

/**
 * Resizes the window for upward mode and moves the content appropriately.
 * Does so either instantaneously, or with animation
 *
 * @param  isAnimating If true: animate. If false: don't.
 */
component.prototype.resizeForUpwardMode = function (isAnimating)
{
	var topLine = this.gadget.getPartById("topLine");	
	var GROW_BY = 42;
	
	if (isAnimating) {
		this.contentPanel.setAttribute("endLocation", 90);
		this.contentPanel.setAttribute("slideDownAnimation", "true");
		this.gadget.getPartById("popupBorder").setAttribute("endHeight", this.BORDER_HEIGHT + GROW_BY);
		this.gadget.getPartById("popupBorder").setAttribute("windowGrowAnimation", "true");
		topLine.hidden = false;
	} else {
		this.contentPanel.top = 90;
		this.gadget.getPartById("popupBorder").setStyle("height", this.BORDER_HEIGHT + GROW_BY);
	}
	
	this.gadget.setStyle("height", this.GADGET_HEIGHT + GROW_BY);
}

/**
 * Called when animation is complete. Removes the animation attribute
 * so animation can be restarted. If it was the popup moving upward
 * animation, sets up the topLine data and starts its animation.
 *
 *
 */
component.prototype.onAnimationExit = function ()
{
	if (this.gadget.getAttribute("popUpAnimation") == "true") {
		//this.alreadyUp = true;
		this.gadget.removeAttribute("popUpAnimation", "false");
		var topLine = this.gadget.getPartById("topLine");	
		topLine.setAttribute("txt", this.category.label);
		//topLine.setAttribute("img", this.category.imagePath);
		this.gadget.setTopLineImage(this.category.imagePath);
		this.gadget.resizeForUpwardMode(true);
	}
}

component.prototype.onDragStart = function (offsetX, offsetY)
{
	this.dragging = true;
	this.dragOffsetX = offsetX;
	this.dragOffsetY = offsetY;
}

component.prototype.onDrag = function (eventObj)
{
	if (this.dragging)
	{
		var x = eventObj.screenX;
		var y = eventObj.screenY;
		var parentX = this.gadget.parent.screenX;
		var parentY = this.gadget.parent.screenY;
		this.gadget.setStyle("left", x - parentX - this.dragOffsetX);
		this.gadget.setStyle("top", y - parentY - this.dragOffsetY);
	}
}

component.prototype.onDragEnd = function ()
{
	this.dragging = false;
	shell.zlangoGlobals.savedUserPopupPosition = new Object();
	shell.zlangoGlobals.savedUserPopupPosition.x = this.gadget.boxX;
	shell.zlangoGlobals.savedUserPopupPosition.y = this.gadget.boxY;
	
	// Save position in user's local prefs:
	this.zlangoRootBox.setPref("popup_x", shell.zlangoGlobals.savedUserPopupPosition.x);
	this.zlangoRootBox.setPref("popup_y", shell.zlangoGlobals.savedUserPopupPosition.y);
	
	// Save position of ICQ window also:
	var icqBox = this.scene.rootBox;
	this.zlangoRootBox.setPref("icq_x", icqBox.screenX);
	this.zlangoRootBox.setPref("icq_y", icqBox.screenY);
}

