/**
 * Shortcut Popup Gadget's constructor.
 *
 * @param  _gadget the popup gadget
 */
function component (_gadget) 
{ 
	this.gadget = _gadget; 
	this.selectedLetterLink = null;
	this.alreadyPopped = false;

}

var ICONS_PER_ROW = 6;

component.prototype.onInitialized = function()
{
}

component.prototype.setUp = function()
{
	this.gadget.hidden = true;
	this.gadget.top = -442;
	this.gadget.left = 16;
	
	var letterBox = this.gadget.getPartById("letterBox");
	
	var letterLink;
	for (var i = 0; i < shell.zlangoGlobals.letters.length; i++) {
		letterLink = this.scene.createBox("letterLink");
		letterLink.setAttribute("letter", shell.zlangoGlobals.letters.charAt(i));
		letterLink.setAttribute("id", "letterLink" + shell.zlangoGlobals.letters.charAt(i).toLowerCase().charCodeAt(0));
		letterLink.top = 0;
		letterLink.left = 15 * i;
		letterBox.addChild(letterLink);		
		if (i == 0) {
			this.selectedLetterLink = letterLink;
		}
	}
	this.selectedLetterLink.getPartById("theLetter").setStyle("textColor", 
			this.selectedLetterLink.getAttribute("selectedColor"));

	this.selectedLetterLink.getPartById("theLetter").setStyle("fontUnderline", "false");
	this.gadget.showListForLetterIndex(0);

	this.gadget.hidden = false;

	this.gadget.setAttribute("letterValue", shell.zlangoGlobals.letters.charAt(0));

}

component.prototype.letterLinkClicked = function (letterLink)
{
	if (letterLink == this.selectedLetterLink) {
		return false;
	}
	var letter = letterLink.getAttribute("letter").toUpperCase();
	this.gadget.setAttribute("letterValue", letter);
	
	this.gadget.showListForLetterIndex(shell.zlangoGlobals.letters.indexOf(letter.toLowerCase()));
	
	// Deselect the old letterLink:
	this.selectedLetterLink.getPartById("theLetter").setStyle("textColor", 
			this.selectedLetterLink.getAttribute("nonSelectedColor"));
	this.selectedLetterLink.getPartById("theLetter").setStyle("fontUnderline", "true");
	
	this.selectedLetterLink = letterLink;
	
	// Select the new letterLink:
	this.selectedLetterLink.getPartById("theLetter").setStyle("textColor", 
			this.selectedLetterLink.getAttribute("selectedColor"));
	this.selectedLetterLink.getPartById("theLetter").setStyle("fontUnderline", "false");

	
}

component.prototype.showListForLetterIndex = function (letterIndex)
{
	var wordList = this.gadget.gadgetParent.gadgetParent.getWordListForShortcuts()[letterIndex];
	var scrollingList = this.gadget.getPartById("scrollingList");
	var shortcutIcon;
	var firstChild = scrollingList.firstChild;
	while (firstChild) {
		scrollingList.deleteChild(firstChild);
		firstChild = scrollingList.firstChild;
	}
	var hbox;
	for (var i = 0; i < wordList.length; i++) {
		if (i % ICONS_PER_ROW == 0) {
			hbox = this.scene.createBox("hbox");
			hbox.setStyle("hSpace", 8);
			scrollingList.addChild(hbox);
		}
		shortcutIcon = this.scene.createBox("shortcutIcon");
		shortcutIcon.setAttribute("wordID", wordList[i].Id);
		shortcutIcon.setAttribute("iconVal", wordList[i].Image);
		shortcutIcon.setAttribute("labelVal", wordList[i].Label);
		shortcutIcon.setAttribute("shortcutVal", wordList[i].Shortcut);
		//"z" + wordList[i].Label.charAt(0).toLowerCase());
		hbox.addChild(shortcutIcon);
	}
	
}
/**
 * When close button is pressed, close the popup.
 *
 */
component.prototype.closeButton = function ()
{
	this.gadget.closePopup();
}

component.prototype.onShut = function()
{
}

/**
 * Function which closes the popup.
 *
 */
component.prototype.closePopup = function ()
{
	this.gadget.opened = false;
}

/**
 * Handle key down events.
 *
 */
component.prototype.onKeyDown = function()
{
	this.gadget.handleKeyDown();
}

component.prototype.handleKeyDown = function()
{
	var eventObj = shell.currentEvent;
	if (eventObj) {
		var keyCode = eventObj.keyCode; // Should be unicode value. Could be uppercase or lowercase.
		if (keyCode == 0)
		{
			// Handle arrow keys and ESC or any other special keys...
			return;
		}
		
		var theChar = String.fromCharCode(keyCode);
		var lowerCaseCharCode = theChar.toLowerCase().charCodeAt(0);
		
		var theLink = this.gadget.getPartById("letterLink" + lowerCaseCharCode);
		if (theLink) {
			this.gadget.letterLinkClicked(theLink);
		}
		else
		{
			wildcard = shell.zlangoGlobals.letters.charAt(shell.zlangoGlobals.letters.length - 1);
			theLink = this.gadget.getPartById("letterLink" + wildcard.charCodeAt(0));
			if (theLink) {
				this.gadget.letterLinkClicked(theLink);
			}
		}
	}
}



/**
 * When popped up, hide the gadget and set it up.
 *
 */
component.prototype.onPopped = function()
{
	if (!this.alreadyPopped) {
		this.gadget.setUp();
		this.alreadyPopped = true;
	}
}

