/**
 * Tab Set's constructor.
 *
 * @param  _gadget the tab set gadget
 */
function component (_gadget) 
{ 
	this.gadget = _gadget;
	this.selectedTab = null; 
}

/**
 * Initialize the tab set by registering with its parent.
 *
 * @param  _gadget the tab panel gadget.
 */
component.prototype.onInitialized = function ()
{
	this.gadget.parent.setTabSet(this.gadget);
}

/**
 * When tab is clicked, call selectTab().
 *
 * @param  tab the tab that was clicked.
 */
component.prototype.clickedTab = function (tab)
{
	this.gadget.selectTab(tab);
}

/**
 * Select a tab and show its panel.
 *
 * @param  tab the tab to select.
 */
component.prototype.selectTab = function (tab)
{
	if (this.selectedTab == tab) {
		return;
	}
	
	if (this.selectedTab) {
		this.selectedTab.zIndex = Number(this.selectedTab.getAttribute("index"));
		this.selectedTab.setSelectedState(false);
		this.selectedTab.ignoreEvents = false;
		this.selectedTab.blockEvents = false;
	}
	tab.zIndex = 99;
	this.selectedTab = tab;	
	this.selectedTab.setSelectedState(true);
	this.selectedTab.ignoreEvents = true;
	this.selectedTab.blockEvents = true;
	
	var tempTab = this.gadget.firstChild;
	var tabIndex = 0;
	while (tempTab != null) {
		if (tempTab.tag != "tab") {
			tempTab = tempTab.nextSibling;
			continue;
		}
		if (tab == tempTab) {
			this.gadget.parent.showPanel(tabIndex);
			break;
		}
		tabIndex++;
		tempTab = tempTab.nextSibling;
	}	
}

/**
 * Get the currently selected tab.
 *
 * @return  the currently selected tab.
 */
component.prototype.getSelectedTab = function ()
{
	if (this.selectedTab) {
		return this.selectedTab;
	} else {
		var currentTab = this.gadget.firstChild;
		while (currentTab != null) {
			if (currentTab.getAttribute("selected") == "true") {
				this.gadget.selectTab(currentTab);
				return currentTab;
			}
			currentTab = currentTab.nextSibling;
		}
	}
	// nothing found, so default to first
	this.gadget.selectTab(this.gadget.firstChild);
	return this.gadget.firstChild;
}