/******************************************
// XtrazBroadcasterClass
// ** Interface **
// addListener (oListener, ... list of event classes ...)
// removeListener (oListener, ... list of event classes ...)
//
// ** Dependencies **
// XtrazDebug
// XtrazUtils
*******************************************/

var XtrazBroadcaster;

if (top.XtrazBroadcaster) {
	XtrazBroadcaster = top.XtrazBroadcaster;
}
else {
	function XtrazBroadcasterClass() {
		this.iQueues = new Array;
	}
	
	XtrazBroadcasterClass.prototype = new Object;
	XtrazBroadcasterClass.eventClassAll = "OxDFL";
	
	/******************************************
	// Init
	*******************************************/
	XtrazBroadcasterClass.prototype.init = function() {
	}	
		 
	/******************************************
	// AddEventClass
	// returns last element inserted
	*******************************************/
	XtrazBroadcasterClass.prototype.addEventClass = function(/* eventClass, ... */) {
		XtrazDebug.assert((arguments.length > 0), "no event classes passed to addeventclass"); 
		var ret;
		for (var i = 0; i < arguments.length; i++) {
			var theClass = arguments[i];
			if (theClass) {
				ret = XtrazUtils.findInArray(this.iQueues, theClass, "EventClass");
				if (ret != null) {
					XtrazDebug.alert(">>addEventClass: class" , theClass , "already exists")
				}
				else {
					this.iQueues.push(this.makeListenerQueue(theClass));
					ret = this.iQueues[this.iQueues.length -1];
				}
			}
			else {
				XtrazDebug.assert(false, "XB addEventClass: bad event class", theClass);
			}
		}
		return ret;
	}
	
		/******************************************
		// RemoveEventClass
		*******************************************/
	XtrazBroadcasterClass.prototype.removeEventClass = function (/* eventClass, ... */) {
		XtrazDebug.assert((arguments.length > 0), "no event classes passed to addeventclass"); 
		for (var i = 0; i < arguments.length; i++) {
			var theClass = arguments[i];
			XtrazUtils.removeFromArray(this.iQueues, theClass, "EventClass");
		}
	}
		
		/******************************************
		// addListener
		*******************************************/
	XtrazBroadcasterClass.prototype.addListener = function (inListener /* eventClass, ... */) {
		var queue, theClass;
	
		for (var i = 1; i < arguments.length; i++) { // iterate on all arguments after listener
			theClass = arguments[i];
			queue = XtrazUtils.findInArray(this.iQueues, theClass, "EventClass");
			if (! queue) { // class does not exist
				queue = this.addEventClass(theClass);
			}
				
			if (queue) {
				if (XtrazUtils.arrayContains(queue.iListeners, inListener)) {
					XtrazDebug.trace("addListener: listener already exists in class " , theClass);
				}
				else {
					queue.iListeners.push(inListener);
				}
			}
			else {
	//			_global.myTrace(0,"addListener: class " , theClass , " does not exists")
			}
		}
	}
	
		/******************************************
		// RemoveListener
		*******************************************/
	XtrazBroadcasterClass.prototype.removeListener = function (inListener /* eventClass, ... */) {
			var i = 0;
			if (arguments.length == 1) { // remove from all queues
				this.forgetListener(inListener);
				return;
			}
		
			for (var i = 1; i < arguments.length; i++) { // iterate on all arguments after listener
				var theClass = arguments[i];
				var queue = XtrazUtils.findInArray(this.iQueues, theClass, "EventClass");
				if (queue) {
					XtrazUtils.removeFromArray(queue.iListeners, inListener);
				}
				else {
		//			_global.myTrace(1,"RemoveListener: class " , theClass , " does not exists")
				}
			}
		}
		
		
		/******************************************
		// forgetListener
		*******************************************/
	XtrazBroadcasterClass.prototype.forgetListener = function (inListener) {
		for (var i = 0; i < this.iQueues.length; i++){
			var queue = this.iQueues[i];
			XtrazUtils.removeFromArray(queue.iListeners, inListener);
		}
	}
		
		/******************************************
		// ForgetListener
		*******************************************/
	XtrazBroadcasterClass.prototype.getEventClassListeners = function (/* event classes */) {
		var ret = new Array;
	
		XtrazDebug.assert(arguments.length > 0, "getEventClassListeners: no event classes");
		
		var j = 0;
		var queue = XtrazUtils.findInArray(this.iQueues, XtrazBroadcasterClass.eventClassAll, "EventClass");
		if (queue) {
			for (j = 0; j < queue.iListeners.length; j++) {
				var listener = queue.iListeners[j];
				if (! XtrazUtils.arrayContains(ret, listener))
					ret.push(listener);
			}
		}
		
		for (var i = 0; i < arguments.length; i++) {
			var theClass = arguments[i];
			queue = XtrazUtils.findInArray(this.iQueues, theClass, "EventClass");
			if (queue) {
				for (j = 0; j < queue.iListeners.length; j++) {
					var listener = queue.iListeners[j];
					if (! XtrazUtils.arrayContains(ret, listener))
						ret.push(listener);
				}
			}
	
		}
		return ret;
	}
		
	XtrazBroadcasterClass.prototype.makeListenerQueue = function(inClass) {
		var ret = new Object;
		ret.iListeners = new Array;
		ret.EventClass = inClass;
		
		return ret;
	}
	
	XtrazBroadcasterClass.prototype.broadcast = function(msg, param1, param2, param3, param4, param5) {
	//	XtrazDebug.trace(3, "broadcasting", msg);
		var listeners = this.getEventClassListeners(msg);
		var len = listeners.length;
		var listener;
		for (var i = 0; i < len; i++) {
			listener = listeners[i];
			XtrazDebug.trace(3, "broadcasting", msg, "to listener", listener.ProgID);
			try {
				listener.listenToMessage(msg, param1, param2, param3, param4, param5);
			}
			catch (e) {
				var id = '(no id)';
				if (typeof(listener.ProgID) != 'undefined')
					id = listener.ProgID;
				XtrazDebug.trace("broadcast failed, message", msg, "index", i, id, e);
			}
		}
	}
	
	XtrazBroadcasterClass.prototype.dump = function() {
		XtrazDebug.trace("listenertable:", this.iQueues);
	}
	
	XtrazBroadcaster = new XtrazBroadcasterClass();
}