var undefined;

var XtrazUI = new Object;
XtrazUI.anonFunctions = null;

XtrazUI.showObject = function(obj, bShow, bInherit) {
	if (arguments.length < 3) {
		bInherit = false;
		if (arguments.length < 2)
			bShow = true;
	}
	obj = DOMUtils.getObject(obj);
	if (obj) {
		var style = obj.style;
		if (style) {
			var str;
			if (bShow) {
				if (bInherit)
					str = "inherit";
				else
					str = "visible";
			}
			else {
				str = "hidden";
			}
			style.visibility = str;
		}
	}
}

XtrazUI.collapseObject = function(obj, bCollapse) {
	if (arguments.length < 2)
		bCollapse = true;
	obj = DOMUtils.getObject(obj);
	if (obj) {
		var style = obj.style;
		if (style) {
			style.display = bCollapse ? "none" : "block";
		}
	}
}

XtrazUI.getObjectPosition = function(obj, bRelative /* defaults to false */) {
	obj = DOMUtils.getObject(obj);
	XtrazDebug.assert(obj, "bad object passed to getObjectPosition");
	var ret = { left: 0, top: 0 };
	if (obj) {
		while (obj.offsetParent) {
			ret.left += parseInt(obj.offsetLeft);
			ret.top += parseInt(obj.offsetTop);
			obj = obj.offsetParent;
		}
	}
	return ret;
}

XtrazUI.moveObjectBy = function(obj, dx, dy) {
	obj = DOMUtils.getObject(obj);
	var top, left;
	if (obj) {
		var bRelative = false;
		try {
			bRelative = (obj.style.position == "relative");
		}
		catch (e) {
		}
		
		var pos = XtrazUI.getObjectPosition(obj, bRelative);
		XtrazDebug.trace("moveobjectby: orig position is", pos);
		XtrazUI.moveObjectTo(obj, pos.left + dx, pos.top + dy);
	}
}

XtrazUI.moveObjectTo = function(obj, x, y) {
	obj = DOMUtils.getObject(obj);
	if (obj && obj.style) {
		obj.style.left = String(x) + "px";
		obj.style.top = String(y) + "px";
	}
	else {
		XtrazDebug.trace("moveObjectBy: no object", obj);
	}
}

XtrazUI.getObjectSize = function(obj) {
	obj = DOMUtils.getObject(obj);
	var ret = {width : 0, height : 0};
	if (obj) {
		if (obj.clientWidth) {
			ret.height = obj.clientHeight;
			ret.width = obj.clientWidth;
		}
		else if (obj.style) {
			if (obj.style.width) {
				ret.height = obj.style.height;
				ret.width = obj.style.width;
			}
		}
	}
	return ret;
}



XtrazUI.getWindowSize = function() {
	var ret = new Object;
	if (window.innerHeight) {
		ret.height = window.innerHeight;
		ret.width = window.innerWidth;
		ret.offsetTop = window.pageYOffset;
		ret.offsetLeft = window.pageXOffset;
	}
	else if (document.documentElement && document.documentElement.clientHeight) {
		ret.height = document.documentElement.clientHeight;
		ret.width = document.documentElement.clientWidth;
        ret.offsetLeft = document.documentElement.scrollLeft;
        ret.offsetTop = document.documentElement.scrollTop;
	}
	else if (document.body) { // other Explorers
		ret.height = document.body.clientHeight;
		ret.width = document.body.clientWidth;
        ret.offsetLeft = document.body.scrollLeft;
        ret.offsetTop = document.body.scrollTop;
	}
	
	return ret;
}

XtrazUI.pinInWindow = function(obj, x, y) {
	var w = obj.clientWidth, h = obj.clientHeight;
	var dim = XtrazUI.getWindowSize();
	x -= dim.offsetLeft;
	y -= dim.offsetTop;
	var diff = dim.width - (x + w) - 10;
	if (diff < 0) {
		x += diff;
	}
	if (x < 5) {
		x = 5;
	}

	var diff = dim.height - (y + h) - 10;
	if (diff < 0) {
		y += diff;
	}
	if (y < 5) {
		y = 5;
	}
	
	x += dim.offsetLeft;
	y += dim.offsetTop;
	
	return { x: x, y : y};	
}

XtrazUI.getEventXY = function(e) {
	var posx = 0;
	var posy = 0;
	try {
		if (!e) {
			e = window.event;
		}
		if (e.pageX || e.pageY)
		{
			posx = e.pageX;
			posy = e.pageY;
		}
		else if (e.clientX || e.clientY)
		{
				posx = e.clientX + document.body.scrollLeft;
				posy = e.clientY + document.body.scrollTop;
		}
	}
	catch (e) {
		XtrazDebug.trace("getEventXY exception", e);
	}
	// posx and posy contain the mouse position relative to the document
	// Do something with this information
	return {x : posx, y : posy};
}

XtrazUI.focusObject = function(sObject, bFocus) {
	if (arguments.length < 2)
		bFocus = true;
	var obj = DOMUtils.getObject(sObject);
	XtrazDebug.assert(obj, "focusObject: bad object", sObject);
	try {
		if (bFocus) 
			obj.focus();
		else 
			obj.blur();
	}
	catch (e) {
	}
}

XtrazUI.disableControl = function(sControl, bDis) {
	var control = DOMUtils.getObject(sControl);
	XtrazDebug.assert(control, "disableControl: bad control", sControl);
	try {
		if (typeof(control.enable) == 'function') {
			control.enable(! bDis);
		}
		else {
			if (bDis) 
				control.setAttribute("disabled", true);
			else
				control.removeAttribute("disabled");
		}
	}
	catch (e) {
		XtrazDebug.trace("disable control", e);
	}
}

XtrazUI.removeEventListener = function(obj, evt, fnc) {
	if (obj.removeEventListener){
		obj.removeEventListener(evt, fnc, false);
	} 
	else if (obj.detachEvent) {
		obj.detachEvent("on" + evt, fnc);
	}
}

XtrazUI.addEventListener = function(obj, evt, fnc, useCapture){
	if (arguments.length < 4) 
		useCapture=false;
	if (obj.addEventListener){
		obj.addEventListener(evt, fnc, useCapture);
	} 
	else if (obj.attachEvent) {
		obj.attachEvent("on"+evt,fnc);
	}
	return true;
}

XtrazUI.addEventListenerObject = function(obj, evt, listener, sFunc){
	var func = function(param) {
		var me = arguments.callee;
		me.commander[me.sCallback](param);
	}
	func.commander = listener;
	func.sCallback = sFunc;
	XtrazUI.addEventListener(obj, evt, func);
	XtrazUI.registerAnonListener(obj, evt, func);
	return true;
}

XtrazUI.registerAnonListener = function(obj, evt, func) {
	if (! XtrazUI.anonFunctions) {
		XtrazUI.anonFunctions = new Array;
		XtrazUI.addEventListener(window, 'unload', XtrazUI.clearAnonHandlers);
	}
	XtrazUI.anonFunctions.push({o: obj, e : evt, f: func});
}

XtrazUI.clearAnonHandlers = function() {
	if (XtrazUI.anonFunctions) {
		var a = XtrazUI.anonFunctions;
		var len = a.length;
		var r, o, f;
		var ex = 0;
		for (var i = 0; i < len; i++) {
			r = a[i];
			o = r.o;
			f = r.f;
			try {
				if (o.removeEventListener){
					o.removeEventListener(r.e, f, false);
				} 
				else if (o.detachEvent) {
					o.detachEvent("on" + r.e, f);
				}
				f.commander = null;
				f.sCallback = null;
			}
			catch (e){ex++}
		}
//		alert("done removing " + len + " anon handlers " + ex + 'exceptions');
	}
}

XtrazUI.getRadioValueFromForm = function(formObj, radioName) {
	if (formObj) {
		var radio = formObj[radioName];
		XtrazDebug.assert(radio, "no radio named", radioName);
		if (radio)
			ret = XtrazUI.getRadioValue(radio);
	}
	return ret;
}

XtrazUI.getRadioValue = function(radioObj) {
	if (radioObj && radioObj.length) {
		var len = radioObj.length;
		for (var i = 0; i < len; i++) {
			radio = radioObj[i];
			if (radio.checked) {
				return radio.value;
			}
		}
	}
	return "";
}

XtrazUI.setRadioValueInForm = function(formObj, radioName, inValue) {
	if (formObj && formObj[radioName] && formObj[radioName].length) {
		var len = formObj[radioName].length;
		var radio;
		var radioObj = formObj[radioName];
		for (var i = 0; i < len; i++) {
			radio = radioObj[i];
			radio.checked = (radio.value == inValue);
		}
	}
}

XtrazUI.getSelectValue = function(oSelect) {
	oSelect = DOMUtils.getObject(oSelect);
	var ret = null;
	if (oSelect) {
		var option = oSelect.options[oSelect.selectedIndex];
		if (option) {
			ret = option.value;
			if (ret == null || ret == undefined || ret == '')
				ret = option.text;
		}
	}
	return ret;
}

XtrazUI.getCheckValue = function(oCheck) {
	var check = DOMUtils.getObject(oCheck);
	XtrazDebug.assert(check, "getCheckValue", oCheck);
	return check.checked;
}

XtrazUI.setCheckValue = function(oCheck, bValue) {
	var check = DOMUtils.getObject(oCheck);
	XtrazDebug.assert(check, "setCheckValue", oCheck);
	bValue = ! (! bValue);
	check.checked = bValue;
}



XtrazUI.setSelectValue = function(oSelect, sValue) {
	oSelect = DOMUtils.getObject(oSelect);
	if ((! oSelect) || (sValue == null) || (sValue == undefined) || (sValue == ''))
		return;
	sValue = String(sValue).toLowerCase();
	var options = oSelect.options;
	var len = oSelect.options.length;
	var val;
	for (var i = 0; i < len; i++) {
		val = options[i].value;
		if (typeof(val) == "string" && val.length > 0) {
			val = val.toLowerCase();
		}
		else {
			val = options[i].text;
			if (typeof(val) == "string")
				val = val.toLowerCase();
		}
		if (val == sValue) {
			oSelect.selectedIndex = i;
			break;
		}
	}
}

XtrazUI.setTextValue = function(oInput, sValue) {
	var input = DOMUtils.getObject(oInput);
	XtrazDebug.assert(input, "setInputValue", oInput);
	var tag = input.tagName.toLowerCase();
	switch (tag) {
		case "input":
			input.value = sValue;
			break;
		default:
			input.innerHTML = sValue;
			break;
	}
}

XtrazUI.getTextValue = function(oInput) {
	var input = DOMUtils.getObject(oInput);
	var ret = '';
	XtrazDebug.assert(input, "getInputValue", oInput);
	var tag = input.tagName.toLowerCase();
	switch (tag) {
		case "input":
			ret = input.value;
			break;
		default:
			ret = input.innerHTML;
			break;
	}
	return ret;
}


XtrazUI.getEventKey = function(e) {
	var key = 0;
	if (! e)
		e = window.event;
	if (e) {
		key = (e.charCode)?e.charCode:
		((e.keyCode)?e.keyCode:((e.which)?e.which:0));
	}
	return key;
}