/**
 * Zlango Interface's constructor.
 *
 * @param  _gadget the zlango interface gadget
 */
function component (_gadget) { 
	this.gadget = _gadget; 
	this.categoryPanel = null;
	this.currentCategory = null;
	this.isDemoOpen = false;
	this.popupTimerID = null;
	this.timeoutCategoryButton = null;
	this.timeoutCategoryType = null;
	this.timerObj = null;
}

/**
 * Initializes the zlango interface object, and give it focus. Hides
 * the popup tab.
 *
 */
component.prototype.onInitialized = function()
{
	this.zlangoRootBox = this.gadget.gadgetParent;
	
	this.zlangoRootBox.setUpLogger();

	/*
	if (this.zlangoRootBox)
	{
		this.scene.alert(this.zlangoRootBox.tag, "zlangoRootBox", 1);
	}
	*/

	var popupTab = this.gadget.getPartById("zIpopupTab");
	popupTab.hidden = true;
	popupTab.setShape("#POPUP_TAB_DETACHED");
	popupTab.initializeImageStack (this.zlangoRootBox.getComposeCategoryList(), this.zlangoRootBox.getReadyMessagesCategoryList(), this.zlangoRootBox.getExtraCategoryList());

	this.gadget.focus();
}

/**
 * Sets the category panel.
 *
 * @param  obj the category panel
 */
component.prototype.setCategoryPanel = function(obj)
{
	this.categoryPanel = obj;
}

/**
 * Gets data for a Compose category.
 *
 * @param  categoryID the id of the compose category.
 */
component.prototype.getComposeCategory = function (categoryID)
{
	return this.gadget.gadgetParent.getComposeCategory(categoryID);
}

component.prototype.getExtraCategory = function (categoryID)
{
	return this.gadget.gadgetParent.getExtraCategory(categoryID);
}

/**
 * Gets data for a Ready Messages category.
 *
 * @param  categoryID the id of the ready messages category.
 */
component.prototype.getReadyMessagesCategory = function (categoryID)
{
	return this.gadget.gadgetParent.getReadyMessagesCategory(categoryID);
}

/**
 * Opens the Compose Category Popup.
 *
 * @param  categoryID the id of the compose category.
 * @param  location layout data for the popup.
 */
component.prototype.openCategoryIconsMenu = function(categoryID, location)
{
	
	var category = this.gadget.getComposeCategory(categoryID);
	var thePopup = this.gadget.getPartById("categoryPopupGadget");

	if (category == this.currentCategory && thePopup.opened) {
		return;
	}
	this.currentCategory = category;
	
	thePopup.setValues(category, location, "wordIconsPanel", 225);

	// If the popup is already open, then we just need to show the correct category
	if (thePopup.opened) {
		thePopup.displayCategoryPopup();
	} else {
		// Changing this attribute triggers Boxely to open the popup:
		thePopup.opened = true;
	}
}

component.prototype.openCategoryExtraIconsMenu = function(categoryID, location)
{
	
	var category = this.gadget.getExtraCategory(categoryID);
	var thePopup = this.gadget.getPartById("categoryPopupGadget");

	if (category == this.currentCategory && thePopup.opened) {
		return;
	}
	this.currentCategory = category;
	
	thePopup.setValues(category, location, "wordIconsPanel", 225);

	// If the popup is already open, then we just need to show the correct category
	if (thePopup.opened) {
		thePopup.displayCategoryPopup();
	} else {
		// Changing this attribute triggers Boxely to open the popup:
		thePopup.opened = true;
	}
}

/**
 * Opens the Ready Messages Category Popup.
 *
 * @param  categoryID the id of the ready messages category.
 * @param  location layout data for the popup.
 */
component.prototype.openReadyMessagesMenu = function(categoryID, location)
{
	var category = this.gadget.getReadyMessagesCategory(categoryID);
	var thePopup = this.gadget.getPartById("categoryPopupGadget");

	if (category == this.currentCategory && thePopup.opened) {
		return;
	}
	this.currentCategory = category;
	
	thePopup.setValues(category, location, "readyMessagesPanel", 260);

	if (thePopup.opened) {
		thePopup.displayCategoryPopup();
	} else {
		thePopup.opened = true;
	}

}

/**
 * Responds to a category button rollover: Begin a timer. When the timer completes, if the mouse is still over the button, then open the category..
 *
 * @param  categoryButton the button that was selected.
 * @param  categoryType the type of category (Compose, Ready Messages).
 */
component.prototype.onMouseOverCategoryButton = function (categoryButton, categoryType)
{
	this.timeoutCategoryButton = categoryButton;
	this.timeoutCategoryType = categoryType;

	var thePopup = this.gadget.getPartById("categoryPopupGadget");
	if (thePopup.opened)
	{
		if (this.timerObj != null)
		{
			this.timerObj.cancel();
		}
		this.timerObj = setTimeoutObject(this.gadget, "onTimeoutCategoryButton", 150);
		return;
	}
}

/**
 * Callback  executed after delay after mouseOver on category button when another category is already open.
 *
 */

component.prototype.onTimeoutCategoryButton = function ()
{
    this.timerObj = null;
    
	// If mouse is still over the original button
	var buttonLeft = this.timeoutCategoryButton.screenX;
	var buttonTop = this.timeoutCategoryButton.screenY;
	var buttonRight = buttonLeft + this.timeoutCategoryButton.boxWidth;
	var buttonBottom = buttonTop + this.timeoutCategoryButton.boxHeight;

	if (appUtils.mouseX >= buttonLeft && appUtils.mouseX <= buttonRight && appUtils.mouseY >= buttonTop && appUtils.mouseY <= buttonBottom)
	{
		this.gadget.onClickCategoryButton (this.timeoutCategoryButton, this.timeoutCategoryType);
	}

}


/**
 * Responds to a category button selection by opening the popup.
 *
 * @param  categoryButton the button that was selected.
 * @param  categoryType the type of category (Compose, Ready Messages).
 */
component.prototype.onClickCategoryButton = function (categoryButton, categoryType)
{
	var location = new Object();
	location.parentX = this.gadget.left;
	location.buttonX = categoryButton.frameX;
	location.buttonY = categoryButton.screenY;
	
	if (categoryType == "COMPOSE") {
		this.gadget.openCategoryIconsMenu(categoryButton.getCategoryID(), location);
	} else if (categoryType == "EXTRA"){
		this.gadget.openCategoryExtraIconsMenu(categoryButton.getCategoryID(), location);
	} else if (categoryType == "READY_MESSAGES") {
		this.gadget.openReadyMessagesMenu(categoryButton.getCategoryID(), location);
	}
}


component.prototype.reshowCategoryPopup =  function ()
{
	var eventObj = shell.currentEvent;
    if (eventObj)
    {
		// This prevents the mouseDown event from causing the freshly opened popup to immediately close!
		eventObj.preventDefault();
		eventObj.stopPropagation(); 
	}              

	var thePopup = this.gadget.getPartById("categoryPopupGadget");
	thePopup.displayCategoryPopup();
}

/**
 * When a key is pressed, passes it to the current category panel.
 *
 */
component.prototype.onKeyDown = function()
{
	var eventObj = shell.currentEvent;
    if (eventObj)
    {
		this.gadget.closeDemo();
		this.categoryPanel.onReceivedKey(eventObj);
    }
}

/**
 * Sets up and shows the popup tab.
 *
 * @param  cat the currently selected category.
 * @param  location the position of the category button.
 */
component.prototype.showPopupTab = function(cat, location)
{
	var popupTab = this.gadget.getPartById("zIpopupTab");
	popupTab.hidden = true;
	popupTab.setShape("#POPUP_TAB_DETACHED");
	popupTab.initializeImageStack (this.zlangoRootBox.getComposeCategoryList(), this.zlangoRootBox.getReadyMessagesCategoryList(), this.zlangoRootBox.getExtraCategoryList());
	popupTab.setUpPopupTab(cat, location);
	popupTab.left += 12;
	popupTab.hidden = false;
	popupTab.getPartById("popup_tab_contents").getBoxByTag("imageStack").setCurrentImage(cat.imagePath);
}

/**
 * Hides the popup tab.
 *
 */
component.prototype.hidePopupTab = function()
{
	var popupTab = this.gadget.getPartById("zIpopupTab");
	popupTab.hidden = true;
}

component.prototype.showShortcuts = function()
{
	var thePopup = this.gadget.getPartById("shortcutsPopup");

	thePopup.opened = true;
}

component.prototype.showZlangoWebPage = function()
{
	this.zlangoRootBox.getConnector().openUrl(shell.zlangoGlobals.webUrl, true);
}

component.prototype.showZlangoHotOrNotPage = function()
{
	this.zlangoRootBox.getConnector().openUrl(shell.zlangoGlobals.webHonUrl, true);
}

component.prototype.openDemo = function()
{
	if (this.isDemoOpen)
	{
		this.gadget.closeDemo();
		return;
	}
	var box1 = this.gadget.getPartById("demo").getPartById("flash1");
	if (box1)
	{
		this.gadget.getPartById("demo").hidden = false;
		box1.Movie = "http://c.icq.com/xtraz2/products/zlango/lang/" + this.zlangoRootBox.getLanguage() + "/zlango_demo.swf";
        this.scene.connectObject(box1.control, "flash1_");
		box1.Rewind();
		box1.Play();
		this.isDemoOpen = true;
	}
	
	this.zlangoRootBox.logDemo();
}

component.prototype.closeDemo = function()
{
	var box1 = this.gadget.getPartById("demo").getPartById("flash1");
	box1.Stop();
	if (this.isDemoOpen)
	{
		this.gadget.getPartById("demo").hidden = true;
    	this.scene.disconnectObject("flash1_");
		this.isDemoOpen = false;
	}
}

component.prototype.onClickDemo = function()
{
	this.gadget.closeDemo();
}


